# Dealing with frealix .ali files
class FrealixAli
	NAME = "Frealix alignment"

	require_relative "FrealixWaypoints.rb"

  def initialize(options={})
    defaults = { :filename => "waypoints.ali", :units => 'Angstr', :number_of_frames => 1}
    options = defaults.merge(options)
		@filename = options[:filename]
		@units    = options[:units]
		@number_of_frames = options[:number_of_frames]
		@file_last_read = Time.new(1900)	# We pretend the file was last read in 1900
		if  File.exist?(@filename)
			readFromDisk()
			getParVersion()
		else
			$LOG.warn() {"File does not exist: #{@filename}"}
		end
		if (@par_version == 4)
      @header = "!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units #{@units}   par_vers. 4\n"
		elsif (@par_version == 5)
      @header = "!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #        Twist         Rise    Score Refine? Units #{@units}    # frames    #{@number_of_frames}   par_vers. 5\n"
		elsif
			@header = "!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #        Twist         Rise    Score LogP   Sigma  Occ    Refine? Units #{@units}    # frames    #{@number_of_frames}   par_vers. 6\n"
		end
  end

	# Read file contents from disk
	def readFromDisk
		if (File.exists?(@filename))
			@file_last_read = Time.now
			@str = File.read(@filename)
			@lines = @str.split(/\n/)
		else
			# File doesn't exist
		end
	end

	def getParVersion
	  header_words = @lines[0].split
	  vers_index = header_words.index("par_vers.") + 1
	  @par_version = header_words[vers_index].to_i
	end

	# Write to disk
	def writeToDisk
	  File.write(@filename,@str)
	end

	# Return an array of waypoints from the contents of the file
	def getWaypoints
		wps = []
		@lines.each do |line|
			next if line.match(/^[!#]/)?true:false
			wps << FrealixWaypoint.new(:string=>line,:par_version=>@par_version)
			$LOG.debug("getWaypoints") {print wps[0]}
		end
		return wps
	end

	# Replace the string content with information from the array of waypoints
	def putWaypoints(wps)
	  @str = @header
	  wps.each{|wp|
	    @str << wp.toString
	  }
	end
end
