# Dealing with frealix runtime parameters
class FrealixParameters
	NAME = "Frealix parameters"

	# Initialiser method with default parameter
	def initialize(filename="frealix_parameters.flx")
		@filename = filename
		# The hash contains the actual runtime parameters in key/value pairs
		@rtp = Hash.new("none")
		if (filename)
		  self.readFromDisk() if File.exist?(@filename)
		end
	end

	# Deep cloning method
#	def dclone()
#	  klone = FrealixParameters.new()
#
#	end

	# Instance variable with automatic getter and setter methods
	attr_accessor :filename
	attr_accessor :rtp

	# Given an array of key strings, check that all those parameters are present
	def checkForParameters?(keys)
	  checkForParameters = true
	  keys.each { |key|
	    checkForParameters = (checkForParameters && (@rtp.has_key?(key) || @rtp.has_key?(key.upcase)))
	  }
	  return checkForParameters
	end

  # Given an array of key strings, check that all those parameters are present
  def findMissingParameters(keys)
    missing = []
    keys.each { |key|
      missing << [key] if ! @rtp.has_key?(key)
    }
    return missing
  end

	# Method to print out to terminal by default. If output is supplied, it's assumed to be a filename to write to.
	def print(output=$stdout)
	  # If we're not printing to output, we'll need to open a file
	  if (output!=$stdout)
	    output = File.open(output,'w')
	  end
		# Find out how long the longest key string is
		maxlength = 0
		@rtp.each do |key,value|
			maxlength = key.length if (key.length > maxlength)
		end
		maxlength += 1
		@rtp.each do |key,value|
			output << sprintf("%#{maxlength}s     %s \n", key, value)
		end
		output.close if (output != $stdout)
	end

	# Method to read from disk
	def readFromDisk(filename=@filename)
		# Check file exists
		if ! File.exist?(filename)
			puts "File does not exist: #{filename}"
			exit 0
		end

		# Parse the file
		File.open(filename,'r').each do |line|
			words = line.split
			# If this line has two words and isn't a comment
			if (words[0-1] && !words[0].match(/^[!#]/) && words[1] != nil)
				@rtp[words[0]] = words[1]
			end
		end
	end

	# Method to write to disk
	def writeToDisk(filename=@filename)
	  if (filename == nil)
	    puts "error: filename = nil"
	    puts caller(start = 0)
	    exit 1
	  end
		#
		self.print(output=filename)
	end
end
