# Dealing with directory where frealix refinement happens
class FrealixRefinementDirectory
  NAME = "Frealix refinement directory"

  require_relative "rubylix.rb"

  # Initialiser method with default parameter
  def initialize(options = {})
    # Default values
    defaults = {  :directory => ENV['PWD'],
                  :number_of_classes => 1 }
    #
    options = defaults.merge(options){  |key,defval,optval|  (optval.nil?) ? defval:optval }
    #
    @directory = options[:directory]
    @number_of_classes = options[:number_of_classes]
    # Check directory exists!
    if (!Dir.exists?(@directory))
      puts "Directory does not exist: #{@directory}"
    end
    @round_dir_root = "frealix_round_"
  end


  # The root name for round subdirectories
  attr_reader :round_dir_root
  attr_reader :directory

  # Create subdirectory for the next round and return its name
  def mkdirCurrentRound()
    dirname = roundSubDir(lastCompletedRound()+1)
    Dir.mkdir(dirname)
    return dirname+"/"
  end

  # Return the round number we're on
  def currentRound()
    self.lastCompletedRound() + 1
  end

  # Return the directory for the round we're currently processing
  def currentRoundDir()
    self.roundSubDir(currentRound())
  end

  def previousRoundDir()
    self.roundSubDir(currentRound()-1)
  end

  # Work out latest completed round
  def lastCompletedRound()
    lastCompletedRound = 0
    while (roundComplete?(lastCompletedRound)) do
      lastCompletedRound += 1
    end
    lastCompletedRound -= 1
    return lastCompletedRound
  end

  def roundComplete?(i)
    round_is_complete = true
    dir = roundSubDir(i)
    if ($enable_multi_ref)
      1.upto(@number_of_classes) do |class_counter|
        class_number = "%03i"%(class_counter)
        round_is_complete = round_is_complete && File.exists?(dir+"/3drec_r"+class_number+".mrc") && File.exists?(dir+"/waypoints_r"+class_number+".ali")
      end
      # Perhaps this round was completed, but not in multi-reference mode?
      if (! round_is_complete)
        round_is_complete = File.exists?(dir+"/3drec.mrc") && File.exists?(dir+"/waypoints.ali")
      end
    else
      round_is_complete = round_is_complete && File.exists?(dir+"/3drec.mrc") && File.exists?(dir+"/waypoints.ali")
    end
    return round_is_complete
  end

  # Check whether a completed round was in single-ref or multi-ref mode
  def multiRefRound?(i)
    return File.exists(roundSubDir(i)+"/waypoints_r001.ali")
  end

  # Check how many classes were refined in that round. Return 0 if single-reference round or if round not completed.
  def numberOfClassesInRound(i)
    dir = roundSubDir(i)
    counter = 0
    while (counter >= 0) do
      class_number = "%03i"%(counter+1)
      if (File.exists?(dir+"/waypoints_r"+class_number+".ali"))
        counter += 1
      else
        break
      end
    end
    return counter
  end

  # Return the subdir name for the round given
  def roundSubDir(round)
    sprintf("%s%03i",@round_dir_root,round)
  end

  # Return the round number given the subdirname
  def self.roundNumFromSubDir(subdir)
    pat = /#{@round_dir_root}_(\d\d\d)/
    subdir.match(pat)[1].to_i
  end

end
