# Interact with a Linux host
class LinuxHost
	NAME = "Linux Host"

	def initialize(options = {})
		defaults = {	:hostname => "localhost"
					}
		options = defaults.merge(options)
		@hostname = options[:hostname]
		# Check it is reachable
		self.is_reachable?
		# Set up default values for processor stats
		@cpu_usage = 100.0
		@cores_available = 0
		@num_cores = 1
		#
		@mem_free_gb = 1.0
		#
		@cpu_stats_update_interval = 120.0 #in seconds
		@cpu_stats_update_last = nil
		# Update CPU stats
		self.update_cpu_stats()
	end

	attr_reader :hostname
	attr_reader :mem_free_gb

	# Return current CPU usage
	def currentCpuUsage
	  self.update_cpu_stats()
	  @cpu_usage
	end

	# Return the number of cores available
	def numCoresAvailable()
	  self.update_cpu_stats()
	  @cores_available
	end

	# Check whether the host is remote
	def is_remote?
		!is_local?
	end

	# Check whether we're on localhost
	def is_local?
		lh = local_hostname
		(@hostname == "localhost" || @hostname == lh || @hostname == lh.split('.')[0])
	end

	# Check whether a host is reachable
	def is_reachable?
		if (is_remote?)
		  command = "ping -q -c 1 #{@hostname}"
		  p = Process.spawn(command,[:err,:out]=>"/dev/null")
		  pid, status = Process.wait2(pid=p)
		  ok = status.success?
		  $LOG.error("#{@hostname} is not reachable by ping.") if (!ok)
		  return ok
		else
		  return true
		end
	end

	# Find out the local hostname
	def local_hostname
		c = IO.popen("hostname","r")
		h = c.read
		c.close
		h.strip
	end

	# Update stats about processor usage
	# (only do so if it's been a while since the last update)
	def update_cpu_stats
	  if (!@cpu_stats_update_last || (Time.now()-@cpu_stats_update_last) > @cpu_stats_update_interval)
  		# This command will return the number of cores in the first line,
  		# then a header line (to be ignored), and then a list of the %CPU
  		# used by all processes. We need a sum of the %CPU, divided by
  		# the number of cores, to get a figure between 0 and 100
  		command = "cat /proc/cpuinfo | grep processor | wc -l; ps -e -o pcpu --sort -pcpu"
      #command = "cat /proc/cpuinfo | grep siblings | awk '{print $3}'; ps -e -o pcpu --sort -pcpu"
  		#command_mem = "grep MemAvailable /proc/meminfo | awk '{print $2}'"
  		command_mem = "free -g | grep 'buffers/cache'"
  		command_mem= "cat /proc/meminfo | awk '/MemAvailable/ {print $2}'"
  		if (self.is_remote?)
  			command = "/usr/bin/ssh -xqn #{@hostname} \"#{command}\""
        command_mem = "/usr/bin/ssh -xqn #{@hostname} \"#{command_mem}\""
  		end
  		f = IO.popen(command,"r")
  		s = f.read
  		f.close
  		l = s.split("\n")
  		@num_cores = l[0].to_f
  		sum = 0.0
  		l[2...-1].each do |val|
  			sum += val.to_f
  		end
  		@cpu_usage = sum / @num_cores
  		self.update_cores_available()
  		@num_cores = @num_cores.to_i
  		# Now we grab the memory info
  		f = IO.popen(command_mem,"r")
      s = f.read
      f.close
      l = s.split()
  		#@mem_free_gb = l[3].to_f
  		@mem_free_gb = l[0].to_f / 1024 / 1024
      # Remember that we just updated this info
  		@cpu_stats_update_last = Time.now()
  		$LOG.debug("update_cpu_stats") {"#{@hostname} has #{@num_cores} cores, cpu usage #{@cpu_usage}% and #{@mem_free_gb} GB of free memory"}
    end # Check for interval
	end # update_cpu_stats

	# Update the number of cores available
	def update_cores_available
	  # Start off with all cores
	  @cores_available = @num_cores.to_i
	  # Number of 100%-busy cores
	  tmp = @cpu_usage / (100.0 / @num_cores.to_f)
	  $LOG.debug("update_cores_available") {"Number of 100% busy cores = #{tmp}"}
	  @cores_available -= tmp.to_i
	  # The remainder
	  tmp = @cpu_usage % (100.0 / @num_cores.to_f)
	  $LOG.debug("update_cores_available") {"Remainder = #{tmp}"}
	  # We will count a core as available if it is less than 40% used
	  @cores_available -= 1 if (tmp > 40.0)
	end



end
