#!/usr/bin/env ruby
#  final_wps_to_ali.rb
#
#  Grab the final WP information from a log file and write out an ali file
#
#  Copyright 2014 Howard Hughes Medical Institute.
#  All rights reserved.
#  Use is subject to Janelia Farm Research Campus Software Copyright 1.1
#  license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
#  
#
#
require_relative "rubylix.rb"
require_relative "FrealixOutput.rb"
require_relative "FrealixAli.rb"
require_relative "FrealixFilament.rb"

require 'optparse'

$LOG.level = Logger::INFO

options = {}
defaults = {}

optparse = OptionParser.new do |opts|
  opts.banner = "Usage: final_wps_to_ali.rb frealix_output.log waypoints.ali"

  # Help message
  opts.on_tail("-h","--help","Show this message") do
    puts opts
    exit
  end
  
#  # Extra hosts
#  opts.on("-H","--hostsfile [hosts.txt]","Flat text list of hosts available for processing. localhost and cluster are recognised.") do |h|
#    options[:hostsfile] = h
#  end

#  options[:max_num_wps] = 1000
#  opts.on("-m","--max_num_wps NUM",Integer, "Filaments with more than NUM waypoints will be split up") do |b|
#    options[:max_num_wps] = b
#  end
#
#  options[:ignore_fal_wps] = nil
#  opts.on("-i","--ignore_fal_wps", "Processing ignores the first and last waypoints of each waypoint") do |b|
#    options[:ignore_fal_wps] = b
#  end

end

optparse.parse!

#
if (ARGV.count != 2)
  puts "Usage: final_wps_to_ali.rb frealix_output.log waypoints.ali"
  exit 1
end

# User-defined parameters
@input_log_filename = ARGV[0]
@output_ali_filename = ARGV[1]

log = FrealixOutput.new(@input_log_filename)
filnums = log.getFilamentNumbers
wps = Array.new(0,FrealixWaypoint)
filnums.each{|filnum|
  wps += log.getFinalWaypoints(filnum)
  
}

# Create the output waypoints file
ali = FrealixAli.new(@output_ali_filename)
ali.putWaypoints(wps)
ali.writeToDisk

$LOG.info("Wrote to file: #{@output_ali_filename}")
