#!/bin/bash
#
# Generate a template parameter file to be used with Frealix's wrapper script
#
# Alexis Rohou
# November 2015
#

output_fn="parameters.flx"

cat > $output_fn <<eof
##
## Parameters for frealix refinement.
##
## Optional parameters are commented out and will be ignored unless you
## uncomment them.
##
##

film_files                              /path/to/micrograph_????.mrc            # Filename pattern for micrographs. Question marks denote digits.
                                                                                # If no question marks are present, it is assumed all micrographs are concatenated into a single image file.

#reverse_film_contrast                   f                                       # t/f. It is assumed that protein density is darker (lower values) than the background. If this isn't the case, set this flag to true (t).

pixel_size                              1.23                                    # Angstroms
acceleration_voltage                    300.0                                   # kV
spherical_aberration                    2.7                                     # mm
amplitude_contrast                      0.07                                    # fraction


##
## CTF information. Supplied either ctffind (3 or 4) or ctftilt results.
## Give "none" for the other. If both are given and this is a new project,
## ctftilt information will be used.
##
ctffind_files                          /path/to/ctffind_output_????.txt         # Question marks denote digits. Index number of ctf results must match numbering of micrographs.
#ctftilt_files                          none


##
## Helical parameters
##
twist_per_subunit                      -22.03                                   # Degrees. Right-handed helices have negative twist/subunit.
rise_per_subunit                       1.408                                    # Angstroms
helix_max_radius                       90.0                                     # Angstroms
asym_unit_mw                            10000.0                                 # Daltons. Expected moelcular weight of the asymetric unit
#stack                                  f                                        # The assembly is a stack rather than a true helix
#symmetry_axial                         2                                        # Rotational symmetry about the helical axis
#symmetry_perp                          2                                        # Rotational symmetry perpendicular to the helical axis (e.g. set this to 2 for DNA)
#num_helical_starts                     1                                        # If the helical parameters above do not describe a 1-start helix, indicate here the nubmer of starts. Note that this is independent of any cyclical symmetries.


##
## Options for crossovers
##
#wps_are_crossovers                     f                                        # t/f. Set to true if the input coordinates are crossovers (i.e. with phi a multiple of 180.0).
#fal_wps_are_crossovers                 f                                        # t/f. Set to true if the first and last waypoints of each filament are also crossovers.


##
## If you do not already have a waypoints.ali file for frealix, you need to supply
## one of the parameters below.
## If you have not yet picked filaments from your micrographs, the simplest is to
## the display program from the Tigris suite (http://sourceforge.net/projects/tigris/)
## in Waypoints selection mode. This handles waypoints.ali files natively and displays
## filament trajectories onto mrc or Imagic image files.
##
#input_box_files                       /path/to/picks_????_????.box              # First set of question marks is for the micrograph identifier. Second set of question marks is for the filament identifier. This second identifier must start at 1 and increment without breaks.
#input_imod_points_filename            /path/to/picks.txt                        # Output of IMOD's model2point command (with -contour option).
#film_selection                        1-120,122,125-128                         # Specify this if you gave film_files with question marks (i.e. a filename pattern rather than the filename of a stack of micrographs). Which films we''re going to analyse - the string here would take all films from 1 to 120, film 122 and films 125 to 128.


##
## Movie parameters
## (Only use these parameters if "film_files" points to a stack of frames)
##
#number_of_frames_per_micrograph        10                                       # Number of frames in each dose-fractionated movie micrograph.
#exposure_per_frame                     1.0                                      # electrons per Angstrom squared. For each frame of dose-fractionated exposures, the number of electrons incident on the specimen.
#unblur_result_filename                 /path/to/unblur_result.txt               # Filename of results from unblur. Should contain one line per micrograph, with relative shifts of frames.
#recompute_subframe_alignemnts          f                                        # t/f. For each waypoints, recompute local alignments parameters between movie frames, starting from unblur results.
#box_size_for_subframe_alignment        1000.0                                   # Angstroms. Size of the box toextract from movie frames around each waypoints when computing local frame alignments.



##
## Refinement options
##
#skip_refinement                         f                                       # t/f. If true, no refinement will be performed, and a 3D reconstruction will be computed from input parameters.
#polarity_check                          f                                       # t/f. Check the polarity of each filaments by attempting refinement assuming both polarities and keeping the better-scoring polarity.
#fal_wps_in_score                        t                                       # t/f. The parts of the filament between the first and second as well as between the penultimate and last waypoints should be included in the scoring function.
#fal_wps_in_threed                       t                                       # t/f. The parts of the filament between the first and second as well as between the penultimate and last waypoints should be included in the 3d reconstruction.
#boost_low_frequency_contrast            f                                       # t/f. Leave low frequencies (until the first peak of the ctf) intact, rather than scaling them by the ctf value. May help in early stages of alignments.

##
## Pre-refinement filament modifications
## (these modifications are applied to filaments on input, before refinement begins)
##
#force_wp_to_wp_distance                 200.0                                   # Angstroms. Resample waypoints along the path of each filament, at the given interval.
#force_wp_to_wp_phi                      90.0                                    # Degrees. Resample waypoints along the path of each filament, at the given interval.
#add_fal_wps                             f                                       # t/f. Add a waypoint at each end of the filament, by extrapolation.


##
## Which parameters do you want to refine?
##
#refine_x                                t                                       # t/f
#refine_y                                t                                       # t/f
#refine_phi                              t                                       # t/f
#refine_theta                            t                                       # t/f
#refine_psi                              t                                       # t/f
#refine_subunit_num                      t                                       # t/f

##
## Refinement strategy
## (not all options are compatible with single-particle refinement)
##
#use_conjugate_gradient                  t                                       # t/f. Use Powell's conjugate gradient minimiser. Not availble for single-particle refinement. Default for full-filament refinement.
#use_downhill_simplex                    f                                       # t/f. Use downhill simplex minimisation. Default for single-particle refinement.
#use_brute_force                         f                                       # t/f. Use exhaustive search.
#search_filament_tilt                    f                                       # t/f. Do a systematic search for the filament tilt. Each filament is tilted as a rigid body. This overrides other refinement modes.

##
## Search/refinement parameters
## (these parameters affect different minimisers differently. For example,
## downhill simplex and conjugate gradient minimisers use these numbers as guides, not hard limits)
##
#search_half_range_x                     10.0                                    # Angstroms
#search_half_range_y                     10.0                                    # Angstroms
#search_half_range_phi                   5.0                                     # Degrees
#search_half_range_theta                 5.0                                     # Degrees
#search_half_range_psi                   5.0                                     # Degrees
#search_half_range_subunit_num           1.0                                     # Number of helical asymmetric units
#search_step_size_x                      1.0                                     # Angstroms
#search_step_size_y                      1.0                                     # Angstroms
#search_step_size_phi                    2.0                                     # Degrees
#search_step_size_theta                  2.0                                     # Degrees
#search_step_size_psi                    2.0                                     # Degrees
#search_step_size_subunit_num            0.1                                     # Number of helical asymmetric units




##
## Restraints
##
#rigid_phi                               f                                       # t/f. Phi is refined per-filament rather than per-waypoint.
#rigid_theta                             f                                       # t/f. Theta is refined per-filament rather than per-waypoint.
#rigid_psi                               f                                       # t/f. Psi is refined per-filament rather than per-waypoint.
#rigid_subunit_num                       f                                       # t/f. Subununit number (~axial phasing) is refined per-filament rather than per-waypoint. This implies constant rise per subunit.
#fragmented_phi                          f                                       # t/f. Phi function is discontinuous, with breaks between waypoints.
#fragmented_subunit_num                  f                                       # t/f. Subunit_num function is discontinuous, with breaks between waypoints.
#constant_rise_per_subunit               f                                       # t/f. The rise per subunit is constant.
#constant_twist                          f                                       # t/f. The twist per unit length is constant.
#use_curvature_restraint                 f                                       # t/f. Restrain the total curvature of each filament according to the known persistence length and the estimated noise level in micrographs.
#use_torsion_restraint                   f                                       # t/f. Restrain the total over- or under-twist of each filament according to the known persistence length and the estimated noise level in micrographs.
#use_stretch_restraint                   f                                       # t/f. Restrain the total stretch or compression of each filament according to the known persistence length and the estimated noise level in micrographs.
#persistence_length                      10000.0                                 # Angstroms
#torsion_persistence_length              10000.0                                 # Angstroms
#stretch_persistence_length              10000.0                                 # Angstroms


##
## Minimiser options
##
#num_minim_runs                          1                                       # Number of times to run the minimizer
#ds_random_start_conf                    f                                       # Randomize the stating conformation of the downhill simplex
#minim_random_start_pos                  f                                       # Randomize the starting position of the minimizer. The position is randomized with Gaussian noise, standard devitation = search half-range.
#refine_wps_simultaneously               t                                       # All waypoints will be minimizes simultaneously. This dramatically increases the dimensionality of the optimization problem.


##
## Single-particle refinement (no restraints other than polarity)
##
#wps_as_sps                              f                                       # Treat each waypoint (wp) as a single particle (sp), i.e. completely independently of other waypoints.
#sp_segment_length                       300.0                                   # Angstroms. The length of each single-particle segment to box out at each waypoint.
#sp_to_ff                                t                                       # t/f. When doing single-particle refinement, attempt to prepare waypoint parameters for transition to full-filament (ff) refinement.
#refine_helical_twist                    f                                       # t/f. Refine helical twist per subunit.
#refine_helical_rise                     f                                       # t/f. Refine helical rise per subunit.
#search_half_range_helical_rise          0.2                                     # Angstroms
#search_half_range_helical_twist         0.1                                     # Degrees
#search_step_size_helical_rise           0.1                                     # Angstroms
#search_step_size_helical_twist          0.05                                    # Degrees

##
## Reconstruction options
##
#min_twist_per_subunit_for_rec           -179.99                                 # Degrees. Only regions where the local twist per subunit is greater than this value will be included in the 3D reconstruction.
#max_twist_per_subunit_for_rec           179.99                                  # Degrees
#min_rise_per_subunit_for_rec            0.00                                    # Angstroms
#max_rise_per_subunit_for_rec            999.999                                 # Angstroms

##
## Diagnostics
##
#single_filament_reconstructions         f                                       # For each filament, compute a 3D reconstruction, and write central (xy) and (yz) slices of this reconstruction to disk.

eof



echo " "
echo "A template parameter file was written to: ${output_fn}"
echo "1. Edit it following instructions therein "
echo "2. Use the Frealix wrapper to get started with refinement by typing flx_wrap.rb --help"
echo " "
