#!/usr/bin/env ruby
#  merge_waypoints.rb
#
#  Given a number of waypoints file describing alternative
#  alignments of the same filaments, output a single waypoints file
#  in which only the best-scoring alignment is kept.
#
#  Copyright 2014 Howard Hughes Medical Institute.
#  All rights reserved.
#  Use is subject to Janelia Farm Research Campus Software Copyright 1.1
#  license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
#  
#
#

require_relative "rubylix.rb"
require_relative "FrealixFilament.rb"
require_relative "FrealixAli.rb"

require 'optparse'

options = {}
defaults = {}

optparse = OptionParser.new do |opts|
  opts.banner = "Usage: merge_waypoints.rb [options] input_filenames output_filename"

  # Help message
  opts.on_tail("-h","--help","Show this message") do
    puts opts
    exit
  end

end

optparse.parse!

@input_waypoints_filenames = ARGV[0...-1]
@output_waypoints_filename = ARGV[-1]

$LOG.info() {"Input filenames = #{@input_waypoints_filenames}"}
$LOG.info() {"Number of input files = #{@input_waypoints_filenames.size}"}
$LOG.info() {"Output filename = #{@output_waypoints_filename}"}
  

#
# Get filaments from the waypoints files and compare to best scoring-filaments
#
fils = []
best_fils = []
@input_waypoints_filenames.each_with_index { |wp_fn,i|
	ali_in = FrealixAli.new(filename=wp_fn)
	fils = FrealixFilament.filamentsFromWaypoints(ali_in.getWaypoints)
	if (i==1) then
		best_fils = fils	
	else
		best_fils.each_with_index {|fil,j| fil = fils[j] if fils[j].score > fil.score}
	end
}

#
# Output best filaments
#
FrealixFilament::filamentsToAliFile(best_fils,@output_waypoints_filename)

$LOG.info() {"#{best_fils.size} filaments are included in the output ali file"}