#!/bin/bash
#
#

#
# Parse the command line
#
max_resolution=0.0
while getopts ":r:" opt; do
	case $opt in
		r)
			echo "Max resol in refinement = $OPTARG A" >&2
			max_resolution=$(echo "1.0 / $OPTARG" | bc -l)
			;;
		\?)
			echo "Invalid option: -$OPTARG" >&2
			;;
	esac
done
shift $((OPTIND-1))


fsc_data_fn=( "$@" )

plot_fn=${fsc_data_fn[${#fsc_data_fn[@]}-1]%%.???}.pdf
titles=${fsc_data_fn[@]}

i=0
for title in ${titles[@]}; do
	foo=${title%/*}
	titles[$i]=${foo}
	i=$(($i+1))
done
echo ${titles[@]%%.dat}


gnuplot <<EOF 
set border linewidth 1.5
set terminal pdfcairo size 5.0cm,4.0cm enhanced font 'Arial,9'
#set output word('${plot_fn[@]}',words('${plot_fn[@]}'))
set output '$plot_fn'

# color definitions
set style line 1 lc rgb '#0060ad' lt 1 lw 2 pt 7 # --- blue
set style line 2 lc rgb 'red'     lt 1 lw 2 pt 7 # --- red
set style line 3 lc rgb 'orange'  lt 1 lw 2 pt 7 # --- orange
set style line 4 lc rgb 'light-blue' lt 1 lw 2 pt 7 # --- light blue
set style line 5 lc rgb 'green'   lt 1 lw 2 pt 7
set style line 6 lc rgb 'gray'    lt 1 lw 2 pt 7

#unset key
set xlabel 'Spatial frequency (1/)'
set ylabel 'FSC'
#set format y '%3.2f'
set tics scale 0.75
#set xrange [0.0:0.4]
set yrange [0.0:1.0]
set ytics 0.0,0.25,1.0
set ytics add ("0.143" 0.143)
set xtics 0.0,0.1,0.4
set mxtics 2
set key samplen 1 noenhanced font 'Arial,6'
set arrow from ${max_resolution},0 to ${max_resolution},1.0 nohead

#plot '$fsc_data_fn'     using (\$1):(\$2)     w lines ls 1

plot for [i=1:words('${fsc_data_fn[@]}')] word('${fsc_data_fn[@]}',i) using (\$1):(\$2) w lines ls i title word('${titles[@]}',i)

EOF

# Now fix the title
echo "InfoKey: Title" > /tmp/pdftemp.txt
echo "InfoValue: $1" >> /tmp/pdftemp.txt
mv ${plot_fn} /tmp/tmp.pdf
if hash pdftk 2>/dev/null; then
	pdftk /tmp/tmp.pdf update_info /tmp/pdftemp.txt output $plot_fn
else
	cp /tmp/tmp.pdf $plot_fn
fi

# Now make a PNG version
convert -density 300 -flatten ${plot_fn} ${plot_fn%%.pdf}.png
