# Check we are using Ruby 1.9 or above
# (we need this because 1.9 provides ordered hash)
if (RUBY_VERSION.to_f < 1.9)
	puts "Sorry, ruby version 1.9 or above is needed"
	exit 0
end

#
# Set up the logger
#
require 'logger'
$LOG = Logger.new(STDOUT)
$LOG.level = Logger::INFO
$LOG.datetime_format = "%Y-%m-%d %H:%M:%S "
$LOG.debug("Created logger")

# A stats library
#require 'statsample'


# Useful parsing functions
class String
  def is_i?
    !!(self =~ /^[-+]?[0-9]+$/)
  end
end
class String
  def is_f?
    !!(self =~ /^[+-]?[0-9]+[.]?[0-9]*$/)
  end
end

# The equivalent to which
def which(program)
  ENV['PATH'].split(':').collect {|d| Dir.entries d if Dir.exists? d}.flatten.include? program
end

# Determine whether frealix is available
def frealixAvailable?
  which("frealix")
end

class Float
	def m180to180
		tmp = self.modulo(360.0)
		if (tmp > 180.0)
			tmp = tmp - 360.0
		end
		if (tmp < -180.0)
			tmp = tmp + 360.0
		end
		return tmp
	end
end

# Define the deep cloning method
class Object
  def dclone
    case self
      when Fixnum,Bignum,Float,NilClass,FalseClass,
           TrueClass  #,Continuation
        klone = self
      when Hash
        klone = self.clone
        self.each{|k,v| klone[k] = v.dclone}
      when Array
        klone = self.clone
        klone.clear
        self.each{|v| klone << v.dclone}
      else
        klone = self.clone
    end
    klone.instance_variables.each {|v|
      klone.instance_variable_set(v,
        klone.instance_variable_get(v).dclone)
    }
    klone
  end
end

class Array
  # returns (mean, standard_dev)
  # if size == 0 returns [nil, nil]
  def sample_stats
    _len = self.size
    return [nil, nil] if _len == 0
    _sum = 0.0 ; _sum_sq = 0.0
    self.each do |val|
      _sum += val
      _sum_sq += val * val
    end
    std_dev = _sum_sq - ((_sum * _sum)/_len)
    std_dev /= ( _len > 1 ? _len-1 : 1 )
	std_dev = 0.0 if std_dev < 0.0
    [_sum.to_f/_len, Math.sqrt(std_dev)]
  end

  def round(n)
	self.each_with_index { |elem,i|
		self[i] = self[i].round(n)
	}
  end

  def abs
    _len = self.size
    return nil if _len == 0
    self.each_index {|i|
      self[i] = self[i].abs
    }
  end

  def median()
    sorted = self.sort
    len = sorted.length
    return (sorted[len/2])
  end

  def all_same?
    return self.count(self[0]) == self.size
  end

  def all_zeroes?
    self.each_index {|i|
      return false if self[i] != 0.0
    }
    return true
  end

  def any_nan?
    self.each_index {|i|
      return true if self[i].nan?
    }
    return false
  end

  def all_zeroes_or_180s?
    self.each_index {|i|
      return false if self[i] != 0.0 && self[i].abs != 180.0
    }
    return true
  end
end



# Return number of seconds from a string such as
# "3 days, 2 hours, 1 minute and 32 seconds"
#
# Code obtained from http://www.ruby-forum.com/topic/166915
# author: Ara Howard (http://codeforpeople.com/)
#
# Under the Ruby license, which is GPL-compatible.
#
# Example usage:
# p duration_for('1 minute')
# p duration_for('1 hour and 1 minute')
# p duration_for('6 days, 3 hours and 1 minute')
#
def duration_for spec
   ret = nil
   if((m = %r/^ (\d+(?:\.\d+)?) : (\d+(?:\.\d+)?) : (\d+(?:\.\d+)?) $/iox.match(spec.to_s)))
     m, h, m, s, ignored = m.to_a
     h, m, s = Float(h), Float(m), Float(s)
     ret = (h * 60 * 60) + (m * 60) + (s)
   else
     pat = %r/(\d+(?:\.\d+)?)\s*([sSmMhHdDwWyY][^\d]*)?/
     begin
       "#{ spec }".scan(pat) do |m|
         n = Float m[0]
         unit = m[1]
         if unit
           factor =
             case unit
               when %r/^m/i
                 case unit
                   when %r/^mo/i
                     7 * (60 * 60 * 24)
                   else
                     60
                 end
               when %r/^h/i
                 60 * 60
               when %r/^d/i
                 60 * 60 * 24
               when %r/^w/i
                 7 * (60 * 60 * 24)
               when %r/^y/i
                 365 * 7 * (60 * 60 * 24)
               else
                 1
             end
           n *= factor
         end
         ret ||= 0.0
         ret += n
       end
     rescue
       raise "bad time spec <#{ spec }>"
     end
   end
   ret
end

$developer_mode = false
$enable_multi_ref = false
$default_ali_format = 5
