!>  \brief  Functions to generate figures & plots from filament objects
module filaments_plots


contains

    !>  \brief  Generate a simple figure of the path of a filament
    subroutine filament_threed_plot(self,filename)
        use filaments
        use filaments_path, only : filament_get_path_subunit_steps
        use plots
        implicit none
        ! Arguments
        type(filament),     intent(inout)  ::  self        !<  Filament object
        character(len=*),   intent(in)  ::  filename    !<  Filename to write to
        ! Private variables
        logical,    parameter    ::  debug   =   .true.
        type(waypoint), allocatable ::  path(:)
        type(plot)  ::fig
        real,   allocatable ::  x(:), y(:), z(:)
        ! Start work

        ! Obtain path of filament (every subunit)
        call filament_get_path_subunit_steps(self,path,STEP=1)
        ! Set up plot
        call plot_init(fig,DIMENSIONALITY=3,FILENAME=FILENAME,   &
                        ALTITUDE=20.0,AZIMUTH=45.0)
        ! Copy coordinates over to explicit arrays
        x = path%coo(1)
        y = path%coo(2)
        z = path%coo(3)
        ! Add series to the plot
        call plot_add_xyz(fig,x,y,z,DRAW_SHADOW=.true.)
        ! Draw the plot
        call plot_draw(fig)
    end subroutine filament_threed_plot

end module filaments_plots
