!>  \brief  Safely compare float
!!
!!  See for example discussion here: http://computer-programming-forum.com/49-fortran/6bd3f309b12a9632.htm
!!
module float_comparisons
    implicit none

    integer, parameter  ::  nulp = 1

    contains

    !>  \brief  Check whether two floats are equal within numerical precision
    !!
    !!  Apparently, this is an f95 implementation of the scheme described in:
    !!  Knuth, Art of Computer Programming, vol 2, 3rd ed (1998), p. 233
    !!  but i haven't checked this myself
    !!  Also of interest may be http://fcmp.sourceforge.net/, which i also haven't checked out
    pure logical function doubles_are_equal(x,y)
        real(kind=8),   intent(in)    ::  x,y
        doubles_are_equal = abs(x-y) < nulp*2*spacing(max(abs(x),abs(y)))
    end function doubles_are_equal

    !>  \brief  Check whether two floats are equal within numerical precision
    !!
    !!  This alternative doesn't require the setting of a nulp parameter, and is due to
    !!  Paul van Dels (see http://computer-programming-forum.com/49-fortran/6bd3f309b12a9632.htm)
    pure logical function f_equal_alt(x,y)
        real(kind=8),   intent(in)    ::  x,y
        ! private variables
        real(kind=8)                ::  epsilon_exponent, epsilon_delta
        real(kind=8),   parameter   ::  one = 1.0d0
        real(kind=8),   parameter   ::  two = 2.0d0
        ! start work
        epsilon_exponent = exponent( max( abs(x), abs(y) ) )
        epsilon_delta    = max( epsilon(one) * two**epsilon_exponent, epsilon(one) )
        f_equal_alt = abs(x-y) < epsilon_delta
    end function f_equal_alt


!
! The routines below should be deprecated
!

    !>    \brief    Test whether two reals are significantly (i.e. above precision) different
    logical function reals_are_different(a,b)
        implicit none
        ! Arguments
        real(kind=4),    intent(in)    ::    a,b
        ! Private variables

        ! Start work
        reals_are_different = abs(A-B) .gt. spacing(a)
    end function reals_are_different

    !>    \brief    Test whether two reals are significantly (i.e. above precision) different
    logical function dbles_are_different(a,b)
        implicit none
        ! Arguments
        real(kind=8),    intent(in)    ::    a,b
        ! Private variables

        ! Start work
        dbles_are_different = abs(A-B) .gt. spacing(a)
    end function dbles_are_different

end module float_comparisons
