program frealix

    use runtime_parameters
    use DatesAndTimes
    use Globals

    implicit none

    type(DateAndTime)   ::  myTime
    ! Current instance of the program

    ! Start work
    if (this_program%IsVerbose()) then
        write(*,'(a)') '<?xml version="1.0"?>'
        write(*,'(a)') '<frealix>'
    endif
    date_start = myTime%DateAndTimeAsIntegers()

    ! Register signal handlers
    call register_signal_handlers()

    !
    call this_program%Init('Frealix','0.0.0','2014')


    ! Call the main subroutine
    call main()
    date_finish = myTime%DateAndTimeAsIntegers()

    write(*,'(/4a/)') myTime%DateAndTimeAsString(), ': frealix says goodbye (total execution time: ', &
                      myTime%DurationAsString(real(myTime%SecondsBetweenDates(date_start,date_finish))), ')'
    if (this_program%IsVerbose()) then
        write(*,'(a)') '</frealix>'
    endif

    contains

    ! The actual subroutine
    subroutine main()

        ! Modules
        use runtime_parameters, only : print_welcome
        use parsers                 ! This gives us access to a bunch of subroutines to parse files, command-line options etc.
        use ui                      ! This gives us access to user-interaction subroutines

        ! Mode modules
        use mode_finalise_3ds
        use mode_calc_fsc
        !use mode_calc_frsc
        use mode_stack_from_par
        use mode_test
        !use mode_apply_filter
        use mode_apply_ctf
        use mode_frealign_stack_to_threed
        use mode_frealign_stack_to_frealix_ali
        use mode_filament_startup
        use mode_filament_refine
        use mode_hextend
        use mode_hmeasure
        use mode_align
        use mode_stack_align_to_sum
        use mode_hmask
        !use mode_find_filaments
        use mode_split_par
        use mode_rebuild_seam
        use mode_synth_micrograph
        use mode_rotate
        use mode_pad
        use mode_mask
        use mode_bfactor
        use mode_unit_tests
        use mode_randomize_waypoints
        use mode_frealix_to_frealign
        use mode_bin
        use mode_estimate_persistence_length
        use mode_calc_occ
        use mode_create_random_classes

        implicit none

        ! Check the command-line - get the control file name
        call parse_command_line_arguments

        ! Get global run-time parameters from the control file
        call get_global_rtps()

        ! Welcome message - make this a subroutine in runtime_parameters.f90
        call print_welcome()

        ! Check which mode the user wanted us to run
        call get_frealix_mode

        ! Call the subroutine corresponding to the mode the user asked for
        select case(upper_case(frealix_mode))
            case ('FINALISE_3DS')
                write(*,'(a)') 'Frealix will run its finalise_3ds mode'
                call mode_finalise_3ds_init_and_run()
            case('CALC_FSC')
                write(*,'(a)') 'Frealix will run its calc_fsc mode'
                call mode_calc_fsc_init_and_run
            !case('CALC_FRSC')
            !    write(*,'(a)') 'Frealix will run its calc_frsc mode'
            !    call mode_calc_frsc_init_and_run
            case('STACK_FROM_PAR')
                write(*,'(a)') 'Frealix will run its stack_from_par mode'
                call mode_stack_from_par_init_and_run
            case('TEST')
                write(*,'(a)') 'Frealix will run its test mode'
                call mode_test_init_and_run
            !case('APPLY_FILTER')
            !    write(*,'(a)')  'Frealix will run its apply_filter mode'
            !    call mode_apply_filter_init_and_run
            case('APPLY_CTF')
                write(*,'(a)')  'Frealix will run its apply_ctf mode'
                call mode_apply_ctf_init_and_run
            case('FREALIGN_STACK_TO_THREED')
                write(*,'(a)')  'Frealix will run its frealign_stack_to_threed mode'
                call mode_frealign_stack_to_threed_init_and_run
            case('FREALIGN_STACK_TO_FREALIX_ALI')
                write(*,'(a)')  'Frealix will run its frealign_stack_to_frealix_ali mode'
                call mode_frealign_stack_to_frealix_ali_init_and_run
            case('FILAMENT_STARTUP')
                write(*,'(a)')  'Frealix will run its filament_startup mode'
                call mode_filament_startup_init_and_run
            case('FILAMENT_REFINE')
                write(*,'(a)')  'Frealix will run its filament_refine mode'
                call mode_filament_refine_init_and_run
            case('HEXTEND')
                write(*,'(a)')  'Frealix will run its hextend mode'
                call mode_hextend_init_and_run
            case('HMEASURE')
                write(*,'(a)')  'Frealix will run its hmeasure mode'
                call mode_hmeasure_init_and_run
            case('STACK_ALIGN_TO_SUM')
                write(*,'(a)') 'Frealix will run its stack_align_to_sum mode'
                call mode_stack_align_to_sum_init_and_run
            case ('HMASK')
                write(*,'(a)') 'Frealix will run its hmask mode'
                call mode_hmask_init_and_run
            !case ('FIND_FILAMENTS')
            !    write(*,'(a)') 'Frealix will run its find_filaments mode'
            !    call mode_find_filaments_init_and_run
            case ('SPLIT_PAR')
                write(*,'(a)') 'Frealix will run its split_par mode'
                call mode_split_par_init_and_run
            case ('REBUILD_SEAM')
                write(*,'(a)') 'Frealix will run its rebuild_seam mode'
                call mode_rebuild_seam_init_and_run
            case ('SYNTH_MICROGRAPH')
                write(*,'(a)') 'Frealix will run its synth_micrograph mode'
                call mode_synth_micrograph_init_and_run
            !case ('ROTATE')
            !    write(*,'(a)') 'Frealix will run its rotate mode'
            !    call mode_rotate_init_and_run
            case ('PAD')
                write(*,'(a)') 'Frealix will run its pad mode'
                call mode_pad_init_and_run
            case ('MASK')
                write(*,'(a)') 'Frealix will run its mask mode'
                call mode_mask_init_and_run
            case ('BIN')
                write(*,'(a)') 'Frealix will run its bin mode'
                call mode_bin_init_and_run
            case ('BFACTOR')
                write(*,'(a)') 'Frealix will run its bfactor mode'
                call mode_bfactor_init_and_run
            case ('UNIT_TESTS')
                write(*,'(a)') 'Frealix will run its unit_tests mode'
                call mode_unit_tests_init_and_run
            case ('RANDOMIZE_WAYPOINTS')
                write(*,'(a)') 'Frealix will run its randomize_waypoints mode'
                call mode_randomize_waypoints_init_and_run
            case ('FREALIX_TO_FREALIGN')
                write(*,'(a)') 'Frealix will run its frealix_to_frealign mode'
                call mode_frealix_to_frealign_init_and_run
            case('ESTIMATE_PERSISTENCE_LENGTH')
                write(*,'(a)') 'Frealix will run its estimate_persistence_length mode'
                call mode_estimate_pl_init_and_run
            case('CALC_OCC')
                write(*,'(a)') 'Frealix will run its calc_occ mode'
                call mode_calc_occ_init_and_run()
            case ('CREATE_RANDOM_CLASSES')
                write(*,'(a)') 'Frealix will run its create_random_classes mode'
                call mode_create_random_classes_init_and_run()
            case default
                write(*,'(a,a)') '**ERROR: this Frealix mode is not recognised: ', trim(frealix_mode)
                call terminate('Fatal error in Frealix')
        end select

    end subroutine main

end program frealix
