!>  \todo   Consider merging this into the image_comparison module
module fscs
    use images


contains

subroutine fsc_unit_tests()
    implicit none
    call fsc_unit_test_1()
end subroutine fsc_unit_tests

!>  \brief  Check that an image has fsc of 1.0 with itself
subroutine fsc_unit_test_1()
    use images
    use runtime_parameters, only : terminate
    implicit none
    ! Private variables
    type(image) ::  self, self_copy
    ! Start work
    write(*,'(a)') '**info(fsc_unit_test_1): entering'
    self%logical_dimensions = [13,13,13]
    call self%Allocate()
    call random_number(self%real_values)
    self_copy = self
    !call self_copy%Allocate(MOULD=SELF)
    !self_copy%real_values = self%real_values

    !
    !call compute_fsc(self,self_copy,fsc_values)
    !allocate(fsc_values(7))
    !fsc_values = 1.0
    !
    !if (any(fsc_values .LT. 0.999)) then
    !    write(*,'(a)') '**ERROR(fsc_unit_test_1): not all fsc values are 1.0'
    !    write(*,*) fsc_values
    !    call terminate('Failed unit test')
    !else
    !    write(*,'(a)') '**info(fsc_unit_test_1): passed unit test'
    !endif
    write(*,'(a)') '**info(fsc_unit_test_1): exiting'
end subroutine fsc_unit_test_1

!>  \brief  Check that an image has fsc of 1.0 with itself
subroutine fsc_unit_test_2()
    use images
    use runtime_parameters, only : terminate
    implicit none
    ! Private variables
    type(image) ::  self, self_copy
    real, allocatable :: fsc_values(:)
    ! Start work
    write(*,'(a)') '**info(fsc_unit_test_2): entering'
    self%logical_dimensions = [13,13,13]
    call self%Allocate()
    call random_number(self%real_values)
    self_copy = self
    !
    call compute_fsc(self,self_copy,fsc_values)
    !
    if (any(fsc_values .lt. 0.999)) then
        write(*,'(a)') '**ERROR(fsc_unit_test_2): not all fsc values are 1.0'
        write(*,*) fsc_values
        call terminate('Failed unit test')
    else
        write(*,'(a)') '**info(fsc_unit_test_2): passed unit test'
    endif
    write(*,'(a)') '**info(fsc_unit_test_2): exiting'
end subroutine fsc_unit_test_2

!>  \brief  Return the normalised cross-correlation between shells of volumes
subroutine compute_fsc(img1,img2,fsc_out)
    use images
    use runtime_parameters, only  : terminate
    implicit none
    ! Arguments
    type(image),        intent(inout)   ::  img1    !< Volume to be compared
    type(image),        intent(inout)   ::  img2    !< Volume to be compared
    real, allocatable,  intent(out) ::  fsc_out(:)  !< Output FSC. One correlation coefficient per shell.
    ! Private variables
    integer                     ::  i,j,k,irad,ierr
    real(kind=8),   allocatable ::  fsc_num(:)
    real(kind=8),   allocatable ::  fsc_denom1(:)
    real(kind=8),   allocatable ::  fsc_denom2(:)
    !logical                     ::  do_not_fold(3) = (/.true.,.false.,.false./)
    logical                     ::  real_input(2)
    logical,    parameter       ::  iignore_friedel_mates   =   .true.  ! Some Friedel mates are described explicitely by the fftw array, we should not count them twice!
    real                        ::  x,y,z

    ! Allocate memory
    allocate(   fsc_num   (img1%physical_upper_bound_complex(1)),   &
                fsc_denom1(img1%physical_upper_bound_complex(1)),   &
                fsc_denom2(img1%physical_upper_bound_complex(1))  )

    ! Check type
    real_input(1) = img1%is_in_real_space
    real_input(2) = img2%is_in_real_space
    if (real_input(1)) call img1%ForwardFFT()
    if (real_input(2)) call img2%ForwardFFT()

    !
    if ( img1%is_in_real_space .or. img2%is_in_real_space) then
        call terminate('Fatal error in FSC')
    endif

    ! Check dims
    if (.not. img1%HasSameDimensionsAs(img2)) then
        write(*,'(2(A,3(I0,1x)))') '**ERROR(fsc): images dimensions do not match: ', &
                                    img1%logical_dimensions, ' & ', img2%logical_dimensions
        call terminate('Fatal error in FSC')
    endif

    ! For now, fsc only works for cubes or squares
    if (.not. img1%IsSquare()) then
        write(*,'(a,3i0)') '**ERROR(fsc): images are not square / cubic: ', img1%logical_dimensions
        call terminate('Fatal error in FSC')
    endif

    ! Allocate output array
    if (allocated(fsc_out)) deallocate(fsc_out)
    allocate(fsc_out(img1%physical_upper_bound_complex(1)),stat=IERR)
    if (ierr .ne. 0) then
        write(*,'(a)') '**ERROR(fsc): Allocation of output array failed'
        call terminate('Fatal error in FSC')
    endif

    ! Initialise output array
    fsc_out = 0.0
    fsc_num = 0.0D0
    fsc_denom1 = 0.0D0
    fsc_denom2 = 0.0D0

    ! Loop over voxels
    do i=1,img1%physical_upper_bound_complex(1)
        x = real(i-1)**2
        do j=1,img1%logical_dimensions(2)
            if (    iignore_friedel_mates   .and. i == 1    &
                                            .and. j .gt. 1  &
                                            .and. j .lt. img1%physical_index_of_first_negative_frequency(2)) then
                cycle    ! Skip voxels whose Friedel mates are explicit in the fftw description
            endif
            y = real(img1%LogicalIndexGivenPhysicalIndexInFourierSpace(j,2))**2
            do k=1,img1%logical_dimensions(3)
                z = real(img1%LogicalIndexGivenPhysicalIndexInFourierSpace(k,3))**2
                if (iignore_friedel_mates   .and. i == 1    &
                                            .and. j == 1    &
                                            .and. k .gt. 1  &
                                            .and. k .lt. img1%physical_index_of_first_negative_frequency(3)) then
                    cycle
                endif
                irad = nint(sqrt(x+y+z)) + 1 ! Shell 0 (the origin voxel) needs index 1 (Fortran arrays are addressed from 1)
                if (irad .le. img1%physical_upper_bound_complex(1)) then
                    fsc_num(irad)    = fsc_num(irad)    + dble(img1%complex_values(i,j,k) * conjg(img2%complex_values(i,j,k)))
                    fsc_denom1(irad) = fsc_denom1(irad) + dble(abs(img1%complex_values(i,j,k)))**2
                    fsc_denom2(irad) = fsc_denom2(irad) + dble(abs(img2%complex_values(i,j,k)))**2
                endif
            enddo
        enddo
    enddo

    if (any(fsc_denom1 .le. 0.0D0) .or. any(fsc_denom2 .le. 0.0D0)) then
        write(*,'(a)') '**warning(compute_fsc): at least one of the shells appears to be empty!'
    endif

    ! Loop over shells
    do IRAD=1, img1%physical_upper_bound_complex(1)
        if (fsc_denom1(irad) .gt. 0.0D0 .and. fsc_denom2(irad) .gt. 0.0D0) then
            fsc_out(irad) = real( fsc_num(irad)/sqrt(fsc_denom1(irad)*FSC_DENOM2(irad)) )
        else
            fsc_out(irad) = 0.0D0
        endif
    enddo

    ! Return the images in the same space as they were given
    if (real_input(1)) call img1%BackwardFFT()
    if (real_input(2)) call img2%BackwardFFT()
end subroutine compute_fsc

    subroutine compute_fsc_curve(image_1,image_2,fsc_curve,pixel_size)
        use Curves
        use images_comparison
        !
        type(Image),    intent(inout)   ::  image_1, image_2
        type(Curve),    intent(inout)   ::  fsc_curve
        real,           intent(in)      ::  pixel_size              !<  Angstroms
        ! private variables
        real,   allocatable ::  fsc(:)
        ! start work

        call compute_fsc(image_1,image_2,fsc)
        call fsc_to_fsc_curve(fsc,fsc_curve,pixel_size)
    end subroutine compute_fsc_curve

!>  \brief  Compute the p-value, which quantifies the statistical significance with which estimated ssnr > target ssnr
!!
!!  The code largely follows the notations of Bershad & Rockmore (1974)
subroutine fsc_eval_p(fsc,n,rec_dim,obj_diam,n_asym,ssnr_target,p)
    use netlib_erf, only : derf
    implicit none
    ! Arguments
    real,       allocatable,    intent(in)      ::  fsc(:)      !<  fsc values, 1 per shell
    integer,    allocatable,    intent(in)      ::  n(:)        !<  For each shell, the number of values that were used to
                                                                !!  compute the fsc (not counting Friedel mates)
    integer,                    intent(in)      ::  rec_dim     !<  Dimension of the reconstruction volume (pixels)
    real,                       intent(in)      ::  obj_diam    !<  Average diameter of the object (pixels)
    integer,                    intent(in)      ::  n_asym      !<  Number of asymmetric units that contributed to the reconstruction
    real,                       intent(in)      ::  ssnr_target !<  Target ssnr
    real,       allocatable,    intent(inout)   ::  p(:)        !<  P-value
    ! Private variables
    integer ::  current_shell
    real    ::  current_fsc
    real    ::  ssnr_est_biased, ssnr_est_unbiased
    real    ::  ssnr_target_unbiased_var
    real    ::  n_ind
    real    ::  sig2_z_est, sig2_z_target
    real    ::  term_1, term_2, term_3, a, b
    real    ::  fsc_target
    ! Start work

    ! Allocate p-value array
    if (allocated(p)) deallocate(p)
    allocate(p(size(fsc)))

    ! Work out the target FSC. E.g. 0.143
    ! According to the equation below, FSC_TARGET=0.143 <=> SSNR_TARGET=0.333,
    ! which seems to contradict Table a1 in Rosenthal & Henderson (2003)
    fsc_target = ssnr_target / (2.0 + ssnr_target)

    ! Loop over the shells
    do CURRENT_SHELL=1,size(fsc)
        current_fsc = min(fsc(current_shell),0.999)

        ! Equation 7 in B&R
        ssnr_est_biased = 2.0 * current_fsc / (1-CURRENT_FSC)

        ! Number of independent Fourier voxels in the shell
        n_ind = n(current_shell) / real(n_asym) * (obj_diam / real(rec_dim))**2

        ! The variance of the Fisher transform of the fsc estimator at the target ssnr (Eq11 in B&R):
        term_1 = 1.0 / (N_IND-1.0)
        term_2 = (4.0-FSC_TARGET**2)/(2.0*(N_IND-1.0))
        term_3 = (-21.0*FSC_TARGET**2-21.0*FSC_TARGET+176) / (48.0*(N_IND-1.0)**2)

        sig2_z_target = term_1 * ( 1.0 + term_2 + term_3 )

        ! The variance of the Fisher transform of the fsc estimator at the estimated ssnr (Eq11 in B&R):
        term_2 = (4.0-CURRENT_FSC**2)/(2.0*(N_IND-1.0))
        term_3 = (-21.0*CURRENT_FSC**2-21.0*CURRENT_FSC+176) / (48.0*(N_IND-1.0)**2)

        sig2_z_est = term_1 * ( 1.0 + term_2 + term_3 )

        ! The unbiased estimate of ssnr
        a      = exp(-2.0*SIG2_Z_EST)
        b      = -0.5 * (1.0-EXP(-2.0*SIG2_Z_EST))
        ssnr_est_unbiased = a * ssnr_est_biased + b

        ! The error of the unbiased estimator (Eq 14 in B&R) at the target ssnr
        ssnr_target_unbiased_var = (exp(4.0*SIG2_Z_TARGET)-1.0) * (SSNR_TARGET+0.5)**2

        ! We call the Netlib double-precision version of erf, since not all compiler support the f2008 erf intrinsic yet
        p(current_shell) = 0.5*(1.0-DERF(   &
                                real( (SSNR_EST_UNBIASED-SSNR_TARGET)/sqrt(2.0*SSNR_TARGET_UNBIASED_VAR),KIND=8))    &
                                )

    enddo ! End of loop over shells
end subroutine fsc_eval_p

end module fscs
