!>	\brief	Histrogram fitting
!!
!!	\author Rachel Ruskin
module histogram_fit

contains

    function fit_function(values, params) bind(c)
    !Let's make a test placeholder function.
        use, intrinsic :: iso_c_binding
        use fgsl
        use runtime_parameters
        use histogram_fit_params
        implicit none
        integer                 ::      status
        real(kind=C_DOUBLE)     ::      fit_function
        type(c_ptr), value      ::      params
        type(hist_info), pointer        ::  parameters
        type(c_ptr), value      ::      values
        type(fgsl_vector)       ::  val_vec_fgsl
        real(fgsl_double), pointer  ::  val_fgsl(:)
        real(kind=8), allocatable   ::  v(:), rho(:), pdf(:), fit_fun_array(:)
        real(kind=8), pointer   ::model(:)


        call fgsl_obj_c_ptr(val_vec_fgsl,values)
        status = fgsl_vector_align(val_fgsl,val_vec_fgsl)
        if (status .ne. fgsl_success) call terminate('FGSL_VECTOR_ALIGN failed')
        allocate(v(size(val_fgsl)), stat=STATUS)
        if (status .ne. 0) call terminate('Memory allocation failed')
        V=VAL_FGSL

        !parameters is the usable one.
        call c_f_pointer(params, parameters)

        !calling this rho to make code more legible.
        allocate(rho(size(parameters%bin_bounds)), pdf(size(parameters%hist)), fit_fun_array(size(parameters%hist)))
        RHO=PARAMETERS%bin_bounds


        MODEL=>MAIN_PIECEWISE(v, rho, parameters)

        FIT_FUNCTION=SUM((MODEL-PARAMETERS%hist)**2)

        !--------------------------------------------------------------------------------------------
        !       This part is the non-squared fitting function. Keep it just in case
        !--------------------------------------------------------------------------------------------

        !Values: v(1)=p_m, v(2)=2sigma^2, v(3)=A, v(4)=p1, v(5)=p2, v(6)=p0, v(7)=N
        !And here's another very iffy fix for a divide by zero problem:
!       if (v(4) .EQ. v(5)) v(5)=V(4)+0.01
!       if (v(2) .LT. 0.001)    v(2)=0.001
!
!       Y1=EXP(-((v(4)-V(1)) ** 2)/V(2))
!       Y2=-(v(3)/v(5)) * (abs(log(abs(v(6)/v(5)))) ** (0.5))
!       S1=-((v(4)-V(1))/(0.5*V(2)))*EXP(-((v(4)-V(1))**2)/v(2))
!       S2=-V(3)/(V(5)**2) * (abs(log(abs(v(6)/v(5)))) ** (0.5) + (v(6)/(2*V(5))) * abs(log(abs(v(6)/v(5)))) ** (-0.5))
!       S0=(Y1-Y2)/(v(4)-V(5))
!       A=(S1+S2-2*S0)/((v(4)-V(5))**2)
!       B=(S0-S2)/(v(4)-V(5))-(v(4)+2*V(5))*A
!       C=S2-3*A*V(5)**2-2*B*V(5)
!       D=Y2-A*V(5)**3-B*V(5)**2-C*V(5)
!
!       !Values: v(1)=p_m, v(2)=2sigma^2, v(3)=A, v(4)=p1, v(5)=p2, v(6)=p0, v(7)=N
!       do I=1, size(parameters%hist)
!           ! for r < p1
!           if (rho(i) .LE. v(4)) then
!               pdf(i)=V(7)*V(7)*EXP(-((rho(i)-V(1))**2)/(v(2)))
!       !   !for p1<r<p2
!           else if ((rho(i) .LE. v(5)) .AND. (rho(i) .GT. v(4))) then
!               pdf(i)=V(7)*(A*RHO(i)**3+B*RHO(i)**2+C*RHO(i)+D)
!       !   !for r>p2
!           else if (rho(i) .GT. v(5)) then
!
!               pdf(i)=V(7)*(v(3)/rho(i))*ABS(log(abs(v(6)/rho(i))))**(0.5)
!           endif
!       enddo
!
!       !FIT_FUNCTION=SUM((PDF-PARAMETERS%hist)**2)
!       !--------------------------------------------------------------------------------
!       !       This part is the squared fitting function. Keep it just in case
!       !-------------------------------------------------------------------------------
!       !Values: v(1)=p_m, v(2)=2sigma^2, v(3)=A, v(4)=p1, v(5)=p2, v(6)=p0, v(7)=N
!
!       do I=1, size(parameters%hist)
!
!           ! for r < p1
!           if (rho(i) .LE. v(4)) then
!               pdf(i)=V(7)*EXP(-((rho(i)-V(1))**2)/(v(2)/2))
!           !for p1<r<p2
!           !Put in temporary absolute value statement.
!           else if ((rho(i) .LE. v(5)) .AND. (rho(i) .GT. v(4))) then
!               pdf(i)=(v(7)/(v(4)-V(5)))*((1/(V(2)/2))*EXP(-((v(4)-V(1))**2)/(v(2)/2))*((v(5)-RHO(i))**2) * &
!               (2*V(4)**3-V(5)*(v(2)/2)-2*RHO(i)*(v(5)*V(1)+V(2)/2)-2*(v(4)**2)*(v(5)+V(1)+RHO(i))+ &
!               v(4)*(3*(v(2)/2)+2*V(1)*RHO(i)+2*V(5)*(v(1)+RHO(i))))+((v(3)**2)*((v(4)-RHO(i))**2)* &
!               ((v(4)-V(5))*(v(5)-RHO(i))+(3*V(4)*V(5)-5*V(5)**2-2*V(4)*RHO(i)+4*V(5)*RHO(i))*LOG(abs(v(6)/v(5)))) &
!               /(v(5)**3)))
!           !for r>p2
!           else if (rho(i) .GT. v(5)) then
!               pdf(i)=V(7)*((v(3)/rho(i))**2)*LOG(abs(v(6)/rho(i)))
!           endif
!       enddo
!       ----------------------------------------------------------------------------------

!       TEMP=MAXLOC(parameters%hist)
!       do I=1, size(fit_fun_array)
!           if (i .LE. temp(1)) then
!               fit_fun_array(i)=((sqrt(abs(pdf(i)))-PARAMETERS%hist(i))**2)*0.5
!           else
!               fit_fun_array(i)=(sqrt(abs(pdf(i)))-PARAMETERS%hist(i))**2
!           endif
!       !enddo
!       !FIT_FUNCTION=SUM(fit_fun_array)
!       RIDER=0.0
!       if (v(1) .LT. 0) RIDER= RIDER+10**8
!       if (v(4) .LT. 0) RIDER= RIDER+10**8
!       if (v(5) .LT. v(4)) RIDER= RIDER+10**8
!       if (v(4) .LT. v(1)) RIDER= RIDER+10**8
!       if (v(6) .LT. 0) RIDER= RIDER+10**8
!       !if (v(4) .GT. 0.3) fit_function = FIT_FUNCTION+10**8
!       if (v(2) .LT. 0.1) RIDER= RIDER+10**8
!
!       FIT_FUNCTION=SUM((PDF-PARAMETERS%hist)**2)+RIDER
!
!       print *, 'rider', rider
!       print *, 'ff', fit_function
        !----------------------------------------------------------------------------------------------
        ! Here is a model fitting function that fits a Gaussian.
        !do I=1,300
        !   xvals(i)=EXP((-(-1+(I-1)/150.0)**2)/(2*SIGMA**2))
        !enddo

        !do I=1,300
        !   modelvals(i)=EXP((-((-1+(I-1)/150.0)-VAL_VEC(1))**2)/(2*VAL_VEC(2)**2))
        !enddo

        !FIT_FUNCTION=SUM((XVALS-MODELVALS)**2)
        !------------------------------------------------------------------------------------------------


    end function fit_function


end module histogram_fit
