!>	\brief	Used for histogram fitting
!!
!!	\author	Rachel Ruskin
module histogram_fit_params
    ! Contains the parameter type that needs to get shuttled back and forth
    ! Also contains the piecewise function
    implicit none

    type hist_info

    real, allocatable   ::  bin_bounds(:)
    real, allocatable   ::  hist(:)
    real, dimension (7, 2)  ::  ptm !(stands for parameters to minimize) This vector has 0 on parameters to be held fixed.
    !For parameters to be varied, contains the index of the values vector that v should correspond to.
    !For parameters to be fixed, contains the value of the parameter
    !For example, (.13 0.034 0 0.04 0.14 0 0// 0 0 1 0 0 2 3) means that v(3)=VALUES(1), and v(1)=0.13

    end type

    contains

    function main_piecewise(values, rho, params)

        implicit none

        real(kind=8)                        ::  v(7)
        real(kind=8)                        ::  values(:)
        real(kind=8)                        ::  rho(:)
        real(kind=8), allocatable, target   ::  pdf(:)
        real(kind=8), pointer               ::  main_piecewise(:)
        type(hist_info), pointer            ::  params
        integer                             ::  i

        allocate(pdf(size(rho)))

        do I=1, size(v)
            if (params%ptm(i, 2) .gt. 0.5) then
                v(i)=VALUES(int(params%ptm(i, 2)))
            else
                v(i)=PARAMS%ptm(i, 1)
            endif
        enddo

        !v(1)=0.13
        !v(2)=0.034
        !v(3)=VALUES(1)
        !v(4)=0.04
        !v(5)=0.14
        !v(6)=VALUES(2)
        !v(7)=VALUES(3)

!       Y1=EXP(-((v(4)-V(1)) ** 2)/V(2))
!       Y2=-(v(3)/v(5)) * (abs(log(abs(v(6)/v(5)))) ** (0.5))
!       S1=-((v(4)-V(1))/(0.5*V(2)))*EXP(-((v(4)-V(1))**2)/v(2))
!       S2=-V(3)/(V(5)**2) * ((log(abs(v(6)/v(5)))) ** (0.5) + (v(6)/(2*V(5))) * abs(log(abs(v(6)/v(5)))) ** (-0.5))
!       S0=(Y1-Y2)/(v(4)-V(5))
!       A=(S1+S2-2*S0)/((v(4)-V(5))**2)
!       B=(S0-S2)/(v(4)-V(5))-(v(4)+2*V(5))*A
!       C=S2-3*A*V(5)**2-2*B*V(5)
!       D=Y2-A*V(5)**3-B*V(5)**2-C*V(5)

        !Values: v(1)=p_m, v(2)=2sigma^2, v(3)=A, v(4)=p1, v(5)=p2, v(6)=p0, v(7)=N
        !---------------------------------------------------------------------------
        !       unsquared version
        !---------------------------------------------------------------------------
!       do I=1, size(rho)
!           ! for r < p1
!           if (rho(i) .LE. v(4)) then
!               pdf(i)=V(7)*EXP(-((rho(i)-V(1))**2)/(v(2)))
!       !   !for p1<r<p2
!           else if ((rho(i) .LT. v(5)) .AND. (rho(i) .GT. v(4))) then
!               pdf(i)=V(7)*(A*RHO(i)**3+B*RHO(i)**2+C*RHO(i)+D)
!       !   !for r>p2
!           else if (rho(i) .GE. v(5)) then
!
!               pdf(i)=V(7)*(v(3)/rho(i))*ABS(log(abs(v(6)/rho(i))))**(0.5)
!           endif
!       enddo

!---------------------------------------------------------------------------------
!               Squared Version
!---------------------------------------------------------------------------------

        do I=1, size(rho)
            ! for r < p1
            if (rho(i) .le. v(4)) then
            !to prevent floating point underflow (23=10 log 10)
                if (((rho(i)-V(1))**2)/(abs(v(2))/2) .ge. 23) then
                    pdf(i)=0
                else
                    pdf(i)=(v(7)**2)*EXP(-((rho(i)-V(1))**2)/(abs(v(2))/2))
                endif
            !for p1<r<p2
            !Put in temporary absolute value statement.
            else if ((rho(i) .lt. v(5)) .and. (rho(i) .gt. v(4))) then
                pdf(i)=((v(7)**2)/((v(4)-V(5))**3))*(((1/(V(2)/2))*EXP(-((v(4)-V(1))**2)/(abs(v(2))/2))*((v(5)-RHO(i))**2) * &
                (2*V(4)**3-V(5)*(v(2)/2)-2*RHO(i)*(v(5)*V(1)+V(2)/2)-2*(v(4)**2)*(v(5)+V(1)+RHO(i))+ &
                v(4)*(3*(v(2)/2)+2*V(1)*RHO(i)+2*V(5)*(v(1)+RHO(i))))+(((v(3)**2)/(v(5)**3))*((v(4)-RHO(i))**2)* &
                ((v(4)-V(5))*(v(5)-RHO(i))+(3*V(4)*V(5)-5*V(5)**2-2*V(4)*RHO(i)+4*V(5)*RHO(i))*LOG(abs(v(6)/v(5)))))))
            !for r>p2
            else if (rho(i) .ge. v(5)) then
                pdf(i)=(v(7)**2)*((v(3)/rho(i))**2)*LOG(abs(v(6)/rho(i)))
            endif
        enddo

        PDF=SQRT(abs(pdf))
        MAIN_PIECEWISE=>PDF

    end function main_piecewise

end module histogram_fit_params
