!>  \brief  Mode to align two images
module mode_align
    ! Modules
    use modes
    use rtps


    implicit none
    type(mode) :: align

    type(rtp_char), target  ::  input_filename  =               &
        rtp_char(   value='input.mrc',                  &
                    keyword='INPUT_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  reference_filename  =               &
        rtp_char(   value='reference.mrc',                  &
                    keyword='REFERENCE_FILENAME',           &
                    description='')

    type(rtp_logi), target  ::  helical_volume_alignment  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='HELICAL_VOLUME_ALIGNMENT',           &
                    description='The input and reference have helical symmetry around the z axis')

contains

subroutine mode_align_init_and_run
    implicit none
    ! Intialise the mode
    call align%init(command='ALIGN',subroutine=mode_align_slave,short_description='Align an image or volume to a reference')

    ! Add RTPs to the mode
    call align%add_rtp(input_filename)
    call align%add_rtp(reference_filename)
    call align%add_rtp(helical_volume_alignment)

    ! Run the mode
    call align%run()
end subroutine mode_align_init_and_run

subroutine mode_align_slave
    ! Modules
    use images
    use imagefiles
    use images_comparison, only : find_shift_euler
    use euleranglesmod
    implicit none
    ! Variables
    type(imagefile) ::  input_file, ref_file, output_file
    type(image)     ::  volin,volref
    real            ::  shift(3)
    type(euler)     ::  rotation, rotation_halfrange, rotation_sampling


    ! Start work

    ! Initialise input and output files
    call input_file%init(input_filename%value)
    call ref_file%init(reference_filename%value)
    call output_file%init('aligned.mrc')

    ! Read data from disk
    call volin%ReadFromDisk(input_file)
    call volref%ReadFromDisk(ref_file)

    ! Close input files
    call input_file%close()
    call ref_file%close()

    if (helical_volume_alignment%value) then


    else
        ! Set rotation exhaustive search parameters
        rotation_halfrange%psi = 30.0
        rotation_sampling%psi = 5.0
        print *, '**debug(mode_align): setting rotation search half range / sampling = ',  &
                     rotation_halfrange%psi, rotation_sampling%psi


        ! Find alignment parameters
        call find_shift_euler(volin,volref,shift,rotation,  &
                                rotation_sampling=rotation_sampling,rotation_half_range=rotation_halfrange)

        print '(a,3(F8.3,1x))', '**info(mode_align): shift from input to reference = ', shift
        print '(a,F8.3)', '**info(mode_align): rotation from input to reference = ', rotation%psi
    endif

    ! Add step to shift / rotate input -> output


end subroutine mode_align_slave

end module mode_align
