!>  \brief  Frealix mode which calculates the fsc between two volumes
module mode_calc_fsc
    use Curves
    use modes
    use rtps
    implicit none

    type(rtp_char), target  ::  input_volume_1  =               &
        rtp_char(   value='vol1',                  &
                    keyword='INPUT_VOLUME_1',           &
                    description='Input volume 1')


    type(rtp_char), target  ::  input_volume_2  =               &
        rtp_char(   value='vol2',                  &
                    keyword='INPUT_VOLUME_2',           &
                    description='Input volume 2')

    type(rtp_real), target  ::  pixel_size  =               &
        rtp_real(   value=1.0,                  &
                    keyword='pixel_size',           &
                    description='Pixel size in Angstroms (only used for scaling spatial frequencies when printing out FSC curve)')

    type(rtp_real), target  ::  fparticle  =               &
        rtp_real(   value=1.0,                  &
                    keyword='fparticle',           &
                    description='Fraction of non-masked voxels which are part of the object')



contains
subroutine mode_calc_fsc_init_and_run()
    implicit none
    type(mode) :: calc_fsc

    ! Intialise the mode
    call calc_fsc%init(command='CALC_FSC',subroutine=mode_calc_fsc_slave,short_description='Add description here')

    ! Add RTPs to the mode
    call calc_fsc%add_rtp(input_volume_1)
    call calc_fsc%add_rtp(input_volume_2)
    call calc_fsc%add_rtp(pixel_size)
    call calc_fsc%add_rtp(fparticle)

    ! Run the mode
    call calc_fsc%run()


end subroutine mode_calc_fsc_init_and_run

subroutine mode_calc_fsc_slave()
    use Images
    !use imagefiles
    use images_comparison
    !use fscs
    use Curves!, only : Curve

    implicit none

    ! Variables
    type(Image) ::  vol1,vol2,mask
    type(Curve) ::  my_fsc_curve
    type(Curve) ::  my_part_fsc_curve

    ! Get runtime parameters from control file


    ! Read volumes from disk
    call vol1%ReadFromDisk(input_volume_1%value,read_volume_not_slice=.true.)
    call vol2%ReadFromDisk(input_volume_2%value,read_volume_not_slice=.true.)

    ! Compute the normalised cross-correlation
    mask%logical_dimensions = vol1%logical_dimensions
    call mask%Allocate()
    mask%real_values = 1.0
    write(*,'(a,F0.4)') '**info(mode_calc_fsc): normalised cross-correlation coefficient: ',    &
                                                correlation(vol1,vol2,mask,normalise=.true.)

    ! Forward ft
    call vol1%ForwardFFT()
    call vol2%ForwardFFT()

    ! Compute FSC
    my_fsc_curve = vol1%GetFSCWith(vol2,pixel_size_to_use=pixel_size%value, &
                                   fraction_of_voxels=fparticle%value,fsc_part_curve=my_part_fsc_curve)

    ! Print the fsc to screen
    call my_fsc_curve%PrintCurve()
    call my_part_fsc_curve%PrintCurve()

end subroutine mode_calc_fsc_slave

end module mode_calc_fsc
