!>  \brief  Given an input .ali file, create a set of output .ali files, where the occupancies of the
!!          filaments will be randomized. Useful to initialize a classification.
module mode_create_random_classes
    use Globals
    use modes
    use rtps
    implicit none
    type(mode)  ::  create_random_classes

    type(rtp_char), target  ::  input_filename  =               &
        rtp_char(   value='input.ali',                  &
                    keyword='input_filename',           &
                    description='')


    type(rtp_char), target  ::  output_root_filename  =               &
        rtp_char(   value='output_???.ali',                  &
                    keyword='output_root_filename',           &
                    description='')


    type(rtp_intg), target  ::  number_of_classes  =               &
        rtp_intg(   value=4,                  &
                    keyword='number_of_classes',           &
                    description='')

contains

subroutine mode_create_random_classes_init_and_run()
    call create_random_classes%Init(command='create_random_classes',subroutine=mode_create_random_classes_slave, &
                                    short_description='Create a set of random classes')
    !
    call create_random_classes%add_rtp(input_filename)
    call create_random_classes%add_rtp(output_root_filename)
    call create_random_classes%add_rtp(number_of_classes)
    !
    call create_random_classes%run()
end subroutine mode_create_random_classes_init_and_run

subroutine mode_create_random_classes_slave()
    use frealix_parameters
    use filaments
    use parsers
    !
    type(filament), allocatable ::  filament_array(:)
    type(filament), allocatable ::  output_filament_array(:,:)
    integer ::  filament_counter
    integer ::  class_counter
    real    ::  random_occ
    real    ::  sum_occ
    character(len=filename_max_len) ::  class_filename
    !

    ! Read in filament array from file
    call filaments_from_frealix_par_noinit(filament_array,input_filename%value,1.0)

    ! Allocate memory for output filaments
    allocate(output_filament_array(size(filament_array),number_of_classes%value))

    ! Copy input filaments over and randomize occupancies
    do filament_counter=1,size(filament_array)
        call random_seed()
        sum_occ = 0.0
        do class_counter=1,number_of_classes%value
            call random_number(random_occ)
            output_filament_array(filament_counter,class_counter) = filament_array(filament_counter)
            output_filament_array(filament_counter,class_counter)%wp(:)%occupancy = random_occ
            sum_occ = sum_occ + random_occ
        enddo
        ! Normalize the occupancies
        do class_counter=1,number_of_classes%value
            output_filament_array(filament_counter,class_counter)%wp(:)%occupancy =  &
                   output_filament_array(filament_counter,class_counter)%wp(:)%occupancy * 100.0 / sum_occ
        enddo
    enddo

    ! Write out the classes
    do class_counter=1,number_of_classes%value
        call filename_from_wild(output_root_filename%value,class_counter,class_filename)
        call filaments_to_frealix_par(output_filament_array(:,class_counter),class_filename)
    enddo

end subroutine mode_create_random_classes_slave



end module mode_create_random_classes
