!>  \brief  Given a set of boxer files, film files and ctftilt/find files, bootstrap frealix analysis
!!          by generating of parameter file. If approximate helical parameters are known, a 3d reconstruction
!!          can also be generated.
!!
!!
!!
module mode_filament_startup
!dir$ optimize:0
    !
    ! Modules we'll need
    !
    use runtime_parameters
    use modes
    use rtps


    implicit none
    type(mode) :: filament_startup

    type(rtp_char), target  ::  input_box_files  =               &
        rtp_char(   value='input_?????_?????.box',                  &
                    keyword='INPUT_BOX_FILES',           &
                    description='The first segment of ? is the film identifier, the second one is the filament identifier. '//&
                                'This second identifier must start at 1 and increment without breaks.')

    type(rtp_char), target  ::  input_imod_points_filename  =               &
        rtp_char(   value='filaments.txt',                  &
                    keyword='INPUT_IMOD_POINTS_FILENAME',           &
                    description='Output of IMOD''s model2point command (with -contour option). '//&
                                'Coordinates along selected filaments.')

    type(rtp_char), target  ::  output_parameter_file  =               &
        rtp_char(   value='waypoints.ali',                  &
                    keyword='OUTPUT_PARAMETER_FILE',           &
                    description='')


    type(rtp_char), target  ::  ctftilt_files  =               &
        rtp_char(   value='none',                  &
                    keyword='CTFTILT_FILES',           &
                    description='')


    type(rtp_char), target  ::  ctffind_files  =               &
        rtp_char(   value='none',                  &
                    keyword='CTFFIND_FILES',           &
                    description='')


    type(rtp_char), target  ::  film_files  =               &
        rtp_char(   value='film_?????.mrc',                  &
                    keyword='FILM_FILES',           &
                    description='')


    type(rtp_char), target  ::  output_3d_filename  =               &
        rtp_char(   value='model3d_out.mrc',                  &
                    keyword='OUTPUT_3D_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  filament_trace_files  =               &
        rtp_char(   value='trace_?????_?????.box',                  &
                    keyword='FILAMENT_TRACE_FILES',           &
                    description='')


    type(rtp_char), target  ::  film_selection  =               &
        rtp_char(   value='1-120,122,125-128',                  &
                    keyword='FILM_SELECTION',           &
                    description='Which films we''re going to analyse - the string here would take all films from '//&
                                '1 to 120, film 122 and films 125 to 128')


    type(rtp_logi), target  ::  reverse_film_contrast  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='REVERSE_FILM_CONTRAST',           &
                    description='It is assumed that the protein density is darker than the background. If not and you'//&
                                ' want the contrast to be reversed internally, set this to T')


    !type(rtp_logi), target  ::  calc_3d_reconstruction  =               &
    !    rtp_logi(   value=.false.,                  &
    !                keyword='CALC_3D_RECONSTRUCTION',           &
    !                description='Whether we should attempt a 3d reconstruction (requires known helical parameters)')


    type(rtp_real), target  ::  pixel_size  =               &
        rtp_real(   value=9.99,                  &
                    keyword='PIXEL_SIZE',           &
                    description='')


    type(rtp_real), target  ::  acceleration_voltage =  &
        rtp_real(   value=300.0,                        &
                    keyword='acceleration_voltage',     &
                    description='in kV')

    type(rtp_real), target  ::  spherical_aberration =  &
        rtp_real(   value=2.7,                          &
                    keyword='spherical_aberration',     &
                    description='in mm')

    type(rtp_real), target  ::  amplitude_contrast =    &
        rtp_real(   value=0.07,                         &
                    keyword='amplitude_contrast',       &
                    description='Fraction of amplitude contrast')


    type(rtp_logi), target  ::  add_extra_first_last_wps  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='ADD_EXTRA_FIRST_LAST_WPS',           &
                    description='Whether to add extra way points before & after the first & last waypoints '//  &
                                'described by the boxer files')


    type(rtp_real), target  ::  twist  =               &
        rtp_real(   value=0.0,                  &
                    keyword='TWIST_PER_SUBUNIT',           &
                    description='')


    type(rtp_real), target  ::  rise  =               &
        rtp_real(   value=0.0,                  &
                    keyword='RISE_PER_SUBUNIT',           &
                    description='')


    type(rtp_intg), target  ::  symmetry_axial  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_AXIAL',           &
                    description='')


    type(rtp_intg), target  ::  symmetry_perp  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_PERP',           &
                    description='')


    type(rtp_logi), target  ::  wps_are_crossovers  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='WPS_ARE_CROSSOVERS',           &
                    description='Whether waypoints (WPs) are crossovers (e.g. Amyloid beta fibrils)')


    type(rtp_logi), target  ::  fal_wps_are_crossovers  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='FAL_WPS_ARE_CROSSOVERS',           &
                    description='Whether the first and last waypoints are also crossovers. Typically, it''s recommended '// &
                                'extra fal waypoints are picked before and after the first and last waypoint. '//&
                                'Therefore, this would typically false.')


    type(rtp_logi), target  ::  fal_wps_in_threed  =               &
        rtp_logi(   value=.true.,                  &
                    keyword='FAL_WPS_IN_THREED',           &
                    description='If false, the parts of the filament between the first and the second as well as between'//&
                                ' the penultimate and last waypoints will not be included in the 3d reconstruction')


    type(rtp_real), target  ::  helix_max_radius  =               &
        rtp_real(   value=0.0,                  &
                    keyword='HELIX_MAX_RADIUS',           &
                    description='Maximum outer radius of the helix, in Angstroms')


    type(rtp_logi), target  ::  microtubule  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='MICROTUBULE',           &
                    description='Whether the filaments are microtubules')


    type(rtp_intg), target  ::  mt_pf_number  =               &
        rtp_intg(   value=0,                  &
                    keyword='MT_PF_NUMBER',           &
                    description='For microtubules only: number of protofilaments (e.g. 13)')


    type(rtp_intg), target  ::  mt_rise  =               &
        rtp_intg(   value=0,                  &
                    keyword='MT_RISE',           &
                    description='For microtubules only: how much further along the filament (distance measured in '//   &
                                'tubulin subunits) the end of a MT_PF_NUMBER-element "washer" is compared to its '//&
                                'beginning. The 13-pf microtobules have a value of 3.')


    type(rtp_intg), target  ::  num_helical_starts  =               &
        rtp_intg(   value=1,                  &
                    keyword='NUM_HELICAL_STARTS',           &
                    description='Number of helical starts. The helical parameters given above must describeone of '//&
                                'these starts. Not including symmetry (e.g. could be describing a 1-start helix with '//&
                                'an additional 2-fold axial symmetry).')


    type(rtp_logi), target  ::  write_filament_traces  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='WRITE_FILAMENT_TRACES',           &
                    description='Whether to write coordinates (boxer) files showing waypoint positions and filament traces')

contains

subroutine mode_filament_startup_init_and_run
    ! Intialise the mode
    call filament_startup%init(command='FILAMENT_STARTUP',subroutine=mode_filament_startup_slave,&
                                short_description='Add description here')

    ! Add RTPs to the mode
    call filament_startup%add_rtp(input_box_files)
    call filament_startup%add_rtp(input_imod_points_filename)
    call filament_startup%add_rtp(output_parameter_file)
    call filament_startup%add_rtp(ctftilt_files)
    call filament_startup%add_rtp(ctffind_files)
    call filament_startup%add_rtp(film_files)
    call filament_startup%add_rtp(output_3d_filename)
    call filament_startup%add_rtp(filament_trace_files)
    call filament_startup%add_rtp(film_selection)
    call filament_startup%add_rtp(reverse_film_contrast)
    !call filament_startup%add_rtp(calc_3d_reconstruction)
    call filament_startup%add_rtp(pixel_size)
    call filament_startup%add_rtp(acceleration_voltage)
    call filament_startup%add_rtp(spherical_aberration)
    call filament_startup%add_rtp(amplitude_contrast)
    !call filament_startup%add_rtp(add_extra_first_last_wps)
    call filament_startup%add_rtp(twist)
    call filament_startup%add_rtp(rise)
    call filament_startup%add_rtp(symmetry_axial)
    call filament_startup%add_rtp(symmetry_perp)
    call filament_startup%add_rtp(wps_are_crossovers)
    call filament_startup%add_rtp(fal_wps_are_crossovers)
    call filament_startup%add_rtp(fal_wps_in_threed)
    call filament_startup%add_rtp(helix_max_radius)
    call filament_startup%add_rtp(microtubule)
    call filament_startup%add_rtp(mt_pf_number)
    call filament_startup%add_rtp(mt_rise)
    call filament_startup%add_rtp(num_helical_starts)
    call filament_startup%add_rtp(write_filament_traces)

    ! Run the mode
    call filament_startup%run()

end subroutine mode_filament_startup_init_and_run

subroutine mode_filament_startup_slave
    use Globals
    use coos, only : waypoint, boxer_db, imod_point, point
    use coos, only : imod_point_print_info, imod_points_read_from_disk
    use coos, only : boxer_db_read_from_disk
    use Micrographs
    use parsers
    use filaments
    use helices
    use frealix_parameters
    use threeds
    use fmcs
    use interpolation
    use DatesAndTimes
    use ImageFiles
    use StringManipulations, only : StringsAreEqual
    use parsers, only : filename_contains_qmarks
    use ctfs, only : ctf_parameters, ctf_parameters_from_numeric_text_file, ctf_parameters_init
    use ContrastTransferFunctions
    use ctffind_routines
    use ProgressBars

    implicit none
    !
    ! Private variables
    !
    integer,    allocatable         ::  list_of_films(:), num_of_filaments(:)
    integer                         ::  i, j, k, film_num, filament_num, ierr, current_point
    character(len=filename_len)     ::  film_filename
    character(len=filename_len)     ::  ctf_filename
    character(len=filename_len)     ::  box_filename_root, box_filename
    character(len=filename_len)     ::  filament_trace_filename, temp_filename
    logical                         ::  use_ctftilt     ! Whether we'll be using ctftilt (rather than ctffind)
    type(Micrograph)                ::  current_film
    type(boxer_db)                  ::  current_boxer_db
    !type(point)                     ::  points
    type(filament), allocatable     ::  fils(:)
    type(helical_symmetry)          ::  helix
    logical                         ::  new_film, new_filament
    type(image)                     ::  current_micrograph_frames(1)
    type(image)                     ::  new_3dvol
    type(threed)                    ::  new_3drec
    real                            ::  film_hp_param
    type(imagefile)                 ::  film_file
    type(imagefile)                 ::  threed_file
    type(interpolation_scheme)      ::  interpol_scheme
    type(imod_point), allocatable   ::  imod_points(:)
    logical, allocatable            ::  which_film(:)
    type(point), allocatable        ::  current_fil_points(:)
    integer                         ::  first_point, last_point
    logical                         ::  end_of_filament
    logical,    parameter           ::  debug    =  .false.
    character(len=6)                ::  NUM_FILMS_C
    type(ctf_parameters), allocatable   ::  ctf_params(:)
    type(DateAndTime)                   ::  myTime
    integer                             ::  number_of_micrographs
    type(ImageFile)                     ::  temp_image_file
    integer                             ::  point_counter
    integer                             ::  number_of_waypoints
    type(ContrastTransferFunction), allocatable ::  modern_ctfs(:)
    integer                             ::  ctffind_major_version
    character(len=:),   allocatable     ::  ctffind_version_string
    type(ProgressBar)                   ::  my_progress_bar


    !
    ! Get variable values from control file
    !



    !
    ! Start work
    !

    ! Are we given ctffind files or ctftilt files?
    use_ctftilt = .false.
    if (string_equal(ctftilt_files%value,'none')) then
        if (string_equal(ctffind_files%value,'none')) then
            write(*,'(a)') '**error(mode_filament_startup): neither ctffind nor ctftilt files were supplied'
            call terminate('Fatal error in MODE_FILAMENT_STARTUP')
        else
            use_ctftilt = .false.
        endif
    else
        use_ctftilt = .true.
    endif

    ! Diagnostic - are using CTFTILT?
    if (.not. use_ctftilt) then
        print '(a)', '**info(mode_filament_startup): No ctftilt files given, will use ctffind info'
    endif

    ! We'll use either eman boxes or imod points
    if (input_box_files%set_by_user .and. input_imod_points_filename%set_by_user) then
        write(*,'(a)') '**error(mode_filament_startup): you cannot give both an imod points file and box files as input.'
        call terminate('Fatal error in MODE_FILAMENT_STARTUP')
    endif

    ! Check the imod points file exists
    if (input_imod_points_filename%set_by_user) then
        if (.not. file_exists(input_imod_points_filename%value)) then
            write(*,'(2a)') '**error(mode_filament_startup): no such file: ', input_imod_points_filename%value
            call terminate('Fatal error in MODE_FILAMENT_STARTUP')
        endif
    endif


    ! Check
    if (add_extra_first_last_wps%value .and. .not. fal_wps_are_crossovers%value) then
        write(*,'(2a)') '**error: you asked for extra waypoints to be added and said that the first and ',   &
                        'last waypoints supplied were not crossovers.'
        call terminate('Fatal error in MODE_FILAMENT_STARTUP')
    endif

    ! Get list of films we'll be analysing from the user
    if (filename_contains_qmarks(film_files%value)) then
        list_of_films = set_to_list(film_selection%value)
        number_of_micrographs = size(list_of_films)
        write(num_films_c,'(i6)') number_of_micrographs
        if (number_of_micrographs .gt. 100) then
            write(*,'(a,i0,a)') '**info(mode_filament_startup): ', number_of_micrographs, ' films are selected'
        else
            print '(a,i3,a,'//num_films_c//'(i0,1x))', '**info(mode_filament_startup): The following ', &
                                                        number_of_micrographs, ' films are selected: ', list_of_films
        endif
    else
        ! We have a stack of micrographs
        call temp_image_file%Init(film_files%value)
        number_of_micrographs =  temp_image_file%GetStackSize()
        call temp_image_file%Close()
    endif

    ! Allocate memory so we can track the number of filaments per film
    allocate(num_of_filaments(number_of_micrographs),stat=ierr)
    if (ierr .ne. 0) then
        write(*,'(a)') '**error(mode_filament_startup): memory allocation for count of filaments failed with error ', ierr
        call terminate('Fatal error in MODE_FILAMENT_STARTUP')
    endif

    ! Initialise the helical symmetry object
    call helical_symmetry_init( helix,rise%value/pixel_size%value,twist%value,    &
                                helix_max_radius%value/pixel_size%value, &
                                symmetry_axial%value,symmetry_perp%value,num_starts=num_helical_starts%value)

    ! Also, if we specify that the points are crossovers, phi will be set to 0/180
    if (wps_are_crossovers%value) then
        write(*,'(a)') '**info(mode_filament_startup): supplied coordinates are crossovers & will be assigned phi 0 or 180'
        if (.not. fal_wps_are_crossovers%value) then
            write(*,'(a)') '**info(mode_filament_startup): first and last waypoints will not be treated as crossovers'
        endif
    endif

    ! Check that we have the film, ctf and box files available
    do i=1,number_of_micrographs
        if (filename_contains_qmarks(film_files%value)) then
            film_num = list_of_films(i)
            ! Check the relevant film file exists
            call filename_from_wild(film_files%value,film_num,film_filename)
            if (.not. file_exists(film_filename)) then
                write(*,'(a,a)') '**error(mode_filament_startup): file does not exist: ', trim(adjustl(film_filename))
                call terminate('Fatal error in MODE_FILAMENT_STARTUP')
            endif
        else
            film_num = i
        endif

        ! Check the relevant ctffind/ctftilt file exists
        if (use_ctftilt) then
            call filename_from_wild(ctftilt_files%value,film_num,ctf_filename)
        else
            if (filename_contains_qmarks(ctffind_files%value)) then
                call filename_from_wild(ctffind_files%value,film_num,ctf_filename)
                if (.not. file_exists(ctf_filename)) then
                    write(*,'(a,a)') '**error(mode_filament_startup): file does not exist: ', trim(adjustl(ctf_filename))
                    call terminate('Fatal error in MODE_FILAMENT_STARTUP')
                endif
            endif
        endif


        ! Check the boxer file exists
        if (input_box_files%set_by_user) then
            ! Find out the root filename for the box files associated with this film
            ! (filename_from_wild) only acts on the first set of question marks, leaves the rest of the filename alone
            call filename_from_wild(input_box_files%value,film_num,box_filename_root)

            ! If there are questions marks remaining, then we have one box file per filament. If not, we have one box file
            ! per micrograph
            if (filename_contains_qmarks(box_filename_root)) then
                ! Check the first box file exists
                call filename_from_wild(box_filename_root,1,box_filename)
                if (.not. file_exists(box_filename)) then
                    write(*,'(a,a)') '**warning(mode_filament_startup): file does not exist: ', trim(box_filename)
                    write(*,'(a)') 'Note that the .box files must be numbered first with the film identifier, ',    &
                                    'then in a sequence starting at 1'
                    !call terminate('Fatal error in MODE_FILAMENT_STARTUP')
                endif
            endif
        endif
    enddo


    !
    ! Find out how many filaments we'll be dealing with, store that in the num_of_filaments array (indexed by film)
    !
    num_of_filaments = 0
    if (input_box_files%set_by_user) then
        ! We were given .box files as input
        ! The outer loop is over the films, the inner loop over filaments
        write(*,'(a)') '**info(mode_filament_startup): counting number of filaments per micrograph'
        call my_progress_bar%Begin(number_of_micrographs)
        do i=1,number_of_micrographs
            call my_progress_bar%Update(i)
            if (filename_contains_qmarks(film_files%value)) then
                film_num = list_of_films(i)
            else
                film_num = i
            endif
            call filename_from_wild(input_box_files%value,film_num,box_filename_root)
            if (filename_contains_qmarks(box_filename_root)) then
                ! Loop over the box files
                j = 0
                do
                    j = j+1
                    ! Check box file exists
                    call filename_from_wild(box_filename_root,j,box_filename)
                    if (file_exists(box_filename)) then
                        !
                    else
                        ! This file didn't exist
                        num_of_filaments(i) = j-1
                        if (debug .or. this_program%IsVerbose()) then
                            write(*,'(a,i0,a,i0,a)') '**info(mode_filament_startup): micrograph ', film_num, ' has ', &
                                                        num_of_filaments(i), ' associated filaments'
                        endif
                        exit
                    endif
                enddo
            else
                if (file_exists(box_filename_root)) then
                    call boxer_db_read_from_disk(current_boxer_db,box_filename_root)
                    num_of_filaments(i) = current_boxer_db%point(size(current_boxer_db%point))%filament_number
                    if (debug .or. this_program%IsVerbose()) then
                        write(*,'(a,i0,a,i0,a)') '**info(mode_filament_startup): micrograph ', film_num, ' has ', &
                                                            num_of_filaments(i), ' associated filaments'
                    endif
                endif
            endif
        enddo
        call my_progress_bar%Finish()
    else if (input_imod_points_filename%set_by_user) then
        ! We were given imod points as input, let's load them all into memory
        call imod_points_read_from_disk(imod_points,input_imod_points_filename%value)
        if (debug) write(*,'(a)') '**debug(mode_filament_startup): below are the imod points:'
        ! Loop over the imod points
        write(*,'(a)') '**info(mode_filament_startup): parsing imod points and counting filaments'
        call my_progress_bar%Begin(size(imod_points))
        do current_point=1,size(imod_points)
            call my_progress_bar%Update(current_point)
            if (debug) call imod_point_print_info(imod_points(current_point))
            ! Are we on a new filament?
            new_filament = .false.
            if (current_point .eq. 1) then
                new_filament = .true.
            else
                if (imod_points(current_point)%contour_number .ne. imod_points(CURRENT_POINT-1)%contour_number) then
                    new_filament = .true.
                endif
            endif
            ! If we're on a new filament and it belongs to a film we're interested in, we'll need to keep track
            if (new_filament) then
                which_film = imod_points(current_point)%section_number .eq. list_of_films
                if (any(which_film)) then
                    do i=1,size(which_film)
                        if (which_film(i)) then
                            num_of_filaments(i) = num_of_filaments(i)+1
                            exit
                        endif
                    enddo
                endif
            endif
        enddo
        call my_progress_bar%Finish()
    endif

    ! We can now allocate memory for all the filament objects
    print *, '**info(mode_filament_startup): Total number of filaments = ', sum(num_of_filaments)
    if (sum(num_of_filaments) .eq. 0) then
        call this_program%TerminateWithFatalError('mode_filament_startup', &
                            'Did not find any filaments. Check input coordinates filenames.')
    endif
    allocate(fils(sum(num_of_filaments)),stat=ierr)
    if (ierr .ne. 0) then
        write(*,'(a)') '**error(mode_filament_startup): memory allocation for filaments failed with error ', ierr
        call terminate('Fatal error in MODE_FILAMENT_STARTUP')
    endif


    !
    ! If we were given a list of ctffind (>=3.6.0) results, parse
    !
    ! First, let's check whether we're given ctffind4 results
    if (.not. StringsAreEqual(ctffind_files%value,'none') .and. .not. filename_contains_qmarks(ctffind_files%value)) then
        call FindCTFFindVersion(ctffind_files%value,ctffind_major_version,ctffind_version_string)
        if (ctffind_major_version .eq. 4) then
            call ParseCTFFindResultsFile(ctffind_files%value,pixel_size%value,modern_ctfs)
            ! Convert to old-school ctf_paramters objects
            allocate(ctf_params(size(modern_ctfs)))
            do i=1,size(modern_ctfs)
                call ctf_parameters_init(ctf_params(i),acceleration_voltage%value,spherical_aberration%value, &
                                        amplitude_contrast%value, &
                                        modern_ctfs(i)%GetDefocus1InAngstroms()/10000.0, &
                                        modern_ctfs(i)%GetDefocus2InAngstroms()/10000.0, &
                                        modern_ctfs(i)%GetAstigmatismAzimuthInDegrees())
            enddo
        else
            write(*,'(a)') '**info(mode_filament_startup): supplied results don''t look like CTFFIND4 output. Trying CTFFIND3.'
            if (.not. StringsAreEqual('none',ctffind_files%value)) then
                call ctf_parameters_from_numeric_text_file(ctf_params,ctffind_files%value,acceleration_voltage%value, &
                                                           spherical_aberration%value,amplitude_contrast%value)
            endif
        endif
    endif

    !
    ! Now we can loop again, but we don't have to check files exists, and we know how many filaments we're expecting
    !
    if (input_imod_points_filename%set_by_user) then
        filament_num = 0
        first_point = 1
        write(*,'(a)') '**info(mode_filament_startup): parsing imod points and setting up filaments'
        call my_progress_bar%Begin(size(imod_points))
        do current_point=1,size(imod_points)
            call my_progress_bar%Update(current_point)
            ! Are we at the end of the current filament?
            end_of_filament = .false.
            if (current_point .eq. size(imod_points)) then
                end_of_filament = .true.
            else
                if (imod_points(current_point)%contour_number .ne. imod_points(CURRENT_POINT+1)%contour_number) then
                    end_of_filament = .true.
                endif
            endif
            ! If we're at the end of the current filament and it belongs to a film we're interested in, we'll need to keep track
            if (end_of_filament) then
                last_point = current_point
                which_film = imod_points(current_point)%section_number .eq. list_of_films
                if (any(which_film)) then
                    ! We're about to initialise a new filament
                    filament_num = filament_num + 1
                    ! Which film is this filament on?
                    do i=1,size(which_film)
                        if (which_film(i)) then
                            film_num = list_of_films(i)
                            exit
                        endif
                    enddo
                    ! Output for the user
                    if (debug .or. this_program%IsVerbose()) then
                        write(*,'(4(A,i0),a)') '**info(mode_filament_startup): filament ', filament_num, ' of ', size(fils), &
                                                ' is on micrograph ', film_num, ' and has ', last_point-first_point+1 , ' waypoints'
                    endif
                    ! Get the film filename, the ctf information and initialise the film object
                    call filename_from_wild(film_files%value,film_num,film_filename)
                    if (use_ctftilt) then
                        call filename_from_wild(ctftilt_files%value,film_num,ctf_filename)
                        call current_film%Init(film_num,film_filename,pixel_size%value,ctftilt_filename=ctf_filename)
                    else
                        if (filename_contains_qmarks(ctffind_files%value)) then
                            call filename_from_wild(ctffind_files%value,film_num,ctf_filename)
                            call current_film%Init(film_num,film_filename,pixel_size%value,ctffind_filename=ctf_filename)
                        else
                            call current_film%Init(film_num,film_filename,pixel_size%value,ctf_params=ctf_params(film_num))
                        endif
                    endif
                    ! Get the points for the current filament
                    if (allocated(current_fil_points)) deallocate(current_fil_points)
                    allocate(current_fil_points(last_point-first_point+1))
                    do i=1,size(current_fil_points)
                        current_fil_points(i)%coo(1:2) = imod_points(first_point+i-1)%coo(1:2)
                    enddo
                    ! Initialise a filament - this includes a spline fit - since we're not giving way points (just points),
                    ! this will be a parabolic spline, and the psi and theta angles will be worked out from it.
                    call filament_init( fils(filament_num),current_fil_points,  &
                                        film_in=current_film,crossovers=wps_are_crossovers%value,   &
                                        helix=helix,id=filament_num, &
                                        add_extra_fal_wps=add_extra_first_last_wps%value,   &
                                        fal_wps_are_crossovers=fal_wps_are_crossovers%value, &
                                        max_radius=helix_max_radius%value, &
                                        fal_wps_in_threed=fal_wps_in_threed%value, &
                                        microtubule=microtubule%value, mt_pf_number=mt_pf_number%value,mt_rise=mt_rise%value)

                    ! Diagnostic - print filament info
                    call fils(filament_num)%PrintInfo()


                    ! Diagnostic - write out waypoint boxer files
                    if (write_filament_traces%value) then
                        call filename_from_wild(filament_trace_files%value,film_num,filament_trace_filename)
                        call filename_from_wild(filament_trace_filename,fils(filament_num)%id,filament_trace_filename)
                        call filename_remove_extension(filament_trace_filename,temp_filename)
                        filament_trace_filename = trim(adjustl(temp_filename))//'_wp.box'
                        call filament_boxer_db_to_disk(fils(filament_num),filament_trace_filename,with_trace=.false.)
                    endif

                endif ! End of: filament was on one of the selected films
                ! Ready for the next filament
                first_point = current_point+1
                last_point  = current_point+1
            endif ! End of: we were on the last point of a filament
        enddo
        call my_progress_bar%Finish()
    else if (input_box_files%set_by_user) then
        filament_num = 0
        write(*,'(a)') '**info(mode_filament_startup): parsing box files and setting up filaments'
        call my_progress_bar%Begin(number_of_micrographs)
        do i=1,number_of_micrographs
            call my_progress_bar%Update(i)
            if (filename_contains_qmarks(film_files%value)) then
                film_num = list_of_films(i)
            else
                film_num = i
            endif

            ! Find out the root filename for the box files associated with this film
            ! (filename_from_wild) only acts on the first set of question marks, leaves the rest of the filename alone
            call filename_from_wild(input_box_files%value,film_num,box_filename_root)

            ! The filename for the current film
            if (filename_contains_qmarks(film_files%value)) then
                call filename_from_wild(film_files%value,film_num,film_filename)
            endif

            ! Before we loop over the filaments associated with this film, let's initialise a film object
            if (use_ctftilt) then
                call filename_from_wild(ctftilt_files%value,film_num,ctf_filename)
                if (filename_contains_qmarks(film_files%value)) then
                    call current_film%Init(film_num,film_filename,pixel_size%value,ctftilt_filename=ctf_filename)
                else
                    call current_film%Init(film_num,film_files%value,pixel_size%value, &
                                           ctftilt_filename=ctf_filename,location_within_file=film_num)
                endif
            else
                if (filename_contains_qmarks(ctffind_files%value)) then
                    call filename_from_wild(ctffind_files%value,film_num,ctf_filename)
                    if (filename_contains_qmarks(film_files%value)) then
                        call current_film%Init(film_num,film_filename,pixel_size%value,ctffind_filename=ctf_filename)
                    else
                        call current_film%Init(film_num,film_files%value,pixel_size%value, &
                                               ctffind_filename=ctf_filename,location_within_file=film_num)
                    endif
                else
                    if (filename_contains_qmarks(film_files%value)) then
                        call current_film%Init(film_num,film_filename,pixel_size%value,ctf_params=ctf_params(film_num))
                    else
                        call current_film%Init(film_num,film_files%value,pixel_size%value, &
                                               ctf_params=ctf_params(film_num),location_within_file=film_num)
                    endif
                endif
            endif

            !
            ! Loop over the box files
            !
            if (num_of_filaments(i) .gt. 0) then
            do j=1, num_of_filaments(i)

                ! Increment the count of filaments
                filament_num = filament_num + 1

                if (filename_contains_qmarks(box_filename_root)) then
                    call filename_from_wild(box_filename_root,j,box_filename)
                else
                    box_filename = box_filename_root
                endif

                ! Output for the user
                if (debug .or. this_program%IsVerbose()) then
                    write(*,'(a,i0,a,i0,a,i0)') '**info(mode_filament_startup): on filament ', j, ' of ', num_of_filaments(i), &
                                                ' on micrograph ', film_num
                endif

                ! Read the coordinates in
                call boxer_db_read_from_disk(current_boxer_db,box_filename)

                ! Get the points for the current filament
                if (filename_contains_qmarks(box_filename_root)) then
                    if (allocated(current_fil_points)) deallocate(current_fil_points)
                    allocate(current_fil_points(size(current_boxer_db%point)))
                    do k=1,size(current_fil_points)
                        current_fil_points(k)%coo(1:2) = current_boxer_db%point(k)%coo(1:2)
                    enddo
                else
                    number_of_waypoints = 0
                    do point_counter=1,size(current_boxer_db%point)
                        if (current_boxer_db%point(point_counter)%filament_number .eq. j) then
                            number_of_waypoints = number_of_waypoints + 1
                        endif
                    enddo
                    if (allocated(current_fil_points)) deallocate(current_fil_points)
                    allocate(current_fil_points(number_of_waypoints))
                    k = 0
                    do point_counter=1,size(current_boxer_db%point)
                        if (current_boxer_db%point(point_counter)%filament_number .eq. j) then
                            k = k +1
                            current_fil_points(k)%coo(1:2) = current_boxer_db%point(point_counter)%coo(1:2)
                        endif
                    enddo
                endif


                ! Initialise a filament - this includes a spline fit - since we're not giving way points (just points),
                ! this will be a parabolic spline, and the psi and theta angles will be worked out from it.
                call filament_init( fils(filament_num),current_fil_points,film_in=current_film,crossovers=wps_are_crossovers%value,&
                                    helix=helix,id=filament_num, &
                                    add_extra_fal_wps=add_extra_first_last_wps%value,   &
                                    fal_wps_are_crossovers=fal_wps_are_crossovers%value, &
                                    max_radius=helix_max_radius%value, &
                                    fal_wps_in_threed=fal_wps_in_threed%value, &
                                    microtubule=microtubule%value, mt_pf_number=mt_pf_number%value,mt_rise=mt_rise%value)


                ! Diagnostic - print filament info
                if (debug .or. this_program%IsVerbose()) call fils(filament_num)%PrintInfo()

                ! Diagnostic - write out waypoint boxer files
                if (write_filament_traces%value) then
                    call filename_from_wild(filament_trace_files%value,film_num,filament_trace_filename)
                    call filename_from_wild(filament_trace_filename,fils(filament_num)%id,filament_trace_filename)
                    call filename_remove_extension(filament_trace_filename,temp_filename)
                    filament_trace_filename = trim(adjustl(temp_filename))//'_wp.box'
                    call filament_boxer_db_to_disk(fils(filament_num),filament_trace_filename,with_trace=.false.)
                endif

            enddo
            endif

        enddo
        call my_progress_bar%Finish()
    endif

    ! Diagnostic message
    print '(a,i0,a)', '**info(mode_filament_startup): a total of ', filament_num, ' filaments were initialised'


    !
    ! Now that we have an array of initialised filaments, we can write a parameter file to disk
    !
    call filaments_to_frealix_par(fils,output_parameter_file%value)


    !
    !   Here, start work on calculating a 3d reconstruction
    !
!    if (calc_3d_reconstruction%value) then
!
!        !
!        ! Initialise the interpolation scheme
!        !
!        call interpolation_scheme_init(interpol_scheme,'LINEAR','RECT',2)
!
!        ! Initialise a 3d reconstruction - we set its dimensions as a cube of same dimension as the box size given by the user.
!        !! todo: think carefully about how to set the mask parameters. In test images with defocus 15000a, anything larger than 0.7
!        !!       was dangerous because it left visible artefacts from the non-recovery of PSF-delocalised information.
!        new_3drec%logical_dimensions(1:3) = fmc_threed_size(interpol_scheme,helix_max_radius%value/pixel_size%value)
!        write(*,'(a,3(i4,1x))') '**info(mode_filament_startup): setting 3d reconstruction box size to ', &
!                                new_3drec%logical_dimensions
!        call threed_init(   new_3drec,                                          &
!                            mask_rad            =   new_3drec%logical_dimensions(1)/2.0*0.99,  &
!                            mask_falloff        =   new_3drec%logical_dimensions(1)/2.0*0.1,   &
!                            interpol_scheme     =   interpol_scheme)
!
!        do i=1,size(fils)
!            !
!            ! Message for user
!            !
!            write(*,'(/3a,i4,a,i4,a,i4,a)') '**info(mode_filament_startup): ', myTime%DateAndTimeAsString(), &
!                                            ' Reconstructing filament ', fils(i)%id, ' (', i, ' of ', size(fils), ')'
!            !
!            ! Will we need to read a new film image from disk?
!            !
!            if (i .eq. 1) then
!                new_film    =   .true.
!            else if (fils(i)%film%GetFilename() .ne. fils(I-1)%film%GetFilename()) then
!                new_film    =   .true.
!            else
!                new_film    =   .false.
!            endif
!
!            !
!            ! If necessary, read a new film image from disk
!            !
!            if (new_film) then
!                call film_file%init(fils(i)%film%GetFilename())
!                call current_micrograph_frames(1)%ReadFromDisk(film_file)
!                call film_file%close()
!                ! Flip contrast if necessary
!                if (reverse_film_contrast%value) then
!                    current_micrograph_frames(1)%real_values = current_micrograph_frames(1)%real_values * (-1.0)
!                endif
!                ! High-pass filter the film image to bring average density to 0.0 and remove density ramps
!                film_hp_param =   pixel_size%value / (3.0 * helix_max_radius%value)
!                call current_micrograph_frames(1)%ApplyBandPassFilter(film_hp_param,0.0,film_hp_param/2.0)
!            endif
!
!            call filament_calc_threed(fils(i),current_micrograph_frames,new_3drec,finalise=.false.)
!        enddo
!        call threed_eval_reconstruction(new_3drec,new_3dvol)
!        ! Write to disk
!        call threed_write_to_disk(new_3drec,output_3d_filename%value)
!        call threed_file%init(  output_3d_filename%value,    &
!                                dim_x=new_3dvol%logical_dimensions(1),  &
!                                dim_y=new_3dvol%logical_dimensions(2),  &
!                                dim_z=new_3dvol%logical_dimensions(3))
!        call new_3dvol%WriteToDisk(threed_file)
!        call threed_file%close()
!        !call new_3dvol%WriteToDisk(output_3d_filename%value)
!    endif

    write(*,'(/2a/)') '**info(mode_filament_startup): all done. Output parameter file: ', trim(adjustl(output_parameter_file%value))


end subroutine mode_filament_startup_slave

end module mode_filament_startup

