module mode_frealign_stack_to_threed
    ! Modules
    use modes
    use rtps

    implicit none
    type(mode) :: frealign_stack_to_threed

    type(rtp_char), target  ::  frealign_par_filename  =               &
        rtp_char(   value='input.par',                  &
                    keyword='FREALIGN_PAR_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  stack_filename  =               &
        rtp_char(   value='input.mrc',                  &
                    keyword='STACK_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  threed_filename  =               &
        rtp_char(   value='output.mrc',                  &
                    keyword='THREED_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  model_threed_filename  =               &
        rtp_char(   value='model.mrc',                  &
                    keyword='MODEL_THREED_FILENAME',           &
                    description='Model 3d needed for fom filtering')

    type(rtp_char), target  ::  symmetry  =               &
        rtp_char(   value='C1',                  &
                    keyword='SYMMETRY',           &
                    description='Point group symmetry using Sch�nflies notation')


    type(rtp_real), target  ::  twist_per_subunit  =               &
        rtp_real(   value=0.0,                  &
                    keyword='TWIST_PER_SUBUNIT',           &
                    description='In degrees')


    type(rtp_real), target  ::  rise_per_subunit  =               &
        rtp_real(   value=0.0,                  &
                    keyword='RISE_PER_SUBUNIT',           &
                    description='In Angstroems')


    type(rtp_intg), target  ::  symmetry_axial  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_AXIAL',           &
                    description='Rotational symmetry redundancy about the helical axis')


    type(rtp_intg), target  ::  symmetry_perp  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_PERP',           &
                    description='Rotational symmetry redundancy perpendicular to the helical axis (e.g. 2 for dna)')


    type(rtp_real), target  ::  asym_unit_mw  =               &
        rtp_real(   value=0.0,                  &
                    keyword='ASYM_UNIT_MW',           &
                    description='Expected molecular weight (Da) of the asymetric unit')


    type(rtp_real), target  ::  helix_max_radius  =               &
        rtp_real(   value=0.0,                  &
                    keyword='HELIX_MAX_RADIUS',           &
                    description='Maximum outer radius of the helix, in Angstroms')


    type(rtp_intg), target  ::  num_subunits_per_image  =               &
        rtp_intg(   value=1,                  &
                    keyword='NUM_SUBUNITS_PER_IMAGE',           &
                    description='For each image in the stack, how many subunits should be added to the 3d reconstruction')


    type(rtp_real), target  ::  pixel_size  =               &
        rtp_real(   value=1.0,                  &
                    keyword='PIXEL_SIZE',           &
                    description='Pixel size (Angstroms)')


    type(rtp_real), target  ::  accel_voltage  =               &
        rtp_real(   value=200.0,                  &
                    keyword='ACCEL_VOLTAGE',           &
                    description='Acceleration voltage (kV)')


    type(rtp_real), target  ::  amplitude_contrast  =               &
        rtp_real(   value=0.07,                  &
                    keyword='AMPLITUDE_CONTRAST',           &
                    description='Amplitude contast (fraction)')


    type(rtp_real), target  ::  cs  =               &
        rtp_real(   value=2.3,                  &
                    keyword='CS',           &
                    description='Spherical aberation (mm)')


    type(rtp_logi), target  ::  apply_fom_filter  =               &
        rtp_logi(   value=.true.,                  &
                    keyword='APPLY_FOM_FILTER',           &
                    description='')


    type(rtp_logi), target  ::  apply_helical_symmetry  =               &
        rtp_logi(   value=.true.,                  &
                    keyword='APPLY_HELICAL_SYMMETRY',           &
                    description='')

    type(rtp_logi), target  ::  write_matching_projections  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='WRITE_MATCHING_PROJECTIONS',           &
                    description='Output a stack of matching projections generated from the input model 3D')

contains

subroutine mode_frealign_stack_to_threed_init_and_run
    implicit none
    ! Intialise the mode
    call frealign_stack_to_threed%init(command='FREALIGN_STACK_TO_THREED',subroutine=mode_frealign_stack_to_threed_slave,   &
                                        short_description='Add description here')

    ! Add RTPs to the mode
    call frealign_stack_to_threed%add_rtp(frealign_par_filename)
    call frealign_stack_to_threed%add_rtp(stack_filename)
    call frealign_stack_to_threed%add_rtp(threed_filename)
    call frealign_stack_to_threed%add_rtp(model_threed_filename)
    call frealign_stack_to_threed%add_rtp(symmetry)
    call frealign_stack_to_threed%add_rtp(twist_per_subunit)
    call frealign_stack_to_threed%add_rtp(rise_per_subunit)
    call frealign_stack_to_threed%add_rtp(symmetry_axial)
    call frealign_stack_to_threed%add_rtp(symmetry_perp)
    call frealign_stack_to_threed%add_rtp(asym_unit_mw)
    call frealign_stack_to_threed%add_rtp(helix_max_radius)
    call frealign_stack_to_threed%add_rtp(num_subunits_per_image)
    call frealign_stack_to_threed%add_rtp(pixel_size)
    call frealign_stack_to_threed%add_rtp(accel_voltage)
    call frealign_stack_to_threed%add_rtp(amplitude_contrast)
    call frealign_stack_to_threed%add_rtp(cs)
    call frealign_stack_to_threed%add_rtp(apply_fom_filter)
    call frealign_stack_to_threed%add_rtp(apply_helical_symmetry)
    call frealign_stack_to_threed%add_rtp(write_matching_projections)

    ! Run the mode
    call frealign_stack_to_threed%run()


end subroutine mode_frealign_stack_to_threed_init_and_run

subroutine mode_frealign_stack_to_threed_slave
    use images
    use single_particles
    use point_symmetries
    use frealign_parameters
    use threeds
    use ui, only : interval_print
    use helices
    use rotation_matrices
    use images_helices
    use parsers, only: path_from_filename, filename_from_wild
    use images_threed, only : extract_slice
    use images_ctf, only : image_apply_ctf
    use interpolation, only : interpolation_scheme_init
    use UsefulFunctions, only : IsOdd
    use ImageFiles
    use InterpolatingWindows
    use runtime_parameters, only : filename_len
    implicit none
    ! Variables
    type(image)                             ::  vol, slice, proj, model_3d
    type(threed)                            ::  rec3d_current, rec3d
    type(imagefile)                         ::  stack,threed_file, matching_projs, matching_boxes, model_3d_file
    !type(imagefile)                         ::  slices_xy_file, slices_yz_file
    type(single_particle),  allocatable     ::  sp(:)
    type(point_symmetry_symbol)             ::  sym_symbol
    type(helical_symmetry)                  ::  helix
    type(window)                            ::  extract_interpol_window
    type(interpolation_scheme)              ::  interpol
    !
    integer                                 ::  i,stack_dim(3)
    integer                                 ::  current_subunit, subunit_start, subunit_finish
    real                                    ::  mat(4,4)
    real                                    ::  extra_shift_xy(2)
    type(euler)                             ::  tmp_angles
    real                                    ::  h_x,h_y,shift_along_h_axis
    logical,        parameter               ::  debug   =   .false.
    real(kind=8)                            ::  AVERAGE_SCORE
    real                                    ::  bfactor
    logical                                 ::  new_filament, last_in_filament
    type(image)                             ::  rec3d_current_vol, rec3d_current_vol_slice
    character(len=filename_len)             ::  temp_filename
    integer                                 ::  first_loc,last_loc, current_output_loc


    ! Start work

    ! If we need to insert each image several times (because there are >1 helical subunits per image),
    ! let's work out some details now
    if (IsOdd(num_subunits_per_image%value)) then
        subunit_finish = (num_subunits_per_image%VALUE-1)/2
        subunit_start  = - subunit_finish
    else
        subunit_finish = num_subunits_per_image%value/2
        subunit_start  = - subunit_finish + 1
    endif

    ! Override with ugly frealign way
    subunit_start = - nint(num_subunits_per_image%value/2.0)
    subunit_finish =  nint(num_subunits_per_image%value/2.0)
    if (SUBUNIT_FINISH-SUBUNIT_START+1 .gt. num_subunits_per_image%value) subunit_finish = subunit_finish - 1
    if (SUBUNIT_FINISH-SUBUNIT_START+1 .gt. num_subunits_per_image%value) subunit_start = subunit_start + 1


    if (debug) write(*,'(a,2(i0,1x))') '**debug(mode_frealign_stack_to_threed): subunit start, finish = ', &
                                        subunit_start, subunit_finish

    !
    ! Initialise the average helical parameters
    !
    if (apply_helical_symmetry%value) then
        call helical_symmetry_init(helix,rise_per_subunit%value/pixel_size%value,twist_per_subunit%value,    &
                                    helix_max_radius%value/pixel_size%value, &
                                    symmetry_axial%value,symmetry_perp%value)
    endif

    ! Initialise interpolation
    interpol = default_interpol_scheme()
    call extract_interpol_window%AllocateWindow(interpol%window_width,comp=.true.,threed=.true.)


    ! Grab values from input Frealign parameter file
    sp = frealign_par_to_sp(frealign_par_filename%value)
    if (size(sp) .lt. 1) then
        write(*,'(a,i0)') '**info(mode_frealign_stack_to_threed): number of single particles = ', size(sp)
        call terminate('Fatal error in MODE_FREALIGN_STACK_TO_THREED')
    endif
    ! Fill in the other values
    sp%ctf%cs = cs%value
    sp%ctf%ampl_cont = amplitude_contrast%value
    sp%ctf%wl = akv_to_wl(accel_voltage%value)
    ! Convert ctf parameters to pixel units
    do i=1,size(sp)
        call ang_to_pix(sp(i)%ctf,pixel_size%value)
    end do
    ! Convert scores from phase residuals to cross-correlation scores, and compute average score
    average_score = 0.0D0
    do i=1,size(sp)
        sp(i)%score = cos(radians(sp(i)%score))
        average_score = average_score + sp(i)%score
    enddo
    average_score = average_score / real(size(sp),kind=8)
    write(*,'(a,F0.4)') '**info(mode_frealign_stack_to_threed): average score = ', average_score

    !
    ! In the helical case, if we have a large enough shift along the helical axis,
    ! we may be able to rotate the single particle by an integer number of
    ! helical repeat operators and so reduce the shift (with a concomittent change
    ! in phi)
    !
    if (.false.) then   !!!!!! testing testing testing !!!!
    if (helix%rise .ne. 0.0) then
        do i=1,size(sp)
            ! Compute the shift componenent along the helical axis
            h_x = -COS(radians(sp(i)%angles%psi))
            h_y =  sin(radians(sp(i)%angles%psi))
            shift_along_h_axis =    sp(i)%shift_x *  h_x + &
                                    sp(i)%shift_y *  h_y
            shift_along_h_axis = shift_along_h_axis / sin(radians(sp(i)%angles%theta))
            ! In number of h repeats
            if (debug) write(*,'(a,F0.3)') '**debug(mode_frealign_stack_to_threed): shift along axis (in number of h repeats) = ',&
                                             shift_along_h_axis/ helix%rise
            shift_along_h_axis =    nint(shift_along_h_axis / helix%rise)
            !

            ! Modify the shifts & phi angles accordingly
            sp(i)%shift_x    = sp(i)%shift_x    - shift_along_h_axis * h_x * helix%rise * sin(radians(sp(i)%angles%theta))
            sp(i)%shift_y    = sp(i)%shift_y    - shift_along_h_axis * h_y * helix%rise * sin(radians(sp(i)%angles%theta))
            sp(i)%angles%phi = sp(i)%angles%phi + shift_along_h_axis * helix%twist
            sp(i)%angles%phi = degrees_m180_to_180(sp(i)%angles%phi)
        end do
    endif
    endif
    !!!!!! testing

    ! Calculate symmetry matrices
    sym_symbol%symbol = trim(symmetry%value)

    ! Initialize imagefile object to open stack
    call stack%init(stack_filename%value)
    call stack%PrintInfo()
    stack_dim = stack%GetDimensions()

    ! Initialise output stacks for matching projections
    if (write_matching_projections%value) then
        call matching_projs%init('dbg_match_proj.mrc',dim_x=stack_dim(1),dim_y=stack_dim(2),pixel_size=pixel_size%value)
        call model_3d_file%init(model_threed_filename%value)
        call model_3d_file%PrintInfo()
        call model_3d%ReadFromDisk(model_3d_file,read_volume_not_slice=.true.)
        call model_3d_file%close()
        call model_3d%PrintInfo()
        call matching_boxes%init('dbg_match_boxes.mrc',dim_x=stack_dim(1),dim_y=stack_dim(2),  &
                                pixel_size=pixel_size%value)
    endif

    ! Check whether we have expected number of images
    if (stack_dim(3) .ne. size(sp)) then
        write(*,'(a,i5)') '**warning(mode_frealign_stack_to_threed): number of particles described by parameter file = ', size(sp)
        write(*,'(a,i5)') '                                          number of images in input stack = ', stack_dim(3)
        !call terminate('Fatal error in MODE_FREALIGN_STACK_TO_THREED')
    endif

    ! Check images in the stack are square
    if (stack_dim(1) .ne. stack_dim(2)) then
        write(*,'(a,2i4)') '**error(mode_frealign_stack_to_threed): input stack images are not square. ', stack_dim(1), stack_dim(2)
        call terminate('Fatal error in MODE_FREALIGN_STACK_TO_THREED')
    endif

    ! We're ignoring shift_x and shift_y
    ! write(*,'(a)') '**info(mode_frealign_stack_to_threed): x and y shifts from frealign parameter file are ignored'

    ! Initialise & memory for the threed object
    rec3d%logical_dimensions(1:3) = stack_dim(1)
    rec3d_current%logical_dimensions(1:3) = stack_dim(1)
    call init(  rec3d,rec3d%logical_dimensions(1)/2.0*0.9,rec3d%logical_dimensions(1)/2.0*0.1, &
                'LINEAR','RECT',2,SYM_SYMBOL=sym_symbol, &
                helix=helix,helix_max_rad=helix_max_radius%value/pixel_size%value,  &
                asym_unit_mw=asym_unit_mw%value,pixel_size=pixel_size%value,        &
                fom_filter=apply_fom_filter%value)
    call init(  rec3d_current,rec3d_current%logical_dimensions(1)/2.0*0.9,rec3d_current%logical_dimensions(1)/2.0*0.1,&
                'LINEAR','RECT',2,SYM_SYMBOL=sym_symbol, &
                helix=helix,helix_max_rad=helix_max_radius%value/pixel_size%value,  &
                asym_unit_mw=asym_unit_mw%value,pixel_size=pixel_size%value,        &
                fom_filter=apply_fom_filter%value)

    ! Prepare the slice image
    ! slice%logical_dimensions(1:2) = stack_dim(1)
    ! slice%logical_dimensions(3) = 1
    ! slice%type = 'REAL'
    ! slice%object_is_centered_in_box = .TRUE.
    ! call allocate(slice)


    ! Insert slices
    first_loc = 1    !1
    last_loc = 133    !size(sp)
    first_loc = 1
    last_loc = size(sp)
    current_output_loc = 0
    do i=first_loc,last_loc  !1,size(sp)
        if (interval_print(i,size(sp)) .or. .true.) then
            write(*,'(2(A,i6),a,3(F7.2,1x),a,2(F7.2,1x))') '**info(mode_frealign_stack_to_threed): inserting stack image # ', &
                                                            i, ' of ', size(sp), ' with angles ', sp(i)%angles, &
                                                            ' and shifts ', sp(i)%shift_x, sp(i)%shift_y
        endif
        call slice%ReadFromDisk(stack,wanted_location=i)
        ! Are we on a new filament?
        new_filament = .true.
        if (i .gt. 1) NEW_FILAMENT = sp(i)%film_num .ne. sp(I-1)%film_num
        if (i .eq. first_loc) new_filament = .true.
        ! Is this the last image from the current filament
        last_in_filament = .true.
        if (i .lt. size(sp)) last_in_filament = sp(i)%film_num .ne. sp(I+1)%film_num
        if (i .eq. last_loc) last_in_filament = .true.
        ! If we're starting a new filament, wipe the current 3d
        if (new_filament) call threed_wipe(rec3d_current)
        ! Insert all subunits found in the image
        do current_subunit=subunit_start,subunit_finish
            !write(*,'(a,i0)') '**debug(mode_frealign_stack_to_threed): current_subunit = ', current_subunit
            ! Work out the additional shift in xy and change in phi
            mat = euler_to_rotmat(sp(i)%angles%phi,sp(i)%angles%theta,sp(i)%angles%psi,reverse=.false.)
            extra_shift_xy(1:2) = mat(1:2,3) * real(current_subunit)*helix%rise
            tmp_angles = sp(i)%angles
            tmp_angles%phi = tmp_angles%phi + real(current_subunit)*helix%twist
            !write(*,'(a,3(f0.5,1x))') '**debug(mode_frealign_stack_to_threed): dphi, dx, dy = ', &
            !                            real(current_subunit)*helix%twist, extra_shift_xy
            ! Insert image into 3d
            bfactor  = bfactor_from_score(sp(i)%score,real(average_score))
            !write(*,'(a,F0.4)') '**debug(mode_frealign_stack_to_threed): bfactor = ', bfactor
            call threed_add_slice(  rec3d_current,slice,sp(i)%ctf,tmp_angles,   &
                                    shift_x = sp(i)%shift_x - extra_shift_xy(1),  &
                                    shift_y = sp(i)%shift_y - extra_shift_xy(2),   &
                                    bfactor=bfactor)
            if (current_subunit .eq. 0) then
                ! Write out matching projections
                if (write_matching_projections%value) then
                    current_output_loc = current_output_loc + 1
                    ! Compute the projection
                    if (model_3d%IsInRealSpace()) call model_3d%ForwardFFT()
                    if (model_3d%ObjectIsCenteredInBox()) call model_3d%SwapRealSpaceQuadrants()
                    call extract_slice( model_3d,proj,   &
                                        phi=tmp_angles%phi,theta=tmp_angles%theta,psi=tmp_angles%psi,   &
                                        interpol_scheme=interpol, &
                                        win=extract_interpol_window, &
                                        x_shift=-(sp(i)%shift_x - extra_shift_xy(1)),  &
                                        y_shift=-(sp(i)%shift_y - extra_shift_xy(2)) )
                    call proj%SwapRealSpaceQuadrants()
                    call image_apply_ctf(proj,sp(i)%ctf)
                    call proj%BackwardFFT()
                    ! Write out the projection
                    call proj%WriteToDisk(matching_projs,wanted_location=current_output_loc)
                    ! Write out the slice
                    if (slice%IsInRealSpace()) call slice%ForwardFFT()
                    call slice%ApplyBFactor(   bfactor=200.0/(pixel_size%value**2),   &
                                                lp_cosine=.false.,lp_cosine_freq=pixel_size%value/12.0, &
                                                lp_cosine_falloff=pixel_size%value/50.0)
                    call slice%BackwardFFT()
                    call slice%WriteToDisk(matching_boxes,wanted_location=current_output_loc)
                endif
            endif
        enddo   ! End of loop over subunits in the current image
        ! If this was the last image for this filament, add current 3d to the total 3d
        ! and write out the central slice of the current 3d to disk
        if (last_in_filament) then
            call threed_add(rec3d,rec3d_current,rec3d)
            call threed_eval_reconstruction(rec3d_current,rec3d_current_vol)
            ! Get xy slice
            call rec3d_current_vol%GetSlice(rec3d_current_vol_slice,3,rec3d_current_vol%logical_dimensions(3)/2+1)
            temp_filename = path_from_filename(threed_filename%value)
            temp_filename = trim(adjustl(temp_filename))//'3drec_slice_xy_??????.mrc'
            call filename_from_wild(temp_filename,sp(i)%film_num,temp_filename)
            call rec3d_current_vol_slice%WriteToDisk(temp_filename,pixel_size=pixel_size%value)
            ! Get yz slice
            call rec3d_current_vol%GetSlice(rec3d_current_vol_slice,1,rec3d_current_vol%logical_dimensions(1)/2+1)
            temp_filename = path_from_filename(threed_filename%value)
            temp_filename = trim(adjustl(temp_filename))//'3drec_slice_yz_??????.mrc'
            call filename_from_wild(temp_filename,sp(i)%film_num,temp_filename)
            call rec3d_current_vol_slice%WriteToDisk(temp_filename,pixel_size=pixel_size%value)
        endif
    enddo

    ! Close stack file
    call stack%close()


    ! Close the matching projections file
    if (write_matching_projections%value) then
        call matching_projs%PrintInfo()
        call matching_projs%close()
        call matching_boxes%PrintInfo()
        call matching_boxes%close()
    endif

    ! Write the 3d to disk
    call threed_write_to_disk(rec3d,threed_filename%value)

    ! Get the final 3d
    print '(a)', '**info(mode_frealign_stack_to_threed): finalising 3D'
    call threed_eval_reconstruction(rec3d,vol,cylinder_mask=.true.)
    if (apply_helical_symmetry%value) then
        write(*,'(a)') '**info(mode_frealign_stack_to_threed): imposing helical symmetry'
        call image_impose_helical_symmetry_new(vol,helix,rec3d%interpol_scheme)
    endif
    print '(a)', '**info(mode_frealign_stack_to_threed): nearly done'



    call threed_file%init(  threed_filename%value, &
                            dim_x=vol%logical_dimensions(1),DIM_Y=vol%logical_dimensions(2),DIM_Z=vol%logical_dimensions(3),&
                            pixel_size=pixel_size%value)
    call vol%WriteToDisk(threed_file)
    call threed_file%PrintInfo()
    call threed_file%close()

end subroutine mode_frealign_stack_to_threed_slave

    !>  \brief  return the b factor that should be applied to an image before it is inserted into a 3d reconstruction, given its score after refinement, and the average score of the dataset
    real function bfactor_from_score(score,average_score) result(bfactor)
        use runtime_parameters, only : pbc
        use rotation_matrices, only : degrees
        implicit none
        ! arguments
        real,       intent(in)      ::  score           !<  score of the image after refinement
        real,       intent(in)      ::  average_score   !<  average score of all images after refinement
        ! private variables
        logical,    parameter       ::  debug   =   .false.

        ! start work
        if (pbc .eq. 0.0) then
            bfactor = 0.0
        else
            if (debug) write(*,'(a,3(f0.4,1x))') '**debug(bfactor_from_score): average score, score, pbc: ', &
                                                degrees(acos(average_score)), degrees(acos(score)), pbc
            ! we want to return a negative b factor if the image has a better-than-average score
            bfactor = (degrees(acos(score)) - degrees(acos(average_score))) / pbc
            bfactor = max(bfactor,-2.0) ! a la frealign, we do not allow very negative b factors
            bfactor = bfactor * 16. ! this factor is there to match frealign. see notes of 29-apr-2012.
        endif

        if (debug) then
            write(*,'(a,3(f0.4,1x))') '**debug(bfactor_from_score): average score, score, bfactor: ', average_score, score, bfactor
        endif

    end function bfactor_from_score

end module mode_frealign_stack_to_threed
