!>  \brief  Mode to impose helical symmetry onto an input image
module mode_hextend
    ! Modules
    use modes
    use rtps
    implicit none
    type(mode) :: hextend

    type(rtp_char), target  ::  input_filename  =               &
        rtp_char(   value='test_in.mrc',                  &
                    keyword='input_filename',           &
                    description='')


    type(rtp_char), target  ::  output_filename  =               &
        rtp_char(   value='test_out.mrc',                  &
                    keyword='output_filename',           &
                    description='')


    type(rtp_real), target  ::  twist  =               &
        rtp_real(   value=0.0,                  &
                    keyword='twist_per_subunit',           &
                    description='Degrees')


    type(rtp_real), target  ::  rise  =               &
        rtp_real(   value=0.0,                  &
                    keyword='rise_per_subunit',           &
                    description='Angstroms')


    type(rtp_real), target  ::  pixel_size  =               &
        rtp_real(   value=0.0,                  &
                    keyword='PIXEL_SIZE',           &
                    description='Angstroms')


    type(rtp_real), target  ::  max_rad  =               &
        rtp_real(   value=0.0,                  &
                    keyword='helix_max_radius',           &
                    description='Maximum radius (Angstroms)')


    type(rtp_intg), target  ::  num_central_slices  =               &
        rtp_intg(   value=1,                  &
                    keyword='NUM_CENTRAL_SLICES',           &
                    description='')


    type(rtp_intg), target  ::  symmetry_axial  =               &
        rtp_intg(   value=1,                  &
                    keyword='symmetry_axial',           &
                    description='')


    type(rtp_intg), target  ::  symmetry_perp  =               &
        rtp_intg(   value=1,                  &
                    keyword='symmetry_perp',           &
                    description='')

    type(rtp_intg), target  ::  helical_starts  =               &
        rtp_intg(   value=1,                  &
                    keyword='num_helical_starts',           &
                    description='Number of helical starts')

     type(rtp_logi), target  ::  stack  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='stack',           &
                    description='The assembly is a stack rather than a true helix')

    type(rtp_logi), target  ::  new_algorithm  =               &
        rtp_logi(   value=.true.,                  &
                    keyword='new_algorithm',           &
                    description='New algorithm should be faster.')

    type(rtp_logi), target  ::  write_out_asym_unit  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='write_out_asym_unit',           &
                    description='Dump a 3d density to disk with 1 within and 0 outside the asymetric unit')

    type(rtp_char), target  ::  asym_unit_filename  =               &
        rtp_char(   value='asym_unit.mrc',                  &
                    keyword='asym_unit_filename',           &
                    description='If an asymetric unit 3d density is written out, use this filename')


    type(rtp_logi), target  ::  helical_rise_not_resolved  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='helical_rise_not_resolved',           &
                    description='The helical rise is not resolved - subunits are smeared along the helical start')

    type(rtp_logi), target  ::  align_to_lattice  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='align_to_lattice',           &
                    description='Shift and rotate density so that the edges of the subunit density '//&
                                'coincide with the edges of the unit cell')
contains

subroutine mode_hextend_init_and_run
    ! Intialise the mode
    call hextend%init(command='HEXTEND',subroutine=mode_hextend_slave,short_description='Add description here')

    ! Add RTPs to the mode
    call hextend%add_rtp(input_filename)
    call hextend%add_rtp(output_filename)
    call hextend%add_rtp(twist)
    call hextend%add_rtp(rise)
    call hextend%add_rtp(pixel_size)
    call hextend%add_rtp(max_rad)
    call hextend%add_rtp(num_central_slices)
    call hextend%add_rtp(symmetry_axial)
    call hextend%add_rtp(symmetry_perp)
    call hextend%add_rtp(helical_starts)
    call hextend%add_rtp(stack)
    call hextend%add_rtp(new_algorithm)
    call hextend%add_rtp(write_out_asym_unit)
    call hextend%add_rtp(asym_unit_filename)
    call hextend%add_rtp(helical_rise_not_resolved)
    call hextend%add_rtp(align_to_lattice)

    ! Run the mode
    call hextend%run()
end subroutine mode_hextend_init_and_run

subroutine mode_hextend_slave
    use runtime_parameters
    use images_helices
    use interpolation
    use imagefiles
    use interpolation
    use Globals
    !use dates_and_times
    implicit none
    ! Variables
    type(image)                 ::  imgin
    type(helical_symmetry)      ::  helix
    type(interpolation_scheme)  ::  interpol_scheme
    type(imagefile)             ::  asym_unit_file
    type(image)                 ::  asym_unit_vol
    type(helical_symmetry)      ::  rise_smearing_helix
    real                        ::  shift_phi, shift_z
    ! debug
    !integer          ::  my_date_start(8), my_date_finish(8)
    !type(DateAndTime)   ::  my_date
    logical,    parameter       ::  use_soft_h_boundary = .false.
    !


    ! Start work

    ! Setup interpolation scheme
    call interpolation_scheme_init(interpol_scheme,'LINEAR','RECT',2)
    !call interpolation_scheme_init(interpol_scheme,'SINC','HANN',8)

    ! Setup the helical symmetry operator
    call helical_symmetry_init(helix,rise%value/pixel_size%value,twist%value, &
                                max_rad%value/pixel_size%value, &
                                symmetry_axial%value,symmetry_perp%value,&
                                num_starts=helical_starts%value,stack=stack%value)

    call helical_symmetry_print_info(helix)
    call helical_symmetry_print_extra_info(helix)

    ! Read the image from disk
    call imgin%ReadFromDisk(input_filename%value,read_volume_not_slice=.true.)

    !
    if (helical_rise_not_resolved%value) then
        call helical_symmetry_init(rise_smearing_helix,1.0,twist%value/rise%value*pixel_size%value, &
                                                        max_rad%value/pixel_size%value, &
                                                        symmetry_axial%value,symmetry_perp%value, &
                                                        num_starts=helical_starts%value,stack=stack%value)
        call image_smear_asymetric_unit(imgin,helix,rise_smearing_helix,interpol_scheme,max_radius=max_rad%value)
        call imgin%WriteToDisk('dbg_smeared.mrc')
    endif

    ! Compute & write to disk the asymetric unit mask
    if (write_out_asym_unit%value) then
        call asym_unit_vol%Allocate(mould=imgin)
        asym_unit_vol%real_values = 1.0
        call image_mask_helical_subunit(asym_unit_vol,helix,max_radius=max_rad%value)
        call asym_unit_vol%WriteToDisk(asym_unit_filename%value,print_file_info=.true.,pixel_size=pixel_size%value)
    endif


    ! Symmetrise
    if (new_algorithm%value) then
        !my_date_start = my_date%DateAndTimeAsIntegers()
        if (use_soft_h_boundary) then
            if (align_to_lattice%value) then
                call image_align_to_helical_lattice(imgin,helix,0.0,max_rad%value/pixel_size%value,shift_phi,shift_z)
                write(*,'(a,f0.2,a,f0.2,a)') '**info(mode_hextend): density was rotated by ', shift_phi, &
                                                ' degrees and shifted by ', shift_z, ' pixels'
            endif
            call image_impose_helical_symmetry_new_soft(imgin,helix,interpol_scheme,max_radius=max_rad%value/pixel_size%value)
        else
            if (align_to_lattice%value) then
                call image_align_to_helical_lattice(imgin,helix,0.0,max_rad%value/pixel_size%value,shift_phi,shift_z)
                write(*,'(a,f0.2,a,f0.2,a)') '**info(mode_hextend): density was rotated by ', shift_phi, &
                                                ' degrees and shifted by ', shift_z, ' pixels'
            endif
            call image_impose_helical_symmetry_new(imgin,helix,interpol_scheme,max_radius=max_rad%value/pixel_size%value)
        endif
        !my_date_finish = my_date%DateAndTimeAsIntegers()
        !print *, '**time elapsed = ', my_date%SecondsBetweenDates(my_date_start,my_date_finish)
    else
        if (align_to_lattice%value) then
            call image_align_to_helical_lattice(imgin,helix,0.0,max_rad%value/pixel_size%value,shift_phi,shift_z)
            write(*,'(a,f0.2,a,f0.2,a)') '**info(mode_hextend): density was rotated by ', shift_phi, &
                                            ' degrees and shifted by ', shift_z, ' pixels'
        endif
        call image_impose_helical_symmetry(imgin,helix,interpol_scheme,num_central_slices=num_central_slices%value, &
                                            max_radius=max_rad%value/pixel_size%value,&
                                            progress_report=.true.)
    endif
    ! Write to disk
    call imgin%WriteToDisk(output_filename%value,pixel_size=pixel_size%value,print_file_info=.true.,delete_if_already_exists=.true.)


end subroutine mode_hextend_slave

end module mode_hextend
