!>  \brief  Mode to impose helical symmetry onto an input image
module mode_hmask
    ! Modules
    use modes
    use rtps
    implicit none
    type(mode) :: hmask

    type(rtp_char), target  ::  input_filename  =               &
        rtp_char(   value='test_in',                  &
                    keyword='INPUT_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  output_filename  =               &
        rtp_char(   value='test_out',                  &
                    keyword='OUTPUT_FILENAME',           &
                    description='')


    type(rtp_real), target  ::  twist  =               &
        rtp_real(   value=0.0,                  &
                    keyword='TWIST_PER_SUBUNIT',           &
                    description='')


    type(rtp_real), target  ::  rise  =               &
        rtp_real(   value=0.0,                  &
                    keyword='RISE_PER_SUBUNIT',           &
                    description='')


    type(rtp_real), target  ::  pixel_size  =               &
        rtp_real(   value=0.0,                  &
                    keyword='PIXEL_SIZE',           &
                    description='')


    type(rtp_real), target  ::  max_rad  =               &
        rtp_real(   value=0.0,                  &
                    keyword='HELIX_MAX_RADIUS',           &
                    description='Maximum radius (Angstroms)')


    type(rtp_intg), target  ::  symmetry_axial  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_AXIAL',           &
                    description='Axial symmetry (->number of helical starts)')


    type(rtp_intg), target  ::  symmetry_perp  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_PERP',           &
                    description='Perpendicular symmetry (1 or 2)')

    type(rtp_logi), target  ::  stack  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='stack',           &
                    description='The assembly is a stack rather than a true helix')


    type(rtp_real), target  ::  asym_unit_mw  =               &
        rtp_real(   value=0.0,                  &
                    keyword='ASYM_UNIT_MW',           &
                    description='Expected molecular weight (Da) of the asymetric unit')


    type(rtp_real), target  ::  mask_resolution  =               &
        rtp_real(   value=1.0,                  &
                    keyword='MASK_RESOLUTION',           &
                    description='In Angstroms, the maximum resolution of the mask')
contains

subroutine mode_hmask_init_and_run
    implicit none
    ! Intialise the mode
    call hmask%init(command='HMASK',subroutine=mode_hmask_slave,short_description='Add description here')

    ! Add RTPs to the mode
    call hmask%add_rtp(input_filename)
    call hmask%add_rtp(output_filename)
    call hmask%add_rtp(twist)
    call hmask%add_rtp(rise)
    call hmask%add_rtp(pixel_size)
    call hmask%add_rtp(max_rad)
    call hmask%add_rtp(symmetry_axial)
    call hmask%add_rtp(symmetry_perp)
    call hmask%add_rtp(stack)
    call hmask%add_rtp(asym_unit_mw)
    call hmask%add_rtp(mask_resolution)

    ! Run the mode
    call hmask%run()

end subroutine mode_hmask_init_and_run


subroutine mode_hmask_slave
    use runtime_parameters
    use images_helices
    use interpolation
    use imagefiles
    implicit none
    ! Variables
    type(image)             ::  imgin
    type(helical_symmetry)  ::  helix


    ! Start work

    ! Convert the maximum radius to pixels
    if (max_rad%value .ne. 0.0) then
        max_rad%value = max_rad%value / pixel_size%value
    endif

    ! Setup the helical symmetry operator
    call helical_symmetry_init(helix,rise%value/pixel_size%value,twist%value,max_rad%value, &
                                symmetry_axial%value,symmetry_perp%value, &
                                stack=stack%value)
    ! Read the image from disk
    call imgin%ReadFromDisk(input_filename%value,read_volume_not_slice=.true.)
    ! Mask
    call image_helix_mask(imgin,helix,asym_unit_mw%value,pixel_size%value,helix_max_rad=max_rad%value,  &
                            mask_resolution=pixel_size%value/mask_resolution%value)
    ! Write to disk
    call imgin%WriteToDisk(output_filename%value,pixel_size=pixel_size%value)

end subroutine mode_hmask_slave

end module mode_hmask
