module mode_pad
    ! Modules
    use modes
    use rtps
    implicit none
    type(mode) :: pad

    type(rtp_char), target  ::  input_filename  =               &
        rtp_char(   value='input.mrc',                  &
                    keyword='INPUT_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  output_filename  =               &
        rtp_char(   value='output.mrc',                  &
                    keyword='OUTPUT_FILENAME',           &
                    description='')


    type(rtp_intg), target  ::  new_dim  =               &
        rtp_intg(   value=256,                  &
                    keyword='NEW_DIM',           &
                    description='')
    type(rtp_intg), target  ::  new_dim_x  =               &
        rtp_intg(   value=256,                  &
                    keyword='NEW_DIM_X',           &
                    description='')
    type(rtp_intg), target  ::  new_dim_y  =               &
        rtp_intg(   value=256,                  &
                    keyword='NEW_DIM_Y',           &
                    description='')
    type(rtp_intg), target  ::  new_dim_z  =               &
        rtp_intg(   value=256,                  &
                    keyword='NEW_DIM_Z',           &
                    description='')


    type(rtp_logi), target  ::  real_space =    &
        rtp_logi(   value=.true.,               &
                    keyword='REAL_SPACE',       &
                    description='Resize the real-space image, rather than its Fourier transform')
contains

subroutine mode_pad_init_and_run
    implicit none
    ! Intialise the mode
    call pad%init(command='PAD',subroutine=mode_pad_slave,short_description='Resize an image')

    ! Add RTPs to the mode
    call pad%add_rtp(input_filename)
    call pad%add_rtp(output_filename)
    call pad%add_rtp(new_dim)
    call pad%add_rtp(new_dim_x)
    call pad%add_rtp(new_dim_y)
    call pad%add_rtp(new_dim_z)
    call pad%add_rtp(real_space)

    ! Run the mode
    call pad%run()
end subroutine mode_pad_init_and_run


subroutine mode_pad_slave
    use images
    use imagefiles
    implicit none
    ! Variables
    type(image)     ::  vol, output_vol
    type(imagefile) ::  input_file, output_file
    real            ::  output_pixel_size
    integer ::  dim(3)


    ! Start work

    ! Open input file, load data
    call input_file%init(input_filename%value)
    call vol%ReadFromDisk(input_file,read_volume_not_slice=.true.)
    call input_file%PrintInfo()
    output_pixel_size = input_file%GetPixelSize()
    call input_file%close()

    ! Work out output dimensions
    if (new_dim_x%set_by_user) then
        dim = [NEW_DIM_X%value,new_dim_y%value,new_dim_z%VALUE]
    else
        if (vol%logical_dimensions(3) .eq. 1) then ! The 2d case
            dim = [NEW_DIM%value,new_dim%value,1]
        else
            dim = [NEW_DIM%value,new_dim%value,new_dim%VALUE]
        endif
    endif

    ! Work out output pixel size
    if (.not. real_space%value) then
        output_pixel_size = output_pixel_size / dim(1) * vol%logical_dimensions(1)
    endif

    ! Resize
    write(*,'(2(A,3(I0,1x)))') '**info(mode_pad): resizing from ', vol%logical_dimensions, ' to ', dim
    if (.not. real_space%value) then
        write(*,'(a,F0.3)') '**info(mode_pad): resizing in Fourier space. New pixel size = ', output_pixel_size
    endif
    if (.not. real_space%value) call vol%ForwardFFT()
    call output_vol%Allocate(dims=dim)
    call vol%ClipInto(output_vol)
    if (.not. real_space%value) call output_vol%BackwardFFT()



    ! Write out to disk
    call output_file%init(output_filename%value,dim_x=dim(1),dim_y=dim(2),dim_z=dim(3),pixel_size=output_pixel_size)
    call output_vol%WriteToDisk(output_file)
    call output_file%PrintInfo()
    call output_file%close()



end subroutine mode_pad_slave

end module mode_pad
