!>  \brief  Add disorder to waypoints
module mode_randomize_waypoints
    use modes
    use rtps
    implicit none

    type(mode)  ::  randomize_waypoints

    type(rtp_char), target  ::  input_filename = &
                        rtp_char(   value = 'waypoints.ali',    &
                                    keyword = 'INPUT_FILENAME', &
                                    description = 'Filename for input waypoints')

    type(rtp_char), target  ::  output_filename = &
                        rtp_char(   value = 'waypoints_out.ali',    &
                                    keyword = 'OUTPUT_FILENAME', &
                                    description = 'Filename for output waypoints')

    type(rtp_real), target  ::  pixel_size = &
                        rtp_real(   value = 0.0,    &
                                    keyword = 'PIXEL_SIZE',  &
                                    description = 'Pixel size (Angstroms)')

    type(rtp_real), target  ::  std_x = &
                        rtp_real(   value = 0.0,    &
                                    keyword = 'STD_X',  &
                                    description = 'Standard deviation of Gaussian distribution of errors '//&
                                                  'introduced in x coordinate (Angstroms)')

    type(rtp_real), target  ::  std_y = &
                        rtp_real(   value = 0.0,    &
                                    keyword = 'STD_Y',  &
                                    description = 'Standard deviation of Gaussian distribution of errors '//&
                                                  'introduced in y coordinate (Angstroms)')

    type(rtp_real), target  ::  std_phi = &
                        rtp_real(   value = 0.0,    &
                                    keyword = 'STD_PHI',  &
                                    description = 'Standard deviation of Gaussian distribution of errors introduced in phi')

    type(rtp_real), target  ::  std_theta = &
                        rtp_real(   value = 0.0,    &
                                    keyword = 'STD_THETA',  &
                                    description = 'Standard deviation of Gaussian distribution of errors introduced in theta')

    type(rtp_real), target  ::  std_psi = &
                        rtp_real(   value = 0.0,    &
                                    keyword = 'STD_PSI',  &
                                    description = 'Standard deviation of Gaussian distribution of errors introduced in psi')
contains

subroutine mode_randomize_waypoints_init_and_run
    implicit none
    ! Intialise the mode
    call randomize_waypoints%init(command='RANDOMIZE_WAYPOINTS',subroutine=mode_randomize_waypoints_slave,  &
                                short_description='Modify waypoints by adding random noise')

    ! Add RTPs to the mode
    call randomize_waypoints%add_rtp(input_filename)
    call randomize_waypoints%add_rtp(output_filename)
    call randomize_waypoints%add_rtp(pixel_size)
    call randomize_waypoints%add_rtp(std_x)
    call randomize_waypoints%add_rtp(std_y)
    call randomize_waypoints%add_rtp(std_phi)
    call randomize_waypoints%add_rtp(std_theta)
    call randomize_waypoints%add_rtp(std_psi)

    ! Run the mode
    call randomize_waypoints%run()
end subroutine mode_randomize_waypoints_init_and_run

subroutine mode_randomize_waypoints_slave
    use coos, only : waypoints_add_disorder
    use filaments_base
    use frealix_parameters
    implicit none
    type(filament), allocatable ::  fils(:)
    integer ::  current_fil
    ! Parse the input waypoints file
    call filaments_from_frealix_par_noinit(fils,input_filename%value,pixel_size%value)
    ! Loop over filaments
    do current_fil=1,size(fils)
        ! Add noise to waypoints
        call waypoints_add_disorder(fils(current_fil)%wp,   &
                                    cumulative_disorder_std    =(/0.,0.,0.,0.,0.,0.,0.,0.,0./),   &
                                    non_cumulative_disorder_std=(/std_x%value,std_y%value,0.,   &
                                                                std_phi%value,std_theta%value,std_psi%value,0.,&
                                                                0.,0.,0.,0./))
    enddo
    ! Write out the new waypoints file
    call filaments_to_frealix_par(fils,output_filename%value)
    write(*,'(a)') '**info(mode_randomize_waypoints_slave): all done.'

end subroutine mode_randomize_waypoints_slave


end module mode_randomize_waypoints
