!>  \brief  Mode to impose helical symmetry onto an input image
module mode_rebuild_seam
    ! Modules
    use modes
    use rtps
    implicit none
    type(mode) :: rebuild_seam

    type(rtp_char), target  ::  input_file  =               &
        rtp_char(   value='test_in',                  &
                    keyword='INPUT_FILE',           &
                    description='')


    type(rtp_char), target  ::  output_file  =               &
        rtp_char(   value='test_out',                  &
                    keyword='OUTPUT_FILE',           &
                    description='')


    type(rtp_real), target  ::  twist  =               &
        rtp_real(   value=0.0,                  &
                    keyword='TWIST',           &
                    description='')


    type(rtp_real), target  ::  rise  =               &
        rtp_real(   value=0.0,                  &
                    keyword='RISE',           &
                    description='')


    type(rtp_real), target  ::  pixel_size  =               &
        rtp_real(   value=0.0,                  &
                    keyword='PIXEL_SIZE',           &
                    description='')


    type(rtp_real), target  ::  max_rad  =               &
        rtp_real(   value=0.0,                  &
                    keyword='MAX_RAD',           &
                    description='Maximum radius (Angstroms)')


    type(rtp_intg), target  ::  symmetry_axial  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_AXIAL',           &
                    description='')


    type(rtp_intg), target  ::  symmetry_perp  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_PERP',           &
                    description='')


    type(rtp_intg), target  ::  mt_pf_number  =               &
        rtp_intg(   value=0,                  &
                    keyword='MT_PF_NUMBER',           &
                    description='For microtubules only: number of protofilaments (e.g. 13)')


    type(rtp_intg), target  ::  mt_rise  =               &
        rtp_intg(   value=0,                  &
                    keyword='MT_RISE',           &
                    description='For microtubules only: how much further along the filament (distance measured in '//&
                                'tubulin subunits) the end of a MT_PF_NUMBER-element "washer" is compared to its '//&
                                'beginning. The 13-pf microtobules have a value of 3.')


    !type(rtp_real), target  ::  helix_max_radius  =               &
    !    rtp_real(   value=0.0,                  &
    !                keyword='',           &
    !                description='Maximum outer radius of the helix, in Angstroms')
contains

subroutine mode_rebuild_seam_init_and_run
    implicit none

    ! Intialise the mode
    call rebuild_seam%init(command='REBUILD_SEAM',subroutine=mode_rebuild_seam_slave,short_description='Add description here')

    ! Add RTPs to the mode
    call rebuild_seam%add_rtp(input_file)
    call rebuild_seam%add_rtp(output_file)
    call rebuild_seam%add_rtp(twist)
    call rebuild_seam%add_rtp(rise)
    call rebuild_seam%add_rtp(pixel_size)
    call rebuild_seam%add_rtp(max_rad)
    call rebuild_seam%add_rtp(symmetry_axial)
    call rebuild_seam%add_rtp(symmetry_perp)
    call rebuild_seam%add_rtp(mt_pf_number)
    call rebuild_seam%add_rtp(mt_rise)
    !call rebuild_seam%add_rtp(helix_max_radius)

    ! Run the mode
    call rebuild_seam%run()
end subroutine mode_rebuild_seam_init_and_run

subroutine mode_rebuild_seam_slave
    use runtime_parameters
    use images_helices
    use interpolation
    use imagefiles
    use filaments
    use filaments_microtubules, only : image_impose_microtubule_symmetry
    implicit none
    ! Variables
    type(image)             ::  imgin
    type(filament)          ::  fil
    type(helical_symmetry)  ::  helix


    ! Start work

    ! Conver the maximum radius to pixels
    if (max_rad%value .ne. 0.0) then
        max_rad%value = max_rad%value / pixel_size%value
    endif

    ! Setup the helical symmetry operator
    !call helix%init(rise%value,twist%value,symmetry_axial%value,symmetry_perp%value,pixel_size%value)
    call helical_symmetry_init(helix,rise%value/pixel_size%value,twist%value,max_rad%value/pixel_size%value, &
                                symmetry_axial%value,symmetry_perp%value)

    ! Set up a filament object. We don't do a full initialisation, to save time
    fil%microtubule = .true.
    fil%mt_pf_number = mt_pf_number%value
    fil%mt_rise = mt_rise%value
    fil%max_radius = max_rad%value
    fil%helix = helix

    ! Read the image from disk
    call imgin%ReadFromDisk(input_file%value)
    ! Symmetrise
    !call image_impose_helical_symmetry(imgin,helix,num_central_slices,linear,rect,2)
    !call image_impose_helical_symmetry(imgin,helix,num_central_slices,linear,unity_window,2,max_radius=max_rad%value)
    call image_impose_microtubule_symmetry(fil,imgin)
    ! Write to disk
    call imgin%WriteToDisk(output_file%value)

end subroutine mode_rebuild_seam_slave

end module mode_rebuild_seam
