module mode_split_par
!dec$ optimize:0
    ! Modules
    use runtime_parameters
    use modes
    use rtps
    implicit none
    type(mode) :: split_par

    type(rtp_char), target  ::  input_filename  =               &
        rtp_char(   value='input.par',                  &
                    keyword='INPUT_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  output_root_filename  =               &
        rtp_char(   value='output_???.par',                  &
                    keyword='OUTPUT_ROOT_FILENAME',           &
                    description='')


    type(rtp_intg), target  ::  number_of_groups  =               &
        rtp_intg(   value=50,                  &
                    keyword='NUMBER_OF_GROUPS',           &
                    description='')

    type(rtp_logi), target  ::  preserve_order_of_filaments  =               &
        rtp_logi (   value=.false.,                  &
                    keyword='preserve_order_of_filaments',           &
                    description='')

contains

subroutine mode_split_par_init_and_run
    implicit none
    ! Intialise the mode
    call split_par%init(command='SPLIT_PAR',subroutine=mode_split_par_slave,short_description='Add description here')

    ! Add RTPs to the mode
    call split_par%add_rtp(input_filename)
    call split_par%add_rtp(output_root_filename)
    call split_par%add_rtp(number_of_groups)
    call split_par%add_rtp(preserve_order_of_filaments)

    ! Run the mode
    call split_par%run()
end subroutine mode_split_par_init_and_run


subroutine mode_split_par_slave
    use frealix_parameters
    use filaments
    use parsers
    implicit none
    ! Variables
    type(filament), allocatable ::  filament_array(:), filament_group(:)
    integer,        allocatable ::  group_membership(:), members_per_group(:), members_per_group_already_assigned(:)
    integer     ::  i, j, ierr
    integer     ::  count_in_current_group
    integer     ::  current_fil, current_group
    character(len=filename_len) ::  group_filename


    ! Start work

    ! Read in filament array from file
    call filaments_from_frealix_par_noinit(filament_array,input_filename%value,1.0)

    if (preserve_order_of_filaments%value) then
        ! Loop over over filaments, assign group membership
        allocate(group_membership(size(filament_array)),members_per_group(number_of_groups%value))
        current_group = 0
        members_per_group = 0
        members_per_group_already_assigned = 0
        do current_fil=1,size(filament_array)
            if (current_group .eq. number_of_groups%value) then
                current_group = 1
            else
                current_group = current_group + 1
            endif
            group_membership(current_fil) = current_group
            members_per_group(current_group) = members_per_group(current_group) + 1
        enddo
    else
        ! Assign filaments to groups, but be smart about it
        call SplitArrayOfFilaments(filament_array,number_of_groups%value,group_membership,members_per_group)
    endif
    ! Loop over the group
    do i=1,number_of_groups%value
        ! Reallocate filament_group
        if (allocated(filament_group)) deallocate(filament_group)
        allocate(filament_group(members_per_group(i)),stat=ierr)
        if (ierr .ne. 0) call terminate('Memory allocation failed')
        ! Copy filaments over to the group
        count_in_current_group = 0
        do j=1,size(filament_array)
            if (group_membership(j) .eq. i) then
                !write(*,'(3(A,i))') '**debug(mode_split_par): adding filament ',  filament_array(j)%id, ' which has ', filament_array(j)%nwp, ' waypoints ', size(filament_array(j)%wp)
                count_in_current_group = count_in_current_group + 1
                filament_group(count_in_current_group) = filament_array(j)
                filament_group(count_in_current_group)%wp = filament_array(j)%wp
            endif
        enddo
        ! Work out filename for this group's parameter file
        call filename_from_wild(output_root_filename%value,i,group_filename)
        !
        call filaments_to_frealix_par(filament_group,group_filename)
    enddo

end subroutine mode_split_par_slave

end module mode_split_par
