!>  \brief  Create a particle stack from a frealix waypoints parameter file
!!
module mode_stack_from_par
    use modes
    use rtps
    implicit none

    type(mode) :: stack_from_par

    type(rtp_char), target  ::  par_filename  =               &
        rtp_char(   value='waypoints.ali',                  &
                    keyword='INPUT_PAR_FILENAME',           &
                    description='Parameter file')

    type(rtp_char), target  ::  output_frealign_par_filename  =               &
        rtp_char(   value='none',                  &
                    keyword='OUTPUT_FREALIGN_PAR_FILENAME',           &
                    description='Frealign parameter filename for the output')

    type(rtp_char), target  ::  output_frealix_par_filename  =               &
        rtp_char(   value='none',                  &
                    keyword='OUTPUT_FREALIX_PAR_FILENAME',           &
                    description='Frealix waypoints file matching the output frealign par file')

    type(rtp_char), target  ::  micrograph_filename_wild  =               &
        rtp_char(   value='micrograph???_bin8.mrc',                  &
                    keyword='FILM_FILES',           &
                    description='')

    type(rtp_intg), target  ::  number_of_frames_per_micrograph  =               &
        rtp_intg(   value=1,                  &
                    keyword='number_of_frames_per_micrograph',           &
                    description='If the input stack contains frames from dose-fractionated exposures, give the'//&
                                'number of frames. If not, give 1.')

    type(rtp_logi), target  ::  apply_dose_filter = &
        rtp_logi(   value=.false., &
                    keyword='apply_dose_filter', &
                    description='When summing frames from dose-fractionated exposures, filter each frames '//&
                                'as a function of the expected radiation damage from electron exposure')

    type(rtp_real), target  ::  exposure_per_frame = &
        rtp_real(   value=1.0, &
                    keyword='exposure_per_frame', &
                    description='For each frame of dose-fractionated exposures, the number of electrons '//&
                                'incident on the specimen per square Angstroms')


    type(rtp_char), target  ::  ctftilt_files  =               &
        rtp_char(   value='none',                  &
                    keyword='CTFTILT_FILES',           &
                    description='')


    type(rtp_char), target  ::  ctffind_files  =               &
        rtp_char(   value='none',                  &
                    keyword='CTFFIND_FILES',           &
                    description='')


    type(rtp_char), target  ::  reconstruction_filename  =               &
        rtp_char(   value='orig_vol.mrc',                  &
                    keyword='RECONSTRUCTION_FILENAME',           &
                    description='If requested, projections will be calculated from this 3d reconstruction')


    type(rtp_char), target  ::  stack_filename  =               &
        rtp_char(   value='stack.mrc',                  &
                    keyword='OUTPUT_STACK_FILENAME',           &
                    description='Filename for the stack of boxed particles')


    type(rtp_char), target  ::  ctf_stack_filename  =               &
        rtp_char(   value='stack_ctf.mrc',                  &
                    keyword='CTF_STACK_FILENAME',           &
                    description='Filename for the stack of ctf functions')


    type(rtp_char), target  ::  proj_filename  =               &
        rtp_char(   value='proj.mrc',                  &
                    keyword='PROJ_FILENAME',           &
                    description='Output stack filename')


    type(rtp_char), target  ::  ampl_stack_filename  =               &
        rtp_char(   value='stack_ampl.mrc',                  &
                    keyword='AMPL_STACK_FILENAME',           &
                    description='Output stack filename (amplitude spectra)')


    type(rtp_char), target  ::  ampl_ave_filename  =               &
        rtp_char(   value='stack_ampl_ave.mrc',                  &
                    keyword='AMPL_AVE_FILENAME',           &
                    description='Output average amplitude spectrum')


    type(rtp_real), target  ::  segment_length  =       &
        rtp_real(   value=120.,                         &
                    keyword='SEGMENT_LENGTH',           &
                    description='Length of filament segments to be extracted (Angstroms)')

    type(rtp_logi), target  ::  one_image_per_repeat   =   &
        rtp_logi(   value=.false.,      &
                    keyword='ONE_IMAGE_PER_REPEAT',    &
                    description='Output one image per helical repeat, rather than one per waypoint')


    type(rtp_real), target  ::  phi_angle  =               &
        rtp_real(   value=0.0,                  &
                    keyword='PHI_ANGLE',           &
                    description='Phi (helical twist) angle at which segments should be extracted')


    type(rtp_real), target  ::  phi_period  =               &
        rtp_real(   value=180.0,                  &
                    keyword='PHI_PERIOD',           &
                    description='')


    type(rtp_real), target  ::  pixel_size  =               &
        rtp_real(   value=1.17,                  &
                    keyword='PIXEL_SIZE',           &
                    description='Pixel size (Angstroms)')

    type(rtp_real), target  ::  magnification = &
        rtp_real(   value=60000.,   &
                    keyword='MAGNIFICATION', &
                    description='Magnification value for the output Frealign parameter file')


    type(rtp_real), target  ::  acceleration_voltage =  &
        rtp_real(   value=300.0,                        &
                    keyword='acceleration_voltage',     &
                    description='in kV')

    type(rtp_real), target  ::  spherical_aberration =  &
        rtp_real(   value=2.7,                          &
                    keyword='spherical_aberration',     &
                    description='in mm')

    type(rtp_real), target  ::  amplitude_contrast =    &
        rtp_real(   value=0.07,                         &
                    keyword='amplitude_contrast',       &
                    description='Fraction of amplitude contrast')


    type(rtp_logi), target  ::  center_images = &
        rtp_logi(   value=.false.,  &
                    keyword='CENTER_IMAGES',    &
                    description='Apply non-integer shift to boxed images so that they are centered')

    type(rtp_logi), target  ::  ignore_subunit_num = &
        rtp_logi(   value=.false.,  &
                    keyword='IGNORE_SUBUNIT_NUM',    &
                    description='Ignore the subunit_num value of input waypoints')

    type(rtp_logi), target  ::  ctf_correction  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='CTF_CORRECTION',           &
                    description='Whether to correct boxed images for the CTF')


    type(rtp_logi), target  ::  ctf_flip_only  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='CTF_FLIP_ONLY',           &
                    description='Multiply images by the sign of the CTF. Else, actual multiplication is done.')


    type(rtp_logi), target  ::  write_ctf_stack  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='WRITE_CTF_STACK',           &
                    description='Whether to dump a stack of ctf functions to disk. One per image.')


    type(rtp_logi), target  ::  out_of_plane_select  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='OUT_OF_PLANE_SELECT',           &
                    description='Whether to select for a range of out-of-plane tilts')


    type(rtp_real), target  ::  out_of_plane_min  =               &
        rtp_real(   value=0.0,                  &
                    keyword='OUT_OF_PLANE_MIN',           &
                    description='Minimum out-of-plane tilt allowed')


    type(rtp_real), target  ::  out_of_plane_max  =               &
        rtp_real(   value=5.0,                  &
                    keyword='OUT_OF_PLANE_MAX',           &
                    description='Maximum out-of-plane tilt allowed')


    type(rtp_real), target  ::  min_twist_per_subunit  =               &
        rtp_real(   value=-179.9999,                  &
                    keyword='min_twist_per_subunit',           &
                    description='In degrees. Waypoints or parts of filaments with twists per subunit lower than '//&
                                'this will not be included in the output stack')

    type(rtp_real), target  ::  max_twist_per_subunit  =               &
        rtp_real(   value=179.9999,                  &
                    keyword='max_twist_per_subunit',           &
                    description='In degrees. Waypoints or parts of filaments with twists per subunit greater than '//&
                                'this will not be included in the output stack')

    type(rtp_real), target  ::  min_rise_per_subunit  =               &
        rtp_real(   value=0.0,                  &
                    keyword='min_rise_per_subunit',           &
                    description='In Angstroms. Waypoints or parts of filaments with rises per subunit lower than '//&
                                'this will not be included in the output stack')

    type(rtp_real), target  ::  max_rise_per_subunit  =               &
        rtp_real(   value=huge(1.0)*0.0001,                  &
                    keyword='max_rise_per_subunit',           &
                    description='In Angstroms. Waypoints or parts of filaments with rises per subunit greater than '//&
                                'this will not be included in the output stack')



    type(rtp_real), target  ::  min_score  =               &
        rtp_real(   value=-1.0,                  &
                    keyword='MIN_SCORE',           &
                    description='Filaments with scores lower than this threshold will not be included')


    type(rtp_logi), target  ::  apply_env_mask  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='APPLY_ENV_MASK',           &
                    description='Apply an envelope mask to each image, based on the input 3d reconstruction provided')

    type(rtp_logi), target  ::  apply_rect_mask = &
        rtp_logi(   value=.false.,   &
                    keyword='APPLY_RECT_MASK',  &
                    description='Apply a rectangular mask to suppress solvent background')


    type(rtp_logi), target  ::  extract_segments  =               &
        rtp_logi(   value=.true.,                  &
                    keyword='EXTRACT_SEGMENTS',           &
                    description='')


    type(rtp_logi), target  ::  extract_projections  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='EXTRACT_PROJECTIONS',           &
                    description='')


    type(rtp_logi), target  ::  rotate_particles  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='ROTATE_PARTICLES',           &
                    description='')

    type(rtp_logi), target  ::  square_output_images  =               &
        rtp_logi(   value=.true.,                  &
                    keyword='SQUARE_OUTPUT_IMAGES',           &
                    description='Generate square output images & spectra ')


    type(rtp_logi), target  ::  unbend_segments  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='UNBEND_SEGMENTS',           &
                    description='Whether the filament segments should be unbent')


    type(rtp_logi), target  ::  ampl_spectra  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='AMPL_SPECTRA',           &
                    description='Whether we need to calculate amplitude spectra')


    type(rtp_logi), target  ::  ampl_spectra_average  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='AMPL_SPECTRA_AVERAGE',           &
                    description='Whether we need to calculate average of power spectra')


    type(rtp_logi), target  ::  ignore_fal_wps  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='IGNORE_FAL_WPS',           &
                    description='If .TRUE., the first and last waypoints of each filament will be ignored')


    type(rtp_real), target  ::  twist_per_subunit  =               &
        rtp_real(   value=0.0,                  &
                    keyword='TWIST_PER_SUBUNIT',           &
                    description='')


    type(rtp_real), target  ::  rise_per_subunit  =               &
        rtp_real(   value=0.0,                  &
                    keyword='RISE_PER_SUBUNIT',           &
                    description='')


    type(rtp_intg), target  ::  symmetry_axial  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_AXIAL',           &
                    description='')


    type(rtp_intg), target  ::  symmetry_perp  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_PERP',           &
                    description='')


    type(rtp_logi), target  ::  wps_are_crossovers  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='WPS_ARE_CROSSOVERS',           &
                    description='')


    type(rtp_logi), target  ::  fal_wps_are_crossovers  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='FAL_WPS_ARE_CROSSOVERS',           &
                    description='')


    type(rtp_logi), target  ::  amyloid_beta  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='',           &
                    description='')


    type(rtp_real), target  ::  helix_max_radius  =               &
        rtp_real(   value=0.0,                  &
                    keyword='HELIX_MAX_RADIUS',           &
                    description='Maximum outer radius of the helix, in Angstroms')


    type(rtp_real), target  ::  asym_unit_mw  =               &
        rtp_real(   value=0.0,                  &
                    keyword='ASYM_UNIT_MW',           &
                    description='Expected molecular weight (Da) of the asymetric unit')


    type(rtp_intg), target  ::  num_helical_starts  =               &
        rtp_intg(   value=1,                  &
                    keyword='NUM_HELICAL_STARTS',           &
                    description='Number of helical starts. The helical parameters given above must describeone of'//    &
                                ' these starts. Not including symmetry (e.g. could be describing a 1-start helix '//    &
                                'with an additional 2-fold axial symmetry).')


    type(rtp_logi), target  ::  stack  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='stack',           &
                    description='The assembly is a stack rather than a true helix')

     type(rtp_logi), target  ::  fragmented_phi  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='FRAGMENTED_PHI',           &
                    description='Phi function is discontinuous, with breaks between waypoints')

    type(rtp_logi), target  ::  fragmented_subunit_num  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='FRAGMENTED_SUBUNIT_NUM',           &
                    description='Subunit_num function is discontinuous, with breaks between waypoints')

    type(rtp_logi), target  ::  constant_twist  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='CONSTANT_TWIST',           &
                    description='The twist per unit length is constant.')


    type(rtp_logi), target  ::  constant_rise_per_subunit  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='CONSTANT_RISE_PER_SUBUNIT',           &
                    description='The rise per subunit is constant. Note that non-cumulative axial disorder is not accounted for.')

    type(rtp_logi), target  ::  wps_as_sps  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='wps_as_sps',           &
                    description='Treat waypoints as independent single particles')


    type(rtp_char), target  ::  interpol_function  =               &
        rtp_char(   value='LINEAR',                  &
                    keyword='INTERPOL_FUNCTION',           &
                    description='Which interpolation function to use')


    type(rtp_char), target  ::  window_function  =               &
        rtp_char(   value='RECT',                  &
                    keyword='WINDOW_FUNCTION',           &
                    description='Which windowing function to use')


    type(rtp_intg), target  ::  window_width  =               &
        rtp_intg(   value=2,                  &
                    keyword='WINDOW_WIDTH',           &
                    description='Window width for interpolation (kernel support)')

    type(rtp_real), target  ::  step_size  =               &
        rtp_real(   value=0.0,                  &
                    keyword='STEP_SIZE',           &
                    description='Spacing between segments to be cut out (in Angstroms). Give 0.0 use segment length'//&
                                ' so there is no overlap between segments.')

contains

subroutine mode_stack_from_par_init_and_run
    implicit none

    ! Intialise the mode
    call stack_from_par%init(command='STACK_FROM_PAR',subroutine=mode_stack_from_par_slave,short_description='Add description here')

    ! Add RTPs to the mode
    call stack_from_par%add_rtp(par_filename)
    call stack_from_par%add_rtp(output_frealign_par_filename)
    call stack_from_par%add_rtp(output_frealix_par_filename)
    call stack_from_par%add_rtp(micrograph_filename_wild)
    call stack_from_par%add_rtp(number_of_frames_per_micrograph)
    call stack_from_par%add_rtp(apply_dose_filter)
    call stack_from_par%add_rtp(exposure_per_frame)
    call stack_from_par%add_rtp(ctftilt_files)
    call stack_from_par%add_rtp(ctffind_files)
    call stack_from_par%add_rtp(reconstruction_filename)
    call stack_from_par%add_rtp(stack_filename)
    call stack_from_par%add_rtp(ctf_stack_filename)
    call stack_from_par%add_rtp(proj_filename)
    call stack_from_par%add_rtp(ampl_stack_filename)
    call stack_from_par%add_rtp(ampl_ave_filename)
    call stack_from_par%add_rtp(segment_length)
    call stack_from_par%add_rtp(one_image_per_repeat)
    call stack_from_par%add_rtp(phi_angle)
    call stack_from_par%add_rtp(phi_period)
    call stack_from_par%add_rtp(pixel_size)
    call stack_from_par%add_rtp(magnification)
    call stack_from_par%add_rtp(acceleration_voltage)
    call stack_from_par%add_rtp(spherical_aberration)
    call stack_from_par%add_rtp(amplitude_contrast)
    call stack_from_par%add_rtp(center_images)
    call stack_from_par%add_rtp(ignore_subunit_num)
    call stack_from_par%add_rtp(ctf_correction)
    call stack_from_par%add_rtp(ctf_flip_only)
    call stack_from_par%add_rtp(write_ctf_stack)
    call stack_from_par%add_rtp(out_of_plane_select)
    call stack_from_par%add_rtp(out_of_plane_min)
    call stack_from_par%add_rtp(out_of_plane_max)
    call stack_from_par%add_rtp(min_score)
    call stack_from_par%add_rtp(apply_env_mask)
    call stack_from_par%add_rtp(apply_rect_mask)
    call stack_from_par%add_rtp(extract_segments)
    call stack_from_par%add_rtp(extract_projections)
    call stack_from_par%add_rtp(rotate_particles)
    call stack_from_par%add_rtp(square_output_images)
    call stack_from_par%add_rtp(unbend_segments)
    call stack_from_par%add_rtp(step_size)
    call stack_from_par%add_rtp(ampl_spectra)
    call stack_from_par%add_rtp(ampl_spectra_average)
    call stack_from_par%add_rtp(ignore_fal_wps)
    call stack_from_par%add_rtp(twist_per_subunit)
    call stack_from_par%add_rtp(rise_per_subunit)
    call stack_from_par%add_rtp(symmetry_axial)
    call stack_from_par%add_rtp(symmetry_perp)
    call stack_from_par%add_rtp(wps_as_sps)
    call stack_from_par%add_rtp(wps_are_crossovers)
    call stack_from_par%add_rtp(fal_wps_are_crossovers)
    call stack_from_par%add_rtp(amyloid_beta)
    call stack_from_par%add_rtp(helix_max_radius)
    call stack_from_par%add_rtp(asym_unit_mw)
    call stack_from_par%add_rtp(num_helical_starts)
    call stack_from_par%add_rtp(stack)
    call stack_from_par%add_rtp(fragmented_phi)
    call stack_from_par%add_rtp(fragmented_subunit_num)
    call stack_from_par%add_rtp(constant_twist)
    call stack_from_par%add_rtp(constant_rise_per_subunit)
    call stack_from_par%add_rtp(interpol_function)
    call stack_from_par%add_rtp(window_function)
    call stack_from_par%add_rtp(window_width)
	call stack_from_par%add_rtp(min_twist_per_subunit)
	call stack_from_par%add_rtp(max_twist_per_subunit)
    call stack_from_par%add_rtp(min_rise_per_subunit)
	call stack_from_par%add_rtp(max_rise_per_subunit)


    ! Run the mode
    call stack_from_par%run()
end subroutine mode_stack_from_par_init_and_run


subroutine mode_stack_from_par_slave
    use runtime_parameters
    use parsers
    use images
    use imagefiles
    use rotation_matrices
    use images_arithm
    use images_ctf
    use images_threed
    use filaments
    use frealix_parameters
    use helices
    use imagefiles
    use images_helices
    use masks
    use filaments_path
    use single_particles
    use frealign_parameters
    use Splines
    use interpolation, only : interpolation_scheme, interpolation_scheme_init
    use filaments_geometry, only : waypoints_psi_theta_smooth
    use coos, only : wp_print_info, waypoints_grow_array, waypoint_deallocate
    ! Private variables
    integer                             ::  num_of_wps              ! Number of images defined in parameter file
    type(image),            allocatable ::  current_micrograph_frames(:)   ! Will hold current micrograph in memory
    type(image)                         ::  box                     ! Will hold current box in memory
    integer                             ::  current_filament, j
    integer                             ::  current_wp
    logical                             ::  new_micrograph          ! Whether current image comes from a new micrograph
    type(image)                         ::  box_ampl                ! Individual box amplitude spectrum
    type(image)                         ::  ave_ampl                ! Average amplitude spectrum
    integer                             ::  loc_in_stack            ! Current location in output stack file
    integer                             ::  micrograph_in_memory    ! Number of the micrograph currently held in memory
    type(image)                         ::  rec                     ! Will hold the 3d reconstruction in memory
    type(image)                         ::  proj                    ! Will hold the current projection in memory
    type(image)                         ::  proj_mask, rec_mask
    type(filament),         allocatable ::  fils(:)
    type(filament),         allocatable ::  fils_out(:)
    logical                             ::  use_ctftilt
    type(helical_symmetry)              ::  helix_average
    type(imagefile)                     ::  film_file, rec_file
    type(imagefile)                     ::  segments_file, projections_file, ampl_ave_file, ampl_file
    real(kind=8),           allocatable ::  ts(:)
    type(interpolation_scheme)          ::  interpol_scheme, interpol_linear
    logical,                parameter   ::  debug   =   .false.
    type(waypoint)                      ::  wp
    type(waypoint),         allocatable ::  wps(:)
    type(waypoint),         allocatable ::  wps_current_fil(:)
    integer                             ::  box_size
    real                                ::  current_filament_length
    type(single_particle)               ::  sp_tmp
    type(single_particle),  allocatable ::  sps(:), sps_tmp(:)
    logical                             ::  sp_near_edge
    logical,                parameter   ::  skip_filaments_shorter_than_segment_length = .false.
    integer                             ::  current_frame

    !
    ! Some features not yet implemented
    !
    if (out_of_plane_select%value) then
        call terminate('Out-of-plane selection not yet implemented')
    endif
    if (min_score%value .gt. -1.) then
        call terminate('Score selection not yet implemented')
    endif
    if (ignore_fal_wps%value) then
        call terminate('FAL WPs exclusion not yet implemented')
    endif
    if (write_ctf_stack%value) then
        call terminate('CTF stack not implemented yet')
    endif
    if (unbend_segments%value .and. ctf_correction%value) then
        write(*,'(a)') '**error(mode_stack_from_par): ctf correction and unbending options are not compatible'
        call terminate('Fatal error in MODE_STACK_FROM_PAR')
    endif

    if (.not. square_output_images%value) then
        if (.not. rotate_particles%value) then
            write(*,'(a)') '**error(mode_stack_from_par): when not rotating particles, one must output square images'
            call terminate('Fatal error in MODE_STACK_FROM_PAR')
        endif
    endif


    !
    ! Are we given ctffind files or ctftilt files?
    !
    use_ctftilt = .false.
    if (string_equal(ctftilt_files%value,'none')) then
        if (string_equal(ctffind_files%value,'none')) then
            write(*,'(a)') '**error(mode_stack_from_par): neither ctffind nor ctftilt files were supplied'
            call terminate('Fatal error in MODE_STACK_FROM_PAR')
        else
            use_ctftilt = .false.
        endif
    else
        use_ctftilt = .true.
    endif
    ! Diagnostic
    if (.not. use_ctftilt) then
        print '(a)', '**info(mode_stack_from_par): No ctftilt files given, will use ctffind info'
    endif


    ! psf safety radius
    psf_radius_pixels = psf_radius / pixel_size%value

    ! The segment length should be at least twice the psf radius
    if (segment_length%value .lt. 2.0*psf_radius-1.0) then
        write(*,'(a,F0.3,a,F0.3,a)') '**warning(mode_stack_from_par): segment length (', segment_length%value, &
                                     ' A) is less than twice the PSF radius (', psf_radius, ' A)'
    endif

    ! Initialise the average helical parameters
    call helical_symmetry_init(helix_average,rise_per_subunit%value/pixel_size%value,twist_per_subunit%value,    &
                                            helix_max_radius%value/pixel_size%value, &
                                            symmetry_axial%value,symmetry_perp%value,  &
                                            num_starts=num_helical_starts%value, &
                                            stack=stack%value)

    ! Initialise the interpolation scheme
    call interpolation_scheme_init(interpol_scheme,interpol_function%value,window_function%value,window_width%value)
	call interpolation_scheme_init(interpol_linear,'LINEAR','RECT',2)

    ! Work out the box size
    box_size = nint(segment_length%value/pixel_size%value)
    if (is_odd(box_size)) box_size = box_size+1
    write(*,'(a,i0,a)') '**info(mode_stack_from_par): box size = ', box_size, ' pixels'

    !
    ! Read in the frealix parameter file and intialise an array of filament objects
    !
    write(*,'(/a)') '**info(mode_stack_from_par): reading input parameter file and initialising filaments'
    if (use_ctftilt) then
        call filaments_from_frealix_par(    fils,par_filename%value,micrograph_filename_wild%value, &
                                            pixel_size%value,helix_average,    &
                                            ctftilt_filename_wild=ctftilt_files%value,                &
                                            wps_as_sps=wps_as_sps%value, &
                                            wps_are_crossovers=wps_are_crossovers%value,    &
                                            fal_wps_are_crossovers=fal_wps_are_crossovers%value, &
                                            max_radius=helix_max_radius%value/pixel_size%value, &
                                            acceleration_voltage=acceleration_voltage%value,                                &
                                            spherical_aberration=spherical_aberration%value,                                &
                                            amplitude_contrast=amplitude_contrast%value,                                    &
                                            fragmented_phi=fragmented_phi%value, &
                                            fragmented_subunit_num=fragmented_subunit_num%value,    &
                                            constant_rise_per_subunit = constant_rise_per_subunit%value,                    &
                                            constant_twist_per_length = constant_twist%value, &
                                            number_of_frames=number_of_frames_per_micrograph%value, &
                                            exposure_per_frame=exposure_per_frame%value, &
                                            apply_dose_filter=apply_dose_filter%value)
    else
        call filaments_from_frealix_par(    fils,par_filename%value,micrograph_filename_wild%value, &
                                            pixel_size%value,helix_average,    &
                                            ctffind_filename_wild=ctffind_files%value,                &
                                            wps_as_sps=wps_as_sps%value, &
                                            wps_are_crossovers=wps_are_crossovers%value,    &
                                            fal_wps_are_crossovers=fal_wps_are_crossovers%value, &
                                            max_radius=helix_max_radius%value/pixel_size%value,  &
                                            acceleration_voltage=acceleration_voltage%value,                                &
                                            spherical_aberration=spherical_aberration%value,                                &
                                            amplitude_contrast=amplitude_contrast%value,                                    &
                                            fragmented_phi=fragmented_phi%value, &
                                            fragmented_subunit_num=fragmented_subunit_num%value,    &
                                            constant_rise_per_subunit = constant_rise_per_subunit%value,                    &
                                            constant_twist_per_length = constant_twist%value, &
                                            number_of_frames=number_of_frames_per_micrograph%value, &
                                            exposure_per_frame=exposure_per_frame%value, &
                                            apply_dose_filter=apply_dose_filter%value )
    endif
    write(*,'(a,i0,a/)') '**info(mode_stack_from_par): ', size(fils), ' filaments were initialised'



    !
    ! How many filaments & waypoints?
    !
    num_of_wps = 0
    do current_filament=1,size(fils)
        num_of_wps = num_of_wps + fils(current_filament)%nwp
    enddo
    write(*,'(a,i0,a,i0,a)') '**info(stack_from_par): ', size(fils), ' filaments were found in the file, for a total of ', &
                                num_of_wps, ' waypoints.'


    ! Array of single particles
    if (allocated(sps)) deallocate(sps)

    !
    ! Loop over the filaments
    !
    loc_in_stack = 0
    micrograph_in_memory = 0
    current_wp = 0
    do current_filament=1, size(fils)

        current_filament_length =    fils(current_filament)%t(fils(current_filament)%nwp)  &
                                    -fils(current_filament)%t(1)
        if ( skip_filaments_shorter_than_segment_length .and. &
            current_filament_length .lt. segment_length%value/pixel_size%value ) then
            write(*,'(a,i0,2(a,F0.3),a)') '**info(mode_stack_from_par): filament ', fils(current_filament)%id ,   &
                                    ' is shorter (', current_filament_length ,') than the segment size (', &
                                    segment_length%value/pixel_size%value ,'), skipping'
            cycle
        endif

        write(*,'(a,i0,a,i0,a,i0,a)') '**info(mode_stack_from_par): processing filament ', fils(current_filament)%id, &
                                        ' (', current_filament, ' of ', size(fils), ')'

        !
        if (allocated(wps_current_fil)) then
            do j=1,size(wps_current_fil)
                call waypoint_deallocate(wps_current_fil(j))
            enddo
            deallocate(wps_current_fil)
        endif
        if (.not. allocated(fils_out)) allocate(fils_out(size(fils)))

        ! Check whether we are dealing with a new micrograph
        if ( current_filament .gt. 1 ) then
            if (fils(current_filament)%film%GetID() .ne. micrograph_in_memory) then
                new_micrograph = .true.
            else
                new_micrograph = .false.
            endif
        else
            new_micrograph = .true.
        endif

        ! If needed, read micrograph in
        if ( new_micrograph ) then
            call film_file%init(fils(current_filament)%film%GetFilename())
            call film_file%PrintInfo()
            if (.not. allocated(current_micrograph_frames)) then
                allocate(current_micrograph_frames(number_of_frames_per_micrograph%value))
            endif
            do current_frame=1,number_of_frames_per_micrograph%value
                call current_micrograph_frames(current_frame)%ReadFromDisk(film_file, &
                                                        fils(current_filament)%film%GetFrameLocationInFile(current_frame))
            enddo
            call film_file%close()
            micrograph_in_memory = fils(current_filament)%film%GetID()
            ! Apply degridding to micrograph - doesn't work for complex correction yet!
            !call micrograph%correct_3d(interpol_scheme%interpolation_instr_function,interpol_scheme%windowing_instr_function,interpol_scheme%window_width)
        endif

        ! We will use the assumption later on that the filament has t(1) = 0.0
        if (fils(current_filament)%t(1) .ne. 0.0D0) then
            write(*,'(a)') '**error(mode_stack_from_par): first t value is not 0.0'
            call terminate('Fatal error in MODE_STACK_FROM_PAR')
        endif

        ! Get waypoints where we want to extract segments
        if (allocated(wps)) deallocate(wps)
        if (phi_angle%set_by_user) then
            if (step_size%value .gt. 0.0) then
                call terminate('Step size parameter not compatible with phi spec')
            endif
            write(*,'(a,F0.1,a)') '**info(mode_stack_from_par): will output one filament segment every ', phi_period%value, &
                                    ' degree rotation around the helical axis'
            ts = fils(current_filament)%twist_spline%FindRootsOfMultiple(dble(phi_period%value),dble(phi_angle%value))
            do j=1,size(ts)
                ! Check we are not too close to the ends of the filament
                if (    ts(j) + (segment_length%value/pixel_size%value/2.+1.) &
                                                        .gt. fils(current_filament)%t(fils(current_filament)%nwp) .or.    &
                        ts(j) - (segment_length%value/pixel_size%value/2.+1.) .lt. fils(current_filament)%t(1)) then
                    write(*,'(a,F0.5,a)') '**info(MODE_STACk_FROM_PAR): skipping t (', ts(j), &
                                            ') because it is too close to the ends of the filament'
                    cycle
                endif
                call add_waypoint_to_path(wps,fils(current_filament),ts(j))
            enddo
        else if (one_image_per_repeat%value) then
            write(*,'(a)') '**info(mode_stack_from_par): will output one image per helical repeat'
            call filament_get_path(fils(current_filament),wps,step=1.0D0,step_in_repeats=.true.)
        else if (step_size%set_by_user) then
            write(*,'(a,2(F0.1,a))') '**info(mode_stack_from_par): will output one filament segment every ', &
                                                step_size%value, ' Angstroms (', step_size%value/pixel_size%value, ' pixels)'
            call filament_get_path_pixel_steps( fils(current_filament),real(step_size%value/pixel_size%value,kind=8),wps,    &
                                                start_t  = dble(min(segment_length%value/pixel_size%value/2. + 1.0, &
                                                        fils(current_filament)%t(fils(current_filament)%nwp)*0.5-0.001)),   &
                                                finish_t = dble(max(fils(current_filament)%t(fils(current_filament)%nwp)    &
                                                            - segment_length%value/pixel_size%value/2. - 1.0, &
                                                         fils(current_filament)%t(fils(current_filament)%nwp)*0.5+0.001)),  &
                                                strict_start_finish_t=.true., &
                                                integer_subunit_num=.true. &
                                                )
        else
            write(*,'(a)') '**info(mode_stack_from_par): will output one filament segment for every input waypoint'
            allocate(wps(size(fils(current_filament)%wp)))
            wps = fils(current_filament)%wp
        endif


        if (allocated(wps)) then
            ! Ensure that psi and theta are as smooth as possible, even if it
            ! Means breaking the convention of -90<=psi<=90.
            ! This is especially helpful when using Frealign
            call waypoints_psi_theta_smooth(wps)


            if (debug) then
                write(*,'(a,F0.5,a,F0.5)') '**debug(mode_stack_from_par): this filament goes from ', fils(current_filament)%t(1), &
                                            ' to ', fils(current_filament)%t(fils(current_filament)%nwp)
            endif

            ! Loop over points
            do j=1,size(wps)

                wp = wps(j)


                ! Cycle if we are not within helical parameter bounds given by user
                if (        wp%helix%twist .lt. min_twist_per_subunit%value &
                    .or.    wp%helix%twist .gt. max_twist_per_subunit%value &
                    .or.    wp%helix%rise  .lt. min_rise_per_subunit%value / pixel_size%value &
                    .or.    wp%helix%rise  .gt. max_rise_per_subunit%value / pixel_size%value) then
                    if (debug) then
                        write(*,'(a,i0,a)') '**debug(mode_stack_from_par): skipping waypoint ', j, ' because of h params'
                        call helix_print_info(wp%helix)
                    endif
                    cycle
                endif

                call wp_to_sp(  wp, sp_tmp, ctf=fils(current_filament)%film%ctftilt%ctf,    &
                                sp_image=box, segment_length=box_size, &
                                micrograph_frames=current_micrograph_frames, &
                                pixel_size=pixel_size%value, &
                                apply_shift_to_image=center_images%value,    &
                                apply_ctf_to_image=ctf_correction%value,    &
                                ctf_flip_only=ctf_flip_only%value, &
                                rotate_sp_image=rotate_particles%value, &
                                unbend=unbend_segments%value, &
                                fil=fils(current_filament), &
                                ignore_subunit_num=ignore_subunit_num%value, &
                                apply_rect_mask=apply_rect_mask%value,  &
                                too_close_to_edge=sp_near_edge, &
                                square_image=square_output_images%value, &
                                apply_dose_filter=apply_dose_filter%value, &
                                electron_dose=fils(current_filament)%film%electron_dose, &
                                exposure_per_frame=exposure_per_frame%value )

                if(debug) write(*,'(a,2(i0,1x))') '**debug(mode_stack_from_par): segment image dimensions: ', &
                                box%logical_dimensions(1:2)

                if (sp_near_edge) then
                    !
                    write(*,'(a,i0,a)') '**info(mode_stack_from_par): skipping waypoint ', j, ' because it is too close to the edge'
                else
                    ! We'll add a new image to the output stack
                    loc_in_stack = loc_in_stack + 1

                    !
                    call waypoints_grow_array(wps_current_fil,increment=1)
                    wps_current_fil(size(wps_current_fil)) = wp


                    ! Report progress
                    write(*,'(a,i0,a,i0,a)') '**info(mode_stack_from_par): processing output location ', loc_in_stack, &
                                                ' (filament ', fils(current_filament)%id,')'
                    call wp_print_info(wp)

                    ! Grow the array of single particles so that it can accomodate this filament's waypoints/single particles
                    if (allocated(sps)) then
                        sps_tmp = sps
                        deallocate(sps)
                        allocate(sps(loc_in_stack))
                        sps(1:size(sps_tmp)) = sps_tmp
                    else
                        allocate(sps(1))
                    endif

                    ! If we're on the first segment, we'll need to open files to write to, prepare masks etc.
                    if (loc_in_stack .eq. 1) then
                        ! Open files
                        if (extract_segments%value) then
                            call segments_file%init(stack_filename%value,  &
                                                    dim_x=box%logical_dimensions(1), &
                                                    dim_y=box%logical_dimensions(2), &
                                                    dim_z=1,pixel_size=pixel_size%value, &
                                                    delete_if_already_exists=.true.)
                        endif
                        if (ampl_spectra%value) then
                            call ampl_file%init(ampl_stack_filename%value, &
                                                dim_x=box%logical_dimensions(1), &
                                                dim_y=box%logical_dimensions(2), &
                                                dim_z=1,pixel_size=pixel_size%value, &
                                                delete_if_already_exists=.true.)
                        endif
                        if (extract_projections%value) then
                            call projections_file%init(proj_filename%value,    &
                                                        dim_x=box%logical_dimensions(1), &
                                                        dim_y=box%logical_dimensions(2), &
                                                        dim_z=1,pixel_size=pixel_size%value, &
                                                        delete_if_already_exists=.true.)
                        endif
                        if (ampl_spectra_average%value) then
                            call ampl_ave_file%init(ampl_ave_filename%value,   &
                                                    dim_x=box%logical_dimensions(1), &
                                                    dim_y=box%logical_dimensions(2), &
                                                    dim_z=1,pixel_size=pixel_size%value, &
                                                    delete_if_already_exists=.true.)
                        endif
                        ! If we're going to make projections or calculate masks, we need to read
                        ! the 3d reconstruction in & keep its shifted ft in memory
                        if (extract_projections%value .or. apply_env_mask%value) then
                            call rec_file%init(reconstruction_filename%value)
                            call rec_file%PrintInfo()
                            call rec%ReadFromDisk(rec_file,read_volume_not_slice=.true.)
                            call rec_file%close()
                            write(*,'(a)') '**info(mode_stack_from_par): resizing 3d reconstruction to match segment width'
                            call rec%Resize([box%logical_dimensions(1),box%logical_dimensions(1),box%logical_dimensions(1)])
                            write(*,'(a)') '**info(mode_stack_from_par): applying helical symmetry to input 3d reconstruction'
                            call image_impose_helical_symmetry_new( rec,helix_average, interpol_scheme, &
                                                                    max_radius=helix_max_radius%value/pixel_size%value)
                            write(*,'(a)') '**info(mode_stack_from_par): applying mask to input 3d reconstruction'
                            call image_helix_mask( rec, helix_average,asym_unit_mw%value,pixel_size%value,helix_max_radius%value,&
                                                    hard_mask=rec_mask)
                        endif

                        !
                        if (extract_projections%value) then
                            proj%logical_dimensions = box%logical_dimensions
                            call proj%Allocate()
                            call image_project_long_helix(  rec,proj,helix_average,phi_angle%value,   &
                                                            interpol_scheme,    &
                                                            helix_max_radius%value/pixel_size%value)
                        endif

                        ! Prepare the envelope mask
                        if (apply_env_mask%value) then
                            proj_mask%logical_dimensions = box%logical_dimensions
                            call proj_mask%Allocate()
                            call image_project_long_helix(  rec_mask,proj_mask,helix_average,phi_angle%value,   &
                                                            interpol_linear,    &
                                                            helix_max_rad=helix_max_radius%value/pixel_size%value)

                            if (debug) call rec_mask%WriteToDisk('dbg_mask_3d.mrc')
                            if (debug) call proj_mask%WriteToDisk('dbg_mask_beforemask.mrc')
                            ! Binarise & soften the mask
                            call binarise(proj_mask,10.*proj_mask%logical_dimensions(1)*tiny(1.))
                            proj_mask%real_values(:,1:7,1) = 0.0
                            proj_mask%real_values(:,proj_mask%logical_dimensions(2)-6:proj_mask%logical_dimensions(2),1) = 0.0
                            call mask_add_cosine_falloff(proj_mask,5.0)
                            !
                            if (debug) call proj_mask%WriteToDisk('dbg_mask.mrc')
                        endif
                    endif ! End of test for first in stack

                    ! The film_num property of the single particle is used to keep track of the filament number
                    sp_tmp%film_num = fils(current_filament)%id

                    ! Add this single particle to the output array
                    sps(loc_in_stack) = sp_tmp

                    ! Apply envelope mask
                    if (apply_env_mask%value) then
                        call multiply_images(box,proj_mask,box)
                    endif

                    ! Accumulate average amplitude
                    if (ampl_spectra_average%value .or. ampl_spectra%value) then
                        call box%ComputeAmplitudeSpectrum(box_ampl)
                    endif
                    if (ampl_spectra_average%value) then
                        if (loc_in_stack .eq. 1) call ave_ampl%Allocate(mould=box_ampl)
                        ave_ampl%real_values = ave_ampl%real_values + box_ampl%real_values
                    endif

                    ! Write particle out
                    if (extract_segments%value) then
                        call box%WriteToDisk(segments_file,wanted_location=loc_in_stack)
                    endif
                    if (ampl_spectra%value) then
                        call box_ampl%WriteToDisk(ampl_file,wanted_location=loc_in_stack)
                    endif
                endif    ! End of test for sp near edge of micrograph
            enddo   ! End loop over waypoints in the filament

            if (allocated(wps_current_fil)) then
                if (size(wps_current_fil) .ge. 1) then
                    call filament_init(fils_out(current_filament),waypoints_in=wps_current_fil, &
                                       mould=fils(current_filament),wps_as_sps=.true.)
                endif
            endif

        endif ! End of test for allocated(wps)
    enddo ! End of loop over the filaments

    ! Write out filaments
    call filaments_to_frealix_par(fils_out,output_frealix_par_filename%value)

    ! Average amplitude spectra
    if (ampl_spectra_average%value) then
        ave_ampl%real_values = ave_ampl%real_values / real(loc_in_stack)
        call ave_ampl%WriteToDisk(ampl_ave_file)
        call ampl_ave_file%PrintInfo()
        call ampl_ave_file%close()
    endif

    ! Close output files
    if (extract_segments%value) then
        call segments_file%PrintInfo()
        call segments_file%close()
    endif
    if (extract_projections%value) then
        call projections_file%PrintInfo()
        call projections_file%close()
    endif
    if (ampl_spectra%value) then
        call ampl_file%PrintInfo()
        call ampl_file%close()
    endif

    if (.not. string_equal(output_frealign_par_filename%value,'none')) then
        call sp_to_frealign_par(sps,output_frealign_par_filename%value,magnification%value,pixel_size%value)
    endif

end subroutine mode_stack_from_par_slave

end module mode_stack_from_par
