!>  \brief  Mode to create a synthetic micrograph from a helical 3d model density
module mode_synth_micrograph
    ! Modules

    use modes
    use rtps

    implicit none
    type(mode) :: synth_micrograph

    type(rtp_char), target  ::  model_3d_filename  =               &
        rtp_char(   value='model3d.mrc',                  &
                    keyword='MODEL_3D_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  input_par_filename  =               &
        rtp_char(   value='waypoints.ali',                  &
                    keyword='INPUT_PAR_FILENAME',           &
                    description='')


    type(rtp_char), target  ::  ctftilt_filenames  =               &
        rtp_char(   value='none',                  &
                    keyword='CTFTILT_FILENAMES',           &
                    description='')

    type(rtp_char), target  ::  ctffind_filenames  =               &
        rtp_char(   value='none',                  &
                    keyword='CTFFIND_FILENAMES',           &
                    description='')


    type(rtp_char), target  ::  output_film_filenames  =               &
        rtp_char(   value='synth_micrograph_?????.mrc',                  &
                    keyword='OUTPUT_FILM_FILENAMES',           &
                    description='')

    type(rtp_intg), target  ::  output_film_size_x  =               &
        rtp_intg(   value=0,                  &
                    keyword='output_film_size_x',           &
                    description='')

    type(rtp_intg), target  ::  output_film_size_y  =               &
        rtp_intg(   value=0,                  &
                    keyword='output_film_size_y',           &
                    description='')



    type(rtp_char), target  ::  output_par_filename  =               &
        rtp_char(   value='waypoints_out.ali',                  &
                    keyword='OUTPUT_PAR_FILENAME',           &
                    description='')


    type(rtp_real), target  ::  twist_per_subunit  =               &
        rtp_real(   value=0.0,                  &
                    keyword='TWIST_PER_SUBUNIT',           &
                    description='In degrees')


    type(rtp_real), target  ::  rise_per_subunit  =               &
        rtp_real(   value=0.0,                  &
                    keyword='RISE_PER_SUBUNIT',           &
                    description='Angstroms')


    type(rtp_real), target  ::  helix_max_radius  =               &
        rtp_real(   value=0.0,                  &
                    keyword='HELIX_MAX_RADIUS',           &
                    description='Maximum radius (Angstroms)')


    type(rtp_intg), target  ::  symmetry_axial  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_AXIAL',           &
                    description='')


    type(rtp_intg), target  ::  symmetry_perp  =               &
        rtp_intg(   value=1,                  &
                    keyword='SYMMETRY_PERP',           &
                    description='')

    type(rtp_logi), target  ::  stack  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='stack',           &
                    description='The assembly is a stack rather than a true helix')

    type(rtp_real), target  ::  asym_unit_mw  =               &
        rtp_real(   value=0.0,                  &
                    keyword='ASYM_UNIT_MW',           &
                    description='')


    type(rtp_logi), target  ::  straighten_filament  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='STRAIGHTEN_FILAMENT',           &
                    description='Whether the filaments should be straightened')


    type(rtp_real), target  ::  pixel_size  =               &
        rtp_real(   value=1.0,                  &
                    keyword='PIXEL_SIZE',           &
                    description='Pixel size (a)')


    type(rtp_real), target  ::  target_resolution  =               &
        rtp_real(   value=10.,                  &
                    keyword='TARGET_RESOLUTION',           &
                    description='Target resolution (a)')


    type(rtp_real), target  ::  snr  =               &
        rtp_real(   value=0.5,                  &
                    keyword='SNR',           &
                    description='Desired SNR')

    type(rtp_logi), target  ::  individual_subunits  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='INDIVIDUAL_SUBUNITS',           &
                    description='Project individual helical subunits rather than segments')

    type(rtp_real), target  ::  twist_per_subunit_sigma  =               &
        rtp_real(   value=0.0,                  &
                    keyword='TWIST_PER_SUBUNIT_SIGMA',           &
                    description='Standard deviation of the twist per subunit, in degrees. Use this '//&
                                'to add cumulative rotational disorder to the helix.')

    type(rtp_logi), target  ::  real_space_proj  =               &
        rtp_logi(   value=.false.,                  &
                    keyword='REAL_SPACE_PROJ',           &
                    description='Use real-space projection')

    type(rtp_real), target  ::  force_wp_to_wp_distance  =               &
        rtp_real(   value=0.0,                  &
                    keyword='FORCE_WP_TO_WP_DISTANCE',           &
                    description='Each filament''s waypoints will be recalculated so that they are spaced'//&
                                ' by this distance (in Angstroms). If set to 0.0, the waypoints will be left as in input.')

    type(rtp_logi), target  ::  apply_ctf  =               &
        rtp_logi(   value=.true.,                  &
                    keyword='APPLY_CTF',           &
                    description='Apply ctf to the synthetic micrograph')

    type(rtp_logi), target  ::  normalise_model_3d  =               &
        rtp_logi(   value=.true.,                  &
                    keyword='NORMALISE_MODEL_3D',           &
                    description='Normalise the variance of the input model 3d to 1.0 (its average is unchanged)')

    type(rtp_real), target  ::  estimated_model_resolution  =               &
        rtp_real(   value=20.0,                  &
                    keyword='ESTIMATED_MODEL_RESOLUTION',           &
                    description='In Angstroms')


    type(rtp_logi), target  ::  overwrite_micrographs = &
        rtp_logi(   value=.true., &
                    keyword='overwrite_micrographs', &
                    description='Output micrographs should be overwritten')


contains

subroutine mode_synth_micrograph_init_and_run
    implicit none
    ! Intialise the mode
    call synth_micrograph%init(command='SYNTH_MICROGRAPH',subroutine=mode_synth_micrograph_slave,   &
                                short_description='Generate a simulated micrograph given a model 3d density')

    ! Add RTPs to the mode
    call synth_micrograph%add_rtp(model_3d_filename)
    call synth_micrograph%add_rtp(input_par_filename)
    call synth_micrograph%add_rtp(ctftilt_filenames)
    call synth_micrograph%add_rtp(ctffind_filenames)
    call synth_micrograph%add_rtp(output_film_filenames)
    call synth_micrograph%add_rtp(output_film_size_x)
    call synth_micrograph%add_rtp(output_film_size_y)
    call synth_micrograph%add_rtp(output_par_filename)
    call synth_micrograph%add_rtp(twist_per_subunit)
    call synth_micrograph%add_rtp(rise_per_subunit)
    call synth_micrograph%add_rtp(helix_max_radius)
    call synth_micrograph%add_rtp(symmetry_axial)
    call synth_micrograph%add_rtp(symmetry_perp)
    call synth_micrograph%add_rtp(stack)
    call synth_micrograph%add_rtp(asym_unit_mw)
    call synth_micrograph%add_rtp(straighten_filament)
    call synth_micrograph%add_rtp(pixel_size)
    call synth_micrograph%add_rtp(target_resolution)
    call synth_micrograph%add_rtp(snr)
    call synth_micrograph%add_rtp(individual_subunits)
    call synth_micrograph%add_rtp(twist_per_subunit_sigma)
    call synth_micrograph%add_rtp(real_space_proj)
    call synth_micrograph%add_rtp(force_wp_to_wp_distance)
    call synth_micrograph%add_rtp(apply_ctf)
    call synth_micrograph%add_rtp(normalise_model_3d)
    call synth_micrograph%add_rtp(estimated_model_resolution)
    call synth_micrograph%add_rtp(overwrite_micrographs)

    ! Run the mode
    call synth_micrograph%run()
end subroutine mode_synth_micrograph_init_and_run


subroutine mode_synth_micrograph_slave
    use ui, only : interval_print
    use filaments_model_update, only : filament_update_model
    use filaments_path, only : filament_get_path
    use filaments_geometry, only : filament_average_defocus
    use fmcs_evaluation, only : fmc_eval, fmc_model3d_prep
    use filaments_model_modify, only : filament_move_wps_to_nearest_subunit, filament_straighten
    use coos, only : waypoints_add_disorder, waypoints_print_info
    use parsers, only : filename_contains_qmarks
    !use filaments
    use frealix_parameters
    use fmcs
    use fmcs_base
    use images_comparison
    use parsers
    use runtime_parameters
    use images_helices
    use interpolation
    use images_threed
    use ctfs
    use images_ctf
    use masks
    use filaments_base
    use ImageFiles
    use ImageNoise
    use Globals
    use ContrastTransferFunctions
    implicit none
    ! Variables
    type(image), target                 ::  model3dvol
    type(image), target                 ::  my_micrograph(1)
    type(helical_symmetry)              ::  helix
    type(interpolation_scheme)          ::  interpol
    type(ctf_parameters)                ::  local_ctf
    logical,                parameter   ::  debug   =   .false.

    type(filament),     allocatable     ::  fils(:)
    integer                             ::  current_fil
    !integer,    parameter               ::  micrograph_margin = 600 ! in angstroms
    type(filament_model_comparison), target     ::  fmc
    logical,    parameter               ::  refine_params(7) = .false.
    type(waypoint), allocatable         ::  path(:)
    type(waypoint)                      ::  wp_phased
    integer                             ::  current_wp
    real(kind=8)                        ::  rcoo(3)
    integer                             ::  icoo(3)
    integer                             ::  proj_dim(3)
    real                                ::  rect_dim(2)
    real,           parameter           ::  soft_edge = default_falloff_pixels
    real                                ::  dummy
    type(imagefile)                     ::  film_file
    character(len=filename_len)         ::  film_filename
    real(kind=8)                        ::  proj_var
    real(kind=8)                        ::  z_shift
    integer(kind=8)                     ::  proj_var_counter
    !type(image)                         ::  box_ctf
    integer                             ::  current_micrograph
    integer                             ::  selected_fil
    !
    real                    ::  average_defocus
    type(ctf_parameters)    ::  average_ctf
    type(ContrastTransferFunction)  ::  average_ctf_modern
    integer                 ::  new_dim(3)
    logical,    parameter   ::  manual_padding = .false.
    logical,    parameter   ::  force_single_ctf_per_micrograph = .true.
    !logical,    parameter   ::  compute_score = .false.
    logical,    parameter   ::  attempt_noise_modelling = .false.
    logical,    parameter   ::  simulate_drift_during_exposure = .false.
    logical,    parameter   ::  use_new_repeat_routine = .true.
    logical,    parameter   ::  use_new_repeat_routine_with_distortions = .false.
    integer,    parameter   ::  projections_per_frame_for_drift_simulation = 10
    real,       parameter   ::  incident_dose_per_pixel  =  20.0    !   Average number of electron
    logical,    parameter   ::  segment_with_helical_subunits = .false.

    integer ::  current_frame
    integer ::  current_drift_projection
    integer ::  number_of_frames_to_draw
    integer ::  number_of_drift_simulation_projections
    real    ::  shift_during_current_frame(2)
    real    ::  current_projection_shift(2)
    integer ::  number_of_micrographs
    real    ::  seed

    character(len=10) :: current_time


    ! Start work

    ! Notify user if apply_ctf is false
    if (.not. apply_ctf%value) then
        write(*,'(a)') '**warning(mode_synth_micrograph): ctf will not be applied, debug flag activated'
    endif

    ! Set up interpolation
    call interpolation_scheme_init(interpol,'LINEAR','RECT',2)

    ! Convert the maximum radius to pixels
    !if (helix_max_radius%value .ne. 0.0) then
    !    helix_max_radius%value = helix_max_radius%value / pixel_size%value
    !endif

    ! Setup the helical symmetry operator
    call helical_symmetry_init( helix,rise_per_subunit%value/pixel_size%value,twist_per_subunit%value,    &
                                helix_max_radius%value/pixel_size%value, &
                                symmetry_axial%value,symmetry_perp%value, &
                                stack=stack%value)

    ! Setup the filament object
    call filaments_from_frealix_par(    fils,input_par_filename%value,output_film_filenames%value,      &
                                        pixel_size%value,helix,     &
                                        ctftilt_filename_wild=ctftilt_filenames%value,                  &
                                        ctffind_filename_wild=ctffind_filenames%value, &
                                        max_radius=helix_max_radius%value/pixel_size%value,    &
                                        films_exist=.false.,  &
                                        shift_fils_and_set_films_dim = &
                                        (output_film_size_x%value .eq. 0 .and. output_film_size_y%value .eq. 0), &
                                        new_film_dimensions=[output_film_size_x%value,output_film_size_y%value])


    ! Read in the 3d model
    call model3dvol%ReadFromDisk(model_3d_filename%value,read_volume_not_slice=.true.)

    ! Normalise the 3d model
    if (normalise_model_3d%value) then
        call model3dvol%NormaliseVariance()
    endif

    ! If the filename given for the film/micrograph doesn't contain question marks, then
    ! we will create a stack of micrographs, which will start off as blank (zeroes) and
    ! will later be filled with the filaments
    if (.not. filename_contains_qmarks(output_film_filenames%value)) then
        if (output_film_size_x%set_by_user .and. output_film_size_y%set_by_user) then
            ! Work out how many micrographs
            number_of_micrographs = 0
            do current_fil=1,size(fils)
                if (fils(current_fil)%film%GetId() .gt. number_of_micrographs) then
                    number_of_micrographs = fils(current_fil)%film%GetId()
                endif
            enddo
            write(*,'(a,i0,a)') '**info(mode_synth_micrograph): will create a stack of ',number_of_micrographs,' micrographs'
            ! Generate blank micrographs
            call film_file%Init(output_film_filenames%value,dim_x=output_film_size_x%value,dim_y=output_film_size_y%value, &
                                dim_z=number_of_micrographs,pixel_size=pixel_size%value, &
                                delete_if_already_exists=overwrite_micrographs%value)
            if (overwrite_micrographs%value) then
                call my_micrograph(1)%Allocate(dims=[output_film_size_x%value,output_film_size_y%value,1])
                my_micrograph(1) = 0.0e0
                do current_micrograph=1,number_of_micrographs
                    call my_micrograph(1)%WriteToDisk(film_file,wanted_location=current_micrograph)
                enddo
                call my_micrograph(1)%Deallocate()
            endif
        else
            write(*,'(a)') '**info(mode_synth_micrograph): you gave a filename without question marks'//&
                            ' as output micrograph filename, so I was going to try to generate a stack'// &
                            ' of micrographs, but you didn''t specify the desired x,y dimensions'//&
                            ' of the micrographs, so I cannot proceed'
            call this_program%TerminateWithFatalError('synth_micrograph','Please specify x,y dimensions of micrographs')
        endif
    endif



    if (individual_subunits%value) then
        write(*,'(a)') '**info(mode_synth_microgaph): individual helical subunits will be projected'
    else
        if (twist_per_subunit_sigma%value .ne. 0.0) then
            write(*,'(2a)') '**error(mode_synth_micrograph): cannot vary the twist per subunit unless',  &
                            ' individual subunits are projected'
            call terminate('Fatal error in MODE_SYNTH_MICROGRAPH')
        endif
    endif


    ! Initialise variables for noise stats
    proj_var = 0.0D0
    proj_var_counter = 0

    ! Loop over filaments
    do current_fil=1,size(fils)


        ! Print out filament info
        write(*,'(a)') '**debug(mode_synth_micrograph): shifted filament info'
        call fils(current_fil)%PrintInfo()

        ! Straighten if asked for it
        if (straighten_filament%value) then
            write(*,'(a)') '**info(mode_synth_micrograph): straightening filament'
            call filament_straighten(fils(current_fil))
        endif

        if (force_wp_to_wp_distance%value .gt. 0.0) then
            write(*,'(a,F0.3)') '**info(mode_sytnh_microgaph): resampling waypoints with spacing (A): ',    &
                                 force_wp_to_wp_distance%value
            call filament_resample_waypoints(fils(current_fil),real(force_wp_to_wp_distance%value/pixel_size%value,kind=8))
            !print *, '**debug(): resampling at every REPEAT'
            !call filament_resample_waypoints(fils(current_fil),1.0,step_in_repeats=.TRUE.)
        endif

        ! Move waypoints to nearest subunit
        write(*,'(a)') '**info(mode_sytnh_microgaph): moving waypoints to nearest subunits'
        call filament_move_wps_to_nearest_subunit(fils(current_fil))



        ! Prepare the synthetic micrograph image
        if (.not. filename_contains_qmarks(output_film_filenames%value)) then
            call my_micrograph(1)%ReadFromDisk(film_file,wanted_location=fils(current_fil)%film%GetID())
        else
            my_micrograph(1)%logical_dimensions(1:2) = fils(current_fil)%film%GetLogicalDimensions()
            my_micrograph(1)%logical_dimensions(3)   = 1
            write(*,'(a,i0,3a,2(i0,1x))')   '**info(mode_synth_micrograph): filament ', fils(current_fil)%id, &
                                            ' will be on micrograph ', trim(adjustl(fils(current_fil)%film%GetFilename())), &
                                            ', with dimensions: ', my_micrograph(1)%logical_dimensions(1:2)
            if (current_fil .eq. 1) call my_micrograph(1)%Allocate()
            my_micrograph(1) = 0.0e0
        endif

        ! Print out filament info
        call fils(current_fil)%PrintInfo()

        ! Initialise the fmc object
        call fmc_init(  fmc,fils(current_fil),my_micrograph,model3dvol,helix,    &
                        target_resolution%value,interpol,&
                        high_resolution_cutoff=max(7.0,target_resolution%value),&
                        asym_unit_mw=asym_unit_mw%value,    &
                        estimated_resolution_of_model_3d=estimated_model_resolution%value, &
                        project_individual_subunits=individual_subunits%value, &
                        real_space_proj=real_space_proj%value)
        call fmc_print_info(fmc)

        ! Prepare the 3d model
        write(*,'(a)') '**info(mode_sytnh_microgaph): preparing 3d model'
        call fmc_model3d_prep(fmc)

        ! If we're doing real space projection, let's pad the model 3d in Fourier space
        if (fmc%real_space_proj .and. manual_padding) then
            write(*,'(a)') '**info(mode_synth_micrograph): padding in Fourier space'
            new_dim = fmc%model3dvol_private%logical_dimensions*2
            if (fmc%model3dvol_private%is_in_real_space) call fmc%model3dvol_private%ForwardFFT()
            call fmc%model3dvol_private%Resize(new_dim)
            if (.not. fmc%model3dvol_private%is_in_real_space) call fmc%model3dvol_private%BackwardFFT()
        endif

        ! Dump a file with all the subunits
        !if (debug) then
        !    path = filament_get_path(fmc%fil,1.0,step_in_repeats=.TRUE.)
        !    call waypoints_to_frealix_ali(path,'repeats.ali',pixel_size=pixel_size%value,fil_id=fmc%fil%id,film_num=fmc%fil%film%id)
        !endif

        ! Get a path along the filament at regular intervals
        dummy = fmc%step
        fmc%step = fmc%step
        if (individual_subunits%value) then
            call filament_get_path(fmc%fil,path,1.0D0,step_in_repeats=.true.)
        else
            call filament_get_path(fmc%fil,path,fmc%step,step_in_pixels=.true.)
        endif



        ! Perturb the path
        if (twist_per_subunit_sigma%value .gt. 0.0) then
            write(*,'(a,F0.3,a)')   '**info(mode_synth_micrograph: perturbing twist-per-subunit by std dev ', &
                                    twist_per_subunit_sigma%value, ' degrees'
            call waypoints_add_disorder(path,   &
                                        cumulative_disorder_std    =(/0.,0.,0.,twist_per_subunit_sigma%value,0.,0.,0.,0.,0./),   &
                                        non_cumulative_disorder_std=(/0.,0.,0.,0.,                           0.,0.,0.,0.,0./))
        endif

        write(*,'(a)') '**debug(mode_synth_micrograph): path waypoints follow'
        call waypoints_print_info(path)

        ! Dimensions for the projection
        proj_dim(1:2) = fmc%box_size
        proj_dim(3)   = 1


        ! If we are simulating drift during frames, we need to set the number of frames and simulated projections
        number_of_frames_to_draw = 1
        number_of_drift_simulation_projections = 1
        if (simulate_drift_during_exposure) then
            number_of_frames_to_draw = size(path(1)%local_subframe_shift,1)
            number_of_drift_simulation_projections = projections_per_frame_for_drift_simulation
        endif

        call date_and_time(time=current_time)
        write(*,'(2a)') 'Time before projections: ', current_time


        if (use_new_repeat_routine .and. individual_subunits%value) then
            if (use_new_repeat_routine_with_distortions) then
                call image_project_many_h_subunits_with_h_distortion &
                                                        (fmc%model3dvol_private,helix,my_micrograph(1),path, &
                                                         helix_max_radius%value/pixel_size%value, &
                                                         volume_oversampling=fmc%model3d_oversampling)
            else
                do current_wp=1,size(path)
                    path(current_wp)%coo(1:2) = path(current_wp)%coo(1:2) - my_micrograph(1)%physical_address_of_box_center(1:2)
                enddo
                call image_project_many_helical_subunits(fmc%model3dvol_private,helix,my_micrograph(1),path, &
                                                         helix_max_radius%value/pixel_size%value, &
                                                         volume_oversampling=fmc%model3d_oversampling, &
                                                         vol_shift_z=0.0e0)
            endif
        else
        ! Loop over the path waypoints
        do current_wp=1,size(path)

            ! If 0.0 occupancy, skip
            if (path(current_wp)%occupancy .lt. 1.0) then
                cycle
            endif

            ! Loop over frames
            do current_frame=1,number_of_frames_to_draw

                ! Estimate the drift for the current frame
                if (number_of_frames_to_draw .eq. 1) then
                    shift_during_current_frame = 0.0
                else
                    if (current_frame .eq. 1) then
                        shift_during_current_frame =  path(current_wp)%local_subframe_shift(:,current_frame+1) &
                                                    - path(current_wp)%local_subframe_shift(:,current_frame  )
                    else if (current_frame .eq. number_of_frames_to_draw) then
                        shift_during_current_frame =  path(current_wp)%local_subframe_shift(:,current_frame  ) &
                                                    - path(current_wp)%local_subframe_shift(:,current_frame-1)
                    else
                        shift_during_current_frame = (path(current_wp)%local_subframe_shift(:,current_frame+1) &
                                                    - path(current_wp)%local_subframe_shift(:,current_frame-1))*0.5
                    endif
                endif

                ! Loop over drift simulation frames
                do current_drift_projection=1,number_of_drift_simulation_projections

                    ! What shift will we apply to the current project to simulate drift
                    ! during the frames
                    if (simulate_drift_during_exposure) then
                        ! We assume constant movement through the frame
                        current_projection_shift = - shift_during_current_frame*0.5 &
                                                   + shift_during_current_frame &
                                                   *(current_drift_projection-1) &
                                                   /(number_of_drift_simulation_projections-1)
                    else
                        current_projection_shift = 0.0
                    endif

                    if (interval_print(current_wp,size(path))) then
                        write(*,'(a,i0,a,i0)') '**info(mode_synth_micrograph): on projection ', current_wp, ' of ', size(path)
                    endif

                    ! Do the phasing (because probably not integer subunit_num)
                    wp_phased = path(current_wp)
                    !call filament_apply_phasing_to_wp_old(fmc%fil,wp_phased,neg=.FALSE.)
                    ! Work out any changes due to the non-integer value of subunit_num
                    ! (we may have to change phi and z)
                    call filament_model_phasing_from_wp(fils(current_fil),wp_phased,model_z_shift=z_shift, &
                                                        local_h_sym=.true., &
                                                        model3d_oversampling=1) ! The oversampling will be taken into account by the projection routines later on.

                    ! Integer coordinates & remainders
                    icoo(1:2) = nint(wp_phased%coo(1:2))
                    icoo(3) = 1
                    rcoo = wp_phased%coo - icoo

                    ! Local ctf parameters
                    local_ctf = fmc%fil%film%ctftilt%ctf
                    call ctf_ang_to_pix(local_ctf,fmc%fil%film%GetPixelSize())
                    call ctf_at_z(local_ctf,wp_phased%coo(3))
                    call ctf_to_img(local_ctf,fmc%box_ctf(1))

                    ! wp_phased will now contain the information needed for projection_prepare
                    wp_phased%coo(1:2) = rcoo(1:2)+current_projection_shift(1:2)

                    ! When projecting individual subunits, we can't apply the ctf until all subunits have been projected
                    if (debug) write(*,'(a,i0)') '**debug(mode_synth_micrograph): oversampling ', fmc%model3d_oversampling
                    call projection_prepare(fmc%model3dvol_private,fmc%proj(1),proj_dim,wp_phased, &
                                            fmc%box_ctf(1),helix_max_radius%value/pixel_size%value, &
                                            pixel_size=pixel_size%value, &
                                            vol_shift_z = z_shift, &
                                            interpol=fmc%interpol,  &
                                            win=fmc%interp_win_3d(1), &
                                            ctf_squared=.false.,  &
                                            env_mask=fmc%mask_env(1),    &
                                            !mask_with_zeros=.true.,&
                                            apply_disc_mask=.false.,&
                                            real_space_proj=fmc%real_space_proj,&
                                            apply_ctf=          .not. individual_subunits%value &
                                                        .and.   .not. force_single_ctf_per_micrograph &
                                                        .and. apply_ctf%value, &
                                            single_helical_subunit_only = individual_subunits%value,&
                                            helix=helix, &
                                            helix_max_rad=helix_max_radius%value / pixel_size%value, &
                                            segment_with_helical_subunits=segment_with_helical_subunits, &
                                            volume_oversampling=fmc%model3d_oversampling, &
                                            tight_env_mask=.false.)
                    ! DEBUG
                    if (debug) call fmc%proj(1)%WriteToDisk('dbg_synth_proj_raw.mrc',wanted_location=current_wp)


                    ! If we were doing real-space projection, we need to undo the effect of the Fourier-space padding
                    if (fmc%real_space_proj .and. manual_padding) then
                        if (fmc%proj(1)%is_in_real_space) call fmc%proj(1)%ForwardFFT()
                        new_dim = fmc%model3dvol_private%logical_dimensions/2
                        call fmc%model3dvol_private%Resize(new_dim)
                        if (.not. fmc%proj(1)%is_in_real_space) call fmc%proj(1)%BackwardFFT()
                    endif

                    ! Apply rectangular mask (not necessary when projecting individual subunits)
                    if (.not. individual_subunits%value) then
                        if (.not. fmc%proj(1)%is_in_real_space) call fmc%proj(1)%BackwardFFT()
                        rect_dim = (/ min(real(fmc%step,kind=4),2.*helix_max_radius%value/pixel_size%value)+ &
                                    soft_edge,(helix_max_radius%value/pixel_size%value+soft_edge)*2./)
                        rect_dim = (/ real(fmc%step,kind=4)+soft_edge,(helix_max_radius%value/pixel_size%value+soft_edge)*2./)
                        call fmc%proj(1)%ApplySoftRectangularMask(rect_dim,real(wp_phased%euler%psi), &
                                            falloff=soft_edge,wipe_input=.false.,mask_with_average=.false.,&
                                            shift_x=real(rcoo(1)),shift_y=real(rcoo(2)))
                    endif

                    ! Keep track of the variance
                    proj_var = proj_var + fmc%proj(1)%GetVarianceOfValues()
                    proj_var_counter = proj_var_counter + 1

                    ! Add the projection to the micrograph
                    call fmc%proj(1)%AddInto(my_micrograph(1),icoo)

                enddo ! End of loop over frames to simulate drift during frames

            enddo ! End of loop over movie frames

        enddo! End of loop over path waypoints
        endif ! End of test for repeat projection && use new routine

        call date_and_time(time=current_time)
        write(*,'(2a)') 'Time after projections: ', current_time

        if (filename_contains_qmarks(output_film_filenames%value)) then
            ! Do we need to apply CTF?
            if ((individual_subunits%value .or. force_single_ctf_per_micrograph) .and. apply_ctf%value) then
                ! Apply the defocus
                average_defocus = filament_average_defocus(fils(current_fil))
                average_ctf = fmc%fil%film%ctftilt%ctf
                call ctf_at_z(average_ctf,real(average_defocus,kind=8),pixel_size=pixel_size%value)
                if (my_micrograph(1)%is_in_real_space) call my_micrograph(1)%ForwardFFT()
                call average_ctf_modern%Init(acceleration_voltage=wl_to_akv(average_ctf%wl*pixel_size%value), &
                                             spherical_aberration=average_ctf%cs*pixel_size%value*0.0000001, &
                                             amplitude_contrast=average_ctf%ampl_cont, &
                                             defocus_1=average_ctf%dfmid1*pixel_size%value*0.0001, &
                                             defocus_2=average_ctf%dfmid2*pixel_size%value*0.0001, &
                                             astigmatism_azimuth=average_ctf%angast * 180.0 / 3.1415, &
                                             lowest_frequency_for_fitting=0.0,highest_frequency_for_fitting=0.5/pixel_size%value, &
                                             astigmatism_tolerance=-1.0,pixel_size=pixel_size%value,additional_phase_shift=0.0)
                call my_micrograph(1)%ApplyCTF(average_ctf_modern,phase_flip_only=.false.,minimum_absolute_ctf_value=0.0)
                if (.not. my_micrograph(1)%is_in_real_space) call my_micrograph(1)%BackwardFFT()
            else
                my_micrograph(1)%real_values = my_micrograph(1)%real_values * (-1.0)
            endif

            ! Add noise to the micrograph
            if (attempt_noise_modelling) then
                !call micrograph%AddDetectionNoise(dose=incident_dose_per_pixel,counting=.false.)
                stop 'detection noise not implemented, sorry'
            else
                !if (debug)
                write(*,'(a,F0.3)') '**debug(mode_synth_micrograph): adding Gaussian noise, target snr = ', snr%value
                call AddGaussianNoise(my_micrograph(1),stdev_noise=sqrt(real(proj_var)/snr%value))
            endif
        endif


        ! If we're doing real space projection, let's crop the model 3d in Fourier space
        if (fmc%real_space_proj .and. manual_padding) then
            write(*,'(a)') '**info(mode_synth_micrograph): cropping in Fourier space'
            new_dim = fmc%model3dvol_private%logical_dimensions/2
            if (fmc%model3dvol_private%is_in_real_space) call fmc%model3dvol_private%ForwardFFT()
            call fmc%model3dvol_private%Resize(new_dim)
            if (.not. fmc%model3dvol_private%is_in_real_space) call fmc%model3dvol_private%BackwardFFT()
        endif


        ! Write out micrograph
        if (filename_contains_qmarks(output_film_filenames%value)) then
            call filename_from_wild(output_film_filenames%value,fmc%fil%film%GetID(),film_filename)
            call film_file%init(film_filename,pixel_size=pixel_size%value)
            call my_micrograph(1)%WriteToDisk(film_file)
            call film_file%PrintInfo()
            call film_file%Close()
        else
            call my_micrograph(1)%WriteToDisk(film_file,wanted_location=fils(current_fil)%film%GetID())
        endif

    enddo ! end of loop over filaments


    ! The average variance
    proj_var = proj_var / real(proj_var_counter)

    ! Now we've projected all filaments, we need to finish the micrographs
    if (.not. filename_contains_qmarks(output_film_filenames%value)) then
        do current_micrograph=1,film_file%GetStackSize()
            write(*,'(2(a,i0))') '**info(mode_synth_micrograph): finishing micrograph ', current_micrograph, &
                                    ' of ', film_file%GetStackSize()
            call my_micrograph(1)%ReadFromDisk(film_file,wanted_location=current_micrograph)
            if (.not. my_micrograph(1)%IsConstant()) then
                ! Do we need to apply CTF?
                if ((individual_subunits%value .or. force_single_ctf_per_micrograph) .and. apply_ctf%value) then
                    ! Let's find a filament from this micrograph so we can work out the defocus
                    selected_fil = 0
                    do current_fil=1,size(fils)
                        if (fils(current_fil)%film%GetID() .eq. current_micrograph) then
                            selected_fil = current_fil
                        endif
                    enddo
                    ! Apply the defocus
                    average_defocus = filament_average_defocus(fils(selected_fil))
                    average_ctf = fmc%fil%film%ctftilt%ctf
                    call ctf_at_z(average_ctf,real(average_defocus,kind=8),pixel_size=pixel_size%value)
                    if (my_micrograph(1)%is_in_real_space) call my_micrograph(1)%ForwardFFT()
                    call average_ctf_modern%Init(acceleration_voltage=wl_to_akv(average_ctf%wl*pixel_size%value), &
                                                 spherical_aberration=average_ctf%cs*pixel_size%value*0.0000001, &
                                                 amplitude_contrast=average_ctf%ampl_cont, &
                                                 defocus_1=average_ctf%dfmid1*pixel_size%value*0.0001, &
                                                 defocus_2=average_ctf%dfmid2*pixel_size%value*0.0001, &
                                                 astigmatism_azimuth=average_ctf%angast * 180.0 / 3.1415, &
                                                 lowest_frequency_for_fitting=0.0, &
                                                 highest_frequency_for_fitting=0.5/pixel_size%value, &
                                                 astigmatism_tolerance=-1.0,pixel_size=pixel_size%value,additional_phase_shift=0.0)
                    call my_micrograph(1)%ApplyCTF(average_ctf_modern,phase_flip_only=.false.,minimum_absolute_ctf_value=0.0)
                    if (.not. my_micrograph(1)%is_in_real_space) call my_micrograph(1)%BackwardFFT()
                else
                    !my_micrograph(1)%real_values = my_micrograph(1)%real_values * (-1.0)
                endif
                ! Add noise to the micrograph
                if (attempt_noise_modelling) then
                    !call micrograph%AddDetectionNoise(dose=incident_dose_per_pixel,counting=.false.)
                    stop 'detection noise not implemented, sorry'
                else
                    !if (debug)
                    write(*,'(a,F0.3)') '**debug(mode_synth_micrograph): adding Gaussian noise, target snr = ', snr%value
                    call random_number(seed)
                    call AddGaussianNoise(my_micrograph(1),stdev_noise=sqrt(real(proj_var)/snr%value), &
                                          seed=int(seed*10000.0)+current_micrograph)
                endif
                call film_file%PrintInfo()
                call my_micrograph(1)%WriteToDisk(film_file,wanted_location=current_micrograph)
                call film_file%PrintInfo()
            endif ! end of test for blank micrograph
        enddo ! end of loop over micrographs
    endif ! end of test for ?? in output film filename


    ! Write out parameter file
    call filaments_to_frealix_par(fils,output_par_filename%value)

    ! Debug: write out every single subunit
    !path = filament_get_path(fmc%fil,step=1.0,step_in_repeats=.TRUE.)

    !write(*,'(a)') '**debug(mode_synth_micrograph): subunit path waypoints follow'
    !call waypoints_print_info(path)

    call film_file%close()

end subroutine mode_synth_micrograph_slave

end module mode_synth_micrograph
