#!/bin/tcsh


if ($#argv != 1 ) then
echo "No arguments supplied...exiting"
exit 1
endif

if (! -f $argv[1]) then
echo File does not exist: $argv[1]
exit 1
endif

if (! -x $argv[1]) then
echo File is not executable: $argv[1]
exit 1
endif

#set pdb_file = 2OM3.pdb1
set log_file = test.log
set frealix_exec = $1
set plot_fsc_exec = ${1:h}/../scripts/plot_fsc.rb
set starting_model = ../ab_hextend/scratch/Ab42_model_10units_h.mrc
set starting_ali = waypoints_2.ali

#
set h_rise = 4.8
set h_twist = 0.77
set h_rad = 60
set new_dim = 114
set straighten = F
set pixel_size = 1.163
set asym_mw = 3499.09
set target_res = 3.0

# Convert to absolute paths
set starting_model = `readlink -f ${starting_model}`


rm -f $log_file
test -d scratch || mkdir -v scratch
cd scratch
rm -f $log_file

set starting_ali = `readlink -f ${starting_ali}` 

cat <<eof > waypoints_2.ali
!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Angstr   par_vers. 4
    34      1      2   11119.998    6936.677   00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34      5      2   10629.840    6305.115  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     10      2   10128.577    5507.739  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     15      2    9432.905    4706.045  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     20      2    8837.683    3903.710  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     26      2    8198.057    3051.293  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
eof

echo -n `date` "Padding reference model..."
cat > tmp.flx <<eof
FREALIX_MODE PAD
INPUT_FILENAME $starting_model
OUTPUT_FILENAME ${starting_model:t:r}_${new_dim}.${starting_model:e}
NEW_DIM ${new_dim}
eof
test -f ${starting_model:t:r}_${new_dim}.${starting_model:e} || $frealix_exec tmp.flx >>& $log_file
echo "done"

echo -n "Hextending reference..."
cat > tmp.flx <<eof
FREALIX_MODE HEXTEND
INPUT_FILENAME ${starting_model:t:r}_${new_dim}.${starting_model:e}
OUTPUT_FILENAME ${starting_model:t:r}_${new_dim}_h.${starting_model:e}
TWIST_PER_SUBUNIT $h_twist
RISE_PER_SUBUNIT $h_rise
PIXEL_SIZE $pixel_size
HELIX_MAX_RADIUS $h_rad
NEW_ALGORITHM T
SYMMETRY_AXIAL 2
eof
test -f ${starting_model:t:r}_${new_dim}_h.${starting_model:e} || $frealix_exec tmp.flx >>& $log_file
echo "done"


echo -n "Masking reference..."
cat > hmask.flx <<eof
FREALIX_MODE HMASK
INPUT_FILENAME ${starting_model:t:r}_${new_dim}_h.${starting_model:e}
OUTPUT_FILENAME  ${starting_model:t:r}_${new_dim}_hm.${starting_model:e}
TWIST_PER_SUBUNIT $h_twist
RISE_PER_SUBUNIT $h_rise
PIXEL_SIZE $pixel_size
HELIX_MAX_RADIUS $h_rad
ASYM_UNIT_MW $asym_mw
MASK_RESOLUTION 5.0
stack t
eof
test -f ${starting_model:t:r}_${new_dim}_hm.${starting_model:e} || $frealix_exec hmask.flx >>& $log_file
echo " done."

echo -n `date` Generating synthetic micrograph...
cat > synth.flx <<eof
FREALIX_MODE		SYNTH_MICROGRAPH
#test
MODEL_3D_FILENAME	$starting_model
INPUT_PAR_FILENAME	$starting_ali
OUTPUT_PAR_FILENAME	${starting_ali:t:r}_bent.${starting_ali:e}
CTFTILT_FILENAMES	../../ctf/A??_bin2_ctftilt.log
OUTPUT_FILM_FILENAMES	micrograph_????_bin4.mrc
STRAIGHTEN_FILAMENT     $straighten	
TWIST_PER_SUBUNIT	$h_twist
RISE_PER_SUBUNIT	$h_rise
HELIX_MAX_RADIUS	$h_rad
SYMMETRY_AXIAL		2
SYMMETRY_PERP		1
ASYM_UNIT_MW		$asym_mw
PIXEL_SIZE		$pixel_size
TARGET_RESOLUTION 	$target_res
SNR			10000.	#0.25
REAL_SPACE_PROJ         T
INDIVIDUAL_SUBUNITS     F
ESTIMATED_MODEL_RESOLUTION 5.0
eof
test -f micrograph_0002_bin4.mrc || $frealix_exec synth.flx >>& $log_file
echo " done."


echo -n `date` Refining and reconstructing using FF mode...
cat > refine.flx <<eof
FREALIX_MODE		FILAMENT_REFINE
MODEL_3D_FILENAME	$starting_model
INPUT_PAR_FILENAME	${starting_ali:t:r}_bent.${starting_ali:e}
OUTPUT_PAR_FILENAME	junk.ali
CTFTILT_FILES		../../ctf/A??_bin2_ctftilt.log
FILM_FILES		micrograph_????_bin4.mrc
TWIST_PER_SUBUNIT	$h_twist
RISE_PER_SUBUNIT	$h_rise
HELIX_MAX_RADIUS	$h_rad
SYMMETRY_AXIAL		2
SYMMETRY_PERP		1
ASYM_UNIT_MW		$asym_mw
PIXEL_SIZE		$pixel_size
SKIP_REFINEMENT		F
WPS_AS_SPS		F
CALC_3D_RECONSTRUCTION	T
FINALISE_3D_RECONSTRUCTION	T
OUTPUT_3D_FILENAME		3drec_bent.mrc
MATCHING_PROJECTIONS_FILES       match_proj_????.mrc
WRITE_MATCHING_PROJECTIONS       F 
PSF_RADIUS		200
TARGET_RESOLUTION	$target_res
CALC_FSC                F
eof
test -f 3drec_bent.mrc || $frealix_exec refine.flx >>& $log_file
echo " done."

echo -n `date` Computing FSC...
cat > tmp.flx <<eof
FREALIX_MODE CALC_FSC
INPUT_VOLUME_1 ${starting_model:t:r}_${new_dim}_hm.${starting_model:e}
INPUT_VOLUME_2 3drec_bent_hm.mrc
eof
$frealix_exec tmp.flx >& fsc.log
cat fsc.log >>& $log_file
$plot_fsc_exec fsc.log fsc.png >>& $log_file
echo " done."

echo -n `date` "Checking FSC... "
cat > check_fsc.rb << EOF
#!/usr/bin/env ruby

require_relative "../../../scripts/rubylix.rb"
require_relative "../../../scripts/FrealixOutput.rb"

# Initialise log file & find out the frealix mode that was run
log = FrealixOutput.new("fsc.log")

fsc_values = []
fsc_values = log.getFSCValues()

fsc_values.delete_if {|x| x > 0.85}

if (fsc_values.count > 9) then
  puts "FSC looks bad"
  exit 1
else
  puts "FSC looks good"
  exit 0
end
EOF

chmod +x check_fsc.rb
./check_fsc.rb >>& $log_file

if ($?) then
	echo "Test failed"
	exit 1
endif


egrep "sorry|Fatal" $log_file >& /dev/null

if ($?) then
  echo "Test passed"
  exit 0
else
  echo "Test failed"
  echo "Have a look at log files and scratch/fsc.png"
  exit 1
endif

cd ..
