#!/bin/tcsh

if ($#argv != 1 ) then
echo "No arguments supplied...exiting"
exit 1
endif

if (! -f $argv[1]) then
echo File does not exist: $argv[1]
exit 1
endif

if (! -x $argv[1]) then
echo File is not executable: $argv[1]
exit 1
endif

set frealix_exec = $1
set log_file = test.log

rm -f $log_file
test -d scratch || mkdir -v scratch
cd scratch
rm -f $log_file

# Set up the input waypoints file
cat <<eof > waypoints_121_test.ali
!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi    Score 	Refine?   par_vers. 3
   118      1     14    7685.104    4749.691  -34888.449       0.000     4721.443      -90.291       72.782    0.000       T
   118      2     14    7778.144    4414.748  -34890.410     347.681     -718.473      -90.355       76.172    0.000       T
   118      3     14    7824.664    4210.060  -34891.785     557.604    -4002.970      -90.394       78.219    0.000       T
eof


echo -n `date` "Running Frealix mode FILAMENT_REFINE..."

cat > test.flx << EOF
FREALIX_MODE                  	FILAMENT_REFINE
TARGET_RESOLUTION             	15.0
PIXEL_SIZE                    	4.652
WPS_AS_SPS                    	F
POLARITY_CHECK                	F
PHI_POLARITY_CHECK            	F
THETA_RIGID_THEN_FLEXIBLE     	F
SEARCH_FILAMENT_TILT          	F
ESTIMATE_PHI_ANGLES           	F
RIGID_X                       	F
RIGID_Y                       	F
RIGID_Z                       	F
RIGID_PHI                     	T
RIGID_THETA                   	T
RIGID_PSI                     	F
RIGID_TWIST                   	T
RIGID_RISE                    	T
REFINE_X                      	T
REFINE_Y                      	T
REFINE_Z                      	F
REFINE_PHI                    	T
REFINE_THETA                  	T
REFINE_PSI                    	T
SP_SEGMENT_LENGTH             	700.0
USE_DOWNHILL_SIMPLEX            T
DS_RANDOM_START               	T
NUM_MINIM_RUNS                	1
SEARCH_HALF_RANGE_PHI         	12.0
SEARCH_HALF_RANGE_THETA       	10.0
SEARCH_HALF_RANGE_PSI         	0
REAL_SPACE_PROJECTION         	F
# Filenames
FILM_FILES                    	../../scans/A??_bin4.mrc
MATCHING_PROJECTIONS_FILES    	matching_proj_????.mrc
MODEL_3D_FILENAME             	../starting_model.mrc
OUTPUT_3D_FILENAME            	3drec_121.mrc
FILAMENT_TRACE_FILES          	trace_????_???.box
CTFFIND_FILES                 	none
CTFTILT_FILES                 	../../ctf/A??_bin2_ctftilt.log
INPUT_BOX_FILES               	../../100625_tmv/casa_boxes_stripped/A??-6x_???_stripped_bin4.box
INPUT_PAR_FILENAME            	waypoints_121_test.ali
OUTPUT_PAR_FILENAME           	waypoints_121_out.ali
FILM_SELECTION                	0,2,6,8,12,14
# Helical parameters
RISE_PER_SUBUNIT               	1.408
TWIST_PER_SUBUNIT              	-22.03
HELIX_MAX_RADIUS              	100
ASYM_UNIT_MW                  	17589.66
# Output options
WRITE_FILAMENT_TRACES         	T
CALC_FSC                      	F
CALC_3D_RECONSTRUCTION        	T
FINALISE_3D_RECONSTRUCTION    	F
SINGLE_FILAMENT_RECONSTRUCTIONS	T
WRITE_MATCHING_PROJECTIONS    	T
EOF
test -f waypoints_121_out.ali || $frealix_exec test.flx  >>& $log_file
endif
echo "done."

egrep "sorry|Fatal" $log_file >& /dev/null

if ($?) then
  echo "Test passed"
  exit 0
else
  echo "Test failed"
  echo "Have a look at log files and scratch/fsc.png"
  exit 1
endif

cd ..
