#!/bin/tcsh

if ($#argv != 1 ) then
echo "No arguments supplied...exiting"
exit 1
endif

if (! -f $argv[1]) then
echo File does not exist: $argv[1]
exit 1
endif

if (! -x $argv[1]) then
echo File is not executable: $argv[1]
exit 1
endif

set frealix_exec = $1
setenv OMP_NUM_THREADS 1
set pdb_file = 2OM3.pdb1
set log_file = test.log
set frealix_exec = $1
set plot_fsc = ${1:h}/../scripts/plot_fsc.rb
set starting_model = ../hextend_new/scratch/2OM3_h.mrc

# Convert to absolute paths
set starting_model = `readlink -f ${starting_model}`

rm -f $log_file
test -d scratch || mkdir -v scratch
cd scratch
rm -f $log_file

echo -n "Padding reference... "
cat > pad_bin4.flx << EOF
FREALIX_MODE PAD
INPUT_FILENAME $starting_model
OUTPUT_FILENAME ${starting_model:t:r}_130.${starting_model:e}
NEW_DIM 130
EOF
$frealix_exec pad_bin4.flx >>& $log_file 
echo "done."

echo -n "Hextending reference... "
cat > hextend_bin4_old.flx << EOF
FREALIX_MODE HEXTEND
INPUT_FILENAME ${starting_model:t:r}_130.${starting_model:e}
OUTPUT_FILENAME ${starting_model:t:r}_130_hold.${starting_model:e}
TWIST_PER_SUBUNIT -22.03
RISE_PER_SUBUNIT 1.41	#as specified in PDB 2OM3.pdb
PIXEL_SIZE 4.652
HELIX_MAX_RADIUS 100
NEW_ALGORITHM F
EOF
${frealix_exec} hextend_bin4_old.flx >>& $log_file 

cat > hextend_bin4_new.flx << EOF
FREALIX_MODE HEXTEND
INPUT_FILENAME ${starting_model:t:r}_130.${starting_model:e}
OUTPUT_FILENAME ${starting_model:t:r}_130_hnew.${starting_model:e}
TWIST_PER_SUBUNIT -22.03
RISE_PER_SUBUNIT 1.41	#as specified in PDB 2OM3.pdb
PIXEL_SIZE 4.652
HELIX_MAX_RADIUS 100
NEW_ALGORITHM T
EOF
${frealix_exec} hextend_bin4_new.flx >>& $log_file 
echo "done."

foreach suffix ('old' 'new')
  echo -n "Cropping ($suffix)..."
  cat > crop_bin4_${suffix}.flx << EOF
FREALIX_MODE PAD
INPUT_FILENAME ${starting_model:t:r}_130_h${suffix}.${starting_model:e}
OUTPUT_FILENAME ${starting_model:t:r}_h${suffix}_48.${starting_model:e}
NEW_DIM 48
EOF
  ${frealix_exec} crop_bin4_${suffix}.flx >>& $log_file 
  echo "done."
end

foreach suffix ('old' 'new')  
  echo -n "Z masking reference ($suffix)... "
  cat > mask_bin4_${suffix}.flx << EOF
FREALIX_MODE MASK
INPUT_FILENAME ${starting_model:t:r}_h${suffix}_48.${starting_model:e}
OUTPUT_FILENAME  ${starting_model:t:r}_h${suffix}_48_m.${starting_model:e}
MASK_TYPE  Z_EDGES
MASK_RADIUS   5
EOF
  ${frealix_exec} mask_bin4_${suffix}.flx >>& $log_file 
  echo "done."
end

echo -n "Calculating FSC between 3Ds... "
cat > fsc_bin4.flx << EOF
FREALIX_MODE CALC_FSC
INPUT_VOLUME_1 ${starting_model:t:r}_hnew_48_m.${starting_model:e}
INPUT_VOLUME_2 ${starting_model:t:r}_hold_48_m.${starting_model:e}
EOF
${frealix_exec} fsc_bin4.flx >& fsc_bin4.log
cat fsc_bin4.log >>& $log_file 
# $plot_fsc fsc_bin4.log fsc_bin4.png > /dev/null #redirect standard error to null file
echo "done."

#echo "All done. Check fsc_bin4.png. The FSC in there should be very, very good."
#eog fsc_bin4.png

echo -n `date` "Checking FRC... "
cat > check_fsc.rb << EOF
#!/usr/bin/env ruby
require_relative "../../../scripts/rubylix.rb"
require_relative "../../../scripts/FrealixOutput.rb"

# Initialise log file & find out the frealix mode that was run
log = FrealixOutput.new("fsc_bin4.log")

fsc_values = []
fsc_values = log.getFSCValues()

fsc_values.delete_if {|x| x > 0.9}

if (fsc_values.count > 0) then
  puts "FSC looks bad"
  exit 1
else
  puts "FSC looks good"
  exit 0
end
EOF

chmod +x check_fsc.rb
./check_fsc.rb >>& $log_file

if ($?) then
	echo "Test failed"
	exit 1
endif


egrep "sorry|Fatal" $log_file >& /dev/null

if ($?) then
  echo "Test passed"
  exit 0
else
  echo "Test failed"
  echo "Have a look at log files and fsc_bin4.png"
  exit 1
endif

cd ..
