#!/bin/bash
#
# Run tests for frealix
#

#frealix_exec=$PWD/../build_intel/frealix
frealix_exec=$( which $1)
today=`date +%y%m%d`
start_dir=`pwd`


# First, let's run unit tests
echo -n `date` ": Unit tests... "
cd unit_tests/
$frealix_exec test.flx &> test_${today}.log
grep -E 'sorry|Arithmetic exception|Abort' test_${today}.log &> /dev/null
if [ $? -eq 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# Helical symmetrisation test: new algorithm
echo -n `date` ": helical symmetrisation test (new v pdb) ... "
cd hextend_new/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# Helical symmetrisation test: new algorithm
echo -n `date` ": helical symmetrisation test (abeta; new v pdb) ... "
cd ab_hextend/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# Helical symmetrisation test: new v old algorithm
echo -n `date` ": helical symmetrisation test (new v old) ... "
cd hextend_old_v_new/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# CTF, compared to EMAN2
echo -n `date` ": apply CTF test (Frealix v EMAN2) ... "
cd apply_ctf/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# FILAMENT_STARTUP
echo -n `date` ": filament startup test ... "
cd tmv_startup/
./run_test.sh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# Synthetic micrograph: Frealign reconstruction
echo -n `date` ": synthetic filament reconstruction using Frealign test ... "
cd tmv_synth_recon_fln/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# Scoring function test
echo -n `date` ": FF scoring function test (DS)... "
cd ff_scoring_function/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# Scoring function test (CG)
echo -n `date` ": FF scoring function test (CG)... "
cd ff_scoring_function_cg/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# FF synth & reconstruct test
echo -n `date` ": FF synth & reconstruction (bent TMV, no noise) test... "
cd tmv_synth_recon_ff/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# SP synth & reconstruct test
echo -n `date` ": SP synth & reconstruction (straight TMV, no noise) test... "
cd tmv_synth_recon_sp/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# Synthetic micrographs: repeats v segments
echo -n `date` ": synthetic filament test (segments v repeats) ... "
cd tmv_synth_seg_rep/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# FF synth, refine & reconstruct test
echo -n `date` ": FF synth, refine & reconstruction (straight TMV, no noise) test... "
cd tmv_synth_refine_ff/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# FF synth, refine & reconstruct test
echo -n `date` ": FF synth, refine & reconstruction (bent TMV, no noise) test... "
cd tmv_synth_refine_ff_bent/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# FF synth, refine & reconstruct test
echo -n `date` ": FF synth, refine & reconstruction (bent Ab, no noise) test... "
cd ab_synth_refine_ff_bent/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir


# FF synth, refine & reconstruct test (CG)
echo -n `date` ": FF synth, refine & reconstruction (bent TMV, no noise; CG minim) test... "
cd tmv_synth_refine_ff_bent_cg/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir


# SP synth, refine & reconstruct, easy (no noise, straight)
echo -n `date` ": SP synth, refine & reconstruction (straight TMV, no noise) test... "
cd tmv_synth_refine_sp_easy/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# SP synth, h refine & reconstruct, easy (no noise, straight)
echo -n `date` ": SP synth, h. refine & reconstruction (straight TMV, no noise, positive control) test... "
cd tmv_synth_h_refine_sp/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir

# SP synth, h refine & reconstruct, easy (no noise, straight)
echo -n `date` ": SP synth, h. refine & reconstruction (straight TMV, no noise, test) test... "
cd tmv_synth_h_refine_sp_2/
./run_test.csh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir


# TMV FF resampling, easy refinement
echo -n `date` ": FF synth, refine & reconstruction with resampling (straight TMV, no noise) test... "
cd tmv_synth_refine_ff_resample_easy
./run_test.sh $frealix_exec &> test_${today}.log
if [ $? -ne 0 ]; then echo 'FAILED!'; else echo 'passed.'; fi
cd $start_dir
