#!/bin/bash
#
#
if [ $# -ne 1 ]
then
	echo "Usage: $(basename $0) {frealix_executable}"
	exit 1
else
	if [[ ! -f $1 ]]
	then
		echo "File does not exist: $1"
		exit 1
	fi
	if [[ ! -x $1 ]]
	then
		echo "File is not executable: $1"
		exit 1
	fi
fi

#
# Parameters
#
pdb_file="2OM3.pdb1"
log_file="test.log"
frealix_exec=$1
starting_model=$(readlink -f ../hextend_new/scratch/2OM3_h.mrc)
starting_ali="waypoints_2.ali"

h_rad="105"
h_rise="1.39"
h_twist="-22.03"
new_dim="50"
straighten="T"

#
# 
#
rm -f $log_file
test -d scratch || mkdir scratch
cd scratch
starting_ali=$(readlink -f $starting_ali)
rm -f $log_file

#
#
#
cat > waypoints_2.ali <<eof
!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Angstr   par_vers. 4
    34      1      2   11119.998    6936.677   00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34      5      2   10629.840    6305.115  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     10      2   10128.577    5507.739  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     15      2    9432.905    4706.045  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     20      2    8837.683    3903.710  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     26      2    8198.057    3051.293  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
eof

echo -n `date` Generating synthetic micrograph...
output_ali=$(basename $starting_ali)
output_ali=${output_ali%.ali}_out.ali
cat > synth.flx <<eof
FREALIX_MODE		SYNTH_MICROGRAPH
MODEL_3D_FILENAME	$starting_model
INPUT_PAR_FILENAME	$starting_ali
OUTPUT_PAR_FILENAME	$output_ali
CTFTILT_FILENAMES	../../ctf/A??_bin2_ctftilt.log
OUTPUT_FILM_FILENAMES	micrograph_????_bin4.mrc
STRAIGHTEN_FILAMENT     $straighten	
TWIST_PER_SUBUNIT	$h_twist
RISE_PER_SUBUNIT	$h_rise
HELIX_MAX_RADIUS	$h_rad
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		17589.66
PIXEL_SIZE		    4.652
TARGET_RESOLUTION 	9.4
SNR			        1.0	#10000.0	#1.0 #0.5 #10000.	#0.25
REAL_SPACE_PROJ     T
INDIVIDUAL_SUBUNITS F
NORMALISE_MODEL_3D  T
APPLY_CTF			T
eof
test -f micrograph_0002_bin4.mrc || $frealix_exec synth.flx >> $log_file 2>&1
echo " done."

echo -n `date` Calling scoring function...
cat > refine.flx <<eof
FREALIX_MODE				FILAMENT_REFINE
MODEL_3D_FILENAME			$starting_model
INPUT_PAR_FILENAME			$output_ali
OUTPUT_PAR_FILENAME			junk.ali
CTFTILT_FILES				../../ctf/A??_bin2_ctftilt.log
FILM_FILES					micrograph_????_bin4.mrc
TWIST_PER_SUBUNIT			$h_twist
RISE_PER_SUBUNIT			$h_rise
HELIX_MAX_RADIUS			$h_rad
ASYM_UNIT_MW				17589.66
PIXEL_SIZE					4.652
SKIP_REFINEMENT				T
CALC_3D_RECONSTRUCTION		F
PSF_RADIUS					400
USE_CURVATURE_RESTRAINT		T
USE_TORSION_RESTRAINT		T
USE_STRETCH_RESTRAINT		T
PERSISTENCE_LENGTH			14000000.0  #1.4mm
TORSION_PERSISTENCE_LENGTH	10000.0
STRETCH_PERSISTENCE_LENGTH	100.0
eof
test -f junk.ali || $frealix_exec refine.flx >> $log_file 2>&1
echo " done."

