#!/bin/bash
#
#

# Get the frealix executable
frealix_exec=$1
if [ ! -x "$frealix_exec" ]; then
  echo "Need frealix executable as first argument"
  exit 1
fi

# Input variables
log_file="test.log"

# Set up & go to scratch
test -d scratch || mkdir scratch
cd scratch

# Set up the Boxer coordinates file
cat > A14_bin4_01.box <<eof
375     382     64      64      -3
332     469     64      64      -3
304     523     64      64      -3
eof

# Run FILAMENT_STARTUP
cat > startup.flx <<eof
FREALIX_MODE		FILAMENT_STARTUP
INPUT_BOX_FILES		A??_bin4_??.box
OUTPUT_PARAMETER_FILE	waypoints.ali
CTFTILT_FILES		../../ctf/A??_bin2_ctftilt.log
FILM_FILES              ../../scans/A??_bin4.mrc
FILM_SELECTION          14
PIXEL_SIZE              4.652
TWIST_PER_SUBUNIT	-22.03
RISE_PER_SUBUNIT	1.408
HELIX_MAX_RADIUS	100.0
eof
test -f waypoints.ali || $frealix_exec startup.flx &> $log_file 


# Go back to starting directory
cd ..

# Was the test successful?
if  (egrep -q "sorry|Fatal" scratch/$log_file) ; then
  echo "Test failed"
  exit 1
else
  echo "Test passed"
  exit 0
fi
