#!/bin/tcsh


if ($#argv != 1 ) then
echo "No arguments supplied...exiting"
exit 1
endif

if (! -f $argv[1]) then
echo File does not exist: $argv[1]
exit 1
endif

if (! -x $argv[1]) then
echo File is not executable: $argv[1]
exit 1
endif

set pdb_file = 2OM3.pdb1
set log_file = test.log
set frealix_exec = $1
set frealign_exec = /gusr/alr99/linux/apps/frealign_v8.10/bin/frealign_v8_fftw.exe
set frealign_exec = /groups/grigorieff/grigoriefflab/gware/frealign_v8.11/linux64/bin/frealign_v8_fftw.exe
set plot_fsc_exec = ${1:h}/../scripts/plot_fsc.rb
set starting_model = ../hextend_new/scratch/2OM3_h.mrc
set starting_ali = waypoints_2.ali
set fsc_thresh = 0.8
set fsc_tol = 10
set rec_with_flx = 0

set twist = -22.03
set twist_fln = 22.03
set rise = 1.408
set mw = 18000.0


# Convert to absolute paths
set starting_model = `readlink -f ${starting_model}`

rm -f $log_file
test -d scratch || mkdir -v scratch
cd scratch
rm -f $log_file

# Set up the input waypoints file
cat <<eof > $starting_ali
!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Angstr   par_vers. 4
    34      1      2   11119.998    6936.677   00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34      5      2   10629.840    6305.115  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     10      2   10128.577    5507.739  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     15      2    9432.905    4706.045  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     20      2    8837.683    3903.710  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     26      2    8198.057    3051.293  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
eof

set starting_ali = `readlink -f ${starting_ali}`

if (1) then
echo -n `date` "Padding reference..."
cat > tmp.flx <<eof
FREALIX_MODE PAD
INPUT_FILENAME $starting_model
OUTPUT_FILENAME ${starting_model:t:r}_130.${starting_model:e}
NEW_DIM 130
eof
$frealix_exec tmp.flx >>& $log_file
echo "done."

echo -n "Hextending reference..."
cat > tmp.flx <<eof
FREALIX_MODE HEXTEND
INPUT_FILENAME ${starting_model:t:r}_130.${starting_model:e}
OUTPUT_FILENAME ${starting_model:t:r}_130_h.${starting_model:e}
TWIST_PER_SUBUNIT $twist
RISE_PER_SUBUNIT $rise
PIXEL_SIZE 4.652
HELIX_MAX_RADIUS 100
NEW_ALGORITHM T
eof
$frealix_exec tmp.flx >>& $log_file
echo "done."

echo -n "Masking reference..."
cat > tmp.flx <<eof
FREALIX_MODE MASK
INPUT_FILENAME ${starting_model:t:r}_130_h.${starting_model:e}
OUTPUT_FILENAME  ${starting_model:t:r}_130_h_m.${starting_model:e}
MASK_TYPE  Z_EDGES
MASK_RADIUS   5
eof
$frealix_exec tmp.flx >>& $log_file
echo "done."

echo -n "Cropping..."
cat > tmp.flx <<eof
FREALIX_MODE PAD
INPUT_FILENAME ${starting_model:t:r}_130_h.${starting_model:e}
OUTPUT_FILENAME ${starting_model:t:r}_130_h_48.${starting_model:e}
NEW_DIM 48
eof
$frealix_exec tmp.flx >>& $log_file
echo "done."

echo -n "Z masking reference..."
cat > tmp.flx <<eof
FREALIX_MODE MASK
INPUT_FILENAME ${starting_model:t:r}_130_h_48.${starting_model:e}
OUTPUT_FILENAME  ${starting_model:t:r}_130_h_48_m.${starting_model:e}
MASK_TYPE  Z_EDGES
MASK_RADIUS   5
eof
$frealix_exec tmp.flx >>& $log_file
echo " done."

echo -n `date` Generating synthetic micrograph...
cat > synth.flx <<eof
FREALIX_MODE		SYNTH_MICROGRAPH
MODEL_3D_FILENAME	${starting_model:t:r}_130_h_m.${starting_model:e}
INPUT_PAR_FILENAME	$starting_ali
OUTPUT_PAR_FILENAME	${starting_ali:t:r}_straight.${starting_ali:e}
CTFTILT_FILENAMES	../../ctf/A??_bin2_ctftilt.log
OUTPUT_FILM_FILENAMES	micrograph_????_bin4.mrc
STRAIGHTEN_FILAMENT     T
TWIST_PER_SUBUNIT	$twist
RISE_PER_SUBUNIT	$rise
HELIX_MAX_RADIUS	100
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		$mw
PIXEL_SIZE		4.652
TARGET_RESOLUTION 	9.4
SNR			1000.0	#0.25
REAL_SPACE_PROJ         F
INDIVIDUAL_SUBUNITS     F
FORCE_WP_TO_WP_DISTANCE 280.0
eof
test -f micrograph_0002_bin4.mrc || $frealix_exec synth.flx >>& $log_file
echo " done."
endif

set num_wps = `wc -l ${starting_ali:t:r}_straight.${starting_ali:e} | awk '{print $1 - 1}'`
set num_wps_m = 0
set num_wps_mm = 0
@ num_wps_m = $num_wps - 2
@ num_wps_mm = $num_wps - 4

head -1 ${starting_ali:t:r}_straight.${starting_ali:e} > ${starting_ali:t:r}_straight_strip.${starting_ali:e}
cat ${starting_ali:t:r}_straight.${starting_ali:e} | tail -$num_wps_m | head -$num_wps_mm >> ${starting_ali:t:r}_straight_strip.${starting_ali:e}


echo -n `date` Converting from Frealix to Frealign...
cat > flx2fln.flx <<eof
FREALIX_MODE			STACK_FROM_PAR
INPUT_PAR_FILENAME		${starting_ali:t:r}_straight_strip.${starting_ali:e}	#repeats.ali	#${starting_ali:t:r}_straight_strip.${starting_ali:e}
OUTPUT_FREALIGN_PAR_FILENAME	waypoints.par
OUTPUT_STACK_FILENAME		stack.mrc
SEGMENT_LENGTH			600.0
CTF_CORRECTION			F
APPLY_RECT_MASK			F
EXTRACT_PROJECTIONS		F
TWIST_PER_SUBUNIT	$twist
RISE_PER_SUBUNIT	$rise 
HELIX_MAX_RADIUS	100
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		$mw
FILM_FILES	micrograph_????_bin4.mrc
PIXEL_SIZE			4.652
MAGNIFICATION			60190.0
CTFTILT_FILES			../../ctf/A??_bin2_ctftilt.log
eof
test -f waypoints.par || $frealix_exec flx2fln.flx >>& $log_file
echo " done."

echo -n `date` Reconstructing using Frealign...
cp ${starting_model:t:r}_130_h.${starting_model:e}  TMV.mrc 
set num_images  = `wc -l waypoints.par | awk '{print $1-1}'`
set target      = 100.0
set thresh      = 100.0
set pbc         = 100.0
set boff        = 90.0
set dang        = 2.5
set itmax       = 10
set mode        = 0
set fpart       = F
set dfstd       = 50.0
set rrec        = 6.0
set rref        = 10.0
set rlowref     = 500.0
set data_input  = TMV
set raw_images  = TMV_stack
set pixel_s     = 4.652
set dstep       = 28.0
set w_gh        = 0.07	# -1.0 to turn CTF off
set kV1         = 200.0
set cs          = 2.0
set xstd        = 0.0
set fmag        = F
set fdef        = F
set fastig      = F
set rbfact      = 0.0
set iewald      = 0
set asym        = H
set alpha       = $twist_fln
set rise        = $rise
set nsubunits   = 199
set nstarts     = 1
set stiffness   = 5.0
set FBEAUT      = F
set FFILT       = T
set ro          = 100.0
set ri          = 0.0 #20.0
cat <<eof > frealign_input.txt
M,${mode},${fmag},${fdef},${fastig},${fpart},${iewald},${FBEAUT},${FFILT},F,T,0,F,4           !CFORM,IFLAG,FMAG,FDEF,FASTIG,FPART,IEWALD,FBEAUT,FFILT,FBFACT,FMATCH,IFSC,FSTAT,IBLOW
${ro},${ri},${pixel_s},${w_gh},${xstd},${pbc},${boff},${dang},${itmax},10               !RO,RI,PSIZE,WGH,XSTD,PBC,BOFF,DANG,ITMAX,IPMAX
1 1 1 1 1                                                                               !MASK
1,${num_images}                                                                               !IFIRST,ILAST
${asym}                                                                                 !ASYM symmetry card
${alpha},${rise},${nsubunits},${nstarts},${stiffness}                                   !ALPHA,RISE,#SUBUNITS,#STARTS,STIFFNESS
1.0, ${dstep}, ${target}, ${thresh}, ${cs}, ${kV1}, 0., 0.                              !RELMAG,DSTEP,TARGET,THRESH,CS,AKV,TX,TY
${rrec},${rlowref},${rref},${dfstd},${rbfact}                                           !RREC,RMAX1,RMAX2,DFSTD,RBFACT
stack.mrc
${data_input}_match.mrc
waypoints.par
${data_input}.par
${data_input}.shft
0., 0., 0., 0., 0., 0., 0., 0.                                                          !terminator with RELMAG=-100.0 to skip 3D reconstruction
${data_input}.mrc
${data_input}_weights
${data_input}_qfactor
${data_input}_amplitudes
${data_input}_phasediffs
${data_input}_pointspread
eof
test -f ${data_input}_match.mrc || $frealign_exec < frealign_input.txt >>& $log_file
echo " done."

if ($rec_with_flx) then
echo -n `date` Reconstructing using WPS_AS_SPS mode...
cat > refine.flx <<eof
FREALIX_MODE		FILAMENT_REFINE
MODEL_3D_FILENAME	$starting_model
INPUT_PAR_FILENAME	${starting_ali:t:r}_straight_strip.${starting_ali:e}
OUTPUT_PAR_FILENAME	junk.ali
CTFTILT_FILES		../../ctf/A??_bin2_ctftilt.log
FILM_FILES		micrograph_????_bin4.mrc
TWIST_PER_SUBUNIT	$twist
RISE_PER_SUBUNIT 	$rise
HELIX_MAX_RADIUS	100
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		$mw
PIXEL_SIZE		4.652
SKIP_REFINEMENT		T
WPS_AS_SPS		T
SP_SEGMENT_LENGTH       700.0
CALC_3D_RECONSTRUCTION	T
FINALISE_3D_RECONSTRUCTION	T
OUTPUT_3D_FILENAME		3drec_straight.mrc
MATCHING_PROJECTIONS_FILES       match_proj_????.mrc
WRITE_MATCHING_PROJECTIONS       F 
PSF_RADIUS		200
TARGET_RESOLUTION	9.4
CALC_FSC                F
eof
test -f 3drec_straight.mrc ||$frealix_exec refine.flx >>& $log_file
echo " done."
endif

echo -n "Masking reference..."
cat > tmp.flx <<eof
FREALIX_MODE MASK
INPUT_FILENAME ${starting_model:t:r}_130_h.${starting_model:e}
OUTPUT_FILENAME  ${starting_model:t:r}_130_h_z.${starting_model:e}
MASK_TYPE  Z_EDGES
MASK_RADIUS   5
eof
test -f ${starting_model:t:r}_130_h_z.${starting_model:e} || $frealix_exec tmp.flx >>& $log_file
echo "done."

echo -n "Masking TMV..."
cat > tmp.flx <<eof
FREALIX_MODE MASK
INPUT_FILENAME TMV.mrc
OUTPUT_FILENAME  TMV_z.mrc
MASK_TYPE  Z_EDGES
MASK_RADIUS   5
eof
test -f TMV_z.mrc || $frealix_exec tmp.flx >>& $log_file
echo "done."

echo -n `date` "Computing FSC between model and frealign reconstruction..."
cat > tmp.flx <<eof
FREALIX_MODE CALC_FSC
INPUT_VOLUME_1 ${starting_model:t:r}_130_h_z.${starting_model:e}
INPUT_VOLUME_2 TMV_z.mrc
eof
$frealix_exec tmp.flx >& fsc.log
cat fsc.log >>& $log_file
$plot_fsc_exec fsc.log fsc.png >>& $log_file
echo " done."


echo -n `date` "Checking FSC (model v frealign)..."
cat > check_fsc.rb << EOF
#!/usr/bin/env ruby
require_relative "../../../scripts/rubylix.rb"
require_relative "../../../scripts/FrealixOutput.rb"

# Initialise log file & find out the frealix mode that was run
log = FrealixOutput.new("fsc.log")
fsc_values = []
fsc_values = log.getFSCValues()
fsc_values.delete_if {|x| x > $fsc_thresh}
if (fsc_values.count > $fsc_tol) then
  puts "FSC looks bad"
  exit 1
else
  puts "FSC looks good"
  exit 0
end
EOF

chmod +x check_fsc.rb
./check_fsc.rb >>& $log_file

if ($?) then
	echo "Test failed"
	exit 1
endif

egrep "sorry|Fatal" $log_file >& /dev/null

if ($?) then
  echo "Test passed"
  exit 0
else
  echo "Test failed"
  echo "Have a look at log files and fsc.png"
  exit 1
endif
