#!/bin/tcsh

if ($#argv != 1 ) then
echo "No arguments supplied...exiting"
exit 1
endif

if (! -f $argv[1]) then
echo File does not exist: $argv[1]
exit 1
endif

if (! -x $argv[1]) then
echo File is not executable: $argv[1]
exit 1
endif

set pdb_file = 2OM3.pdb1
set log_file = test.log
set frealix_exec = $1
set plot_fsc_exec = ${1:h}/../scripts/plot_fsc.rb
set starting_ali = waypoints_2.ali

#
set h_rad = 105
set h_rise = 1.408
set h_twist = -22.03
set new_dim = 200
set new_dim_large = 200
set straighten = T
set pixel_size = 1.163

rm -f $log_file
test -d scratch || mkdir -v scratch >>& $log_file
cd scratch
rm -f $log_file

# Convert to absolute paths
set starting_ali = `readlink -f ${starting_ali}`

cat <<eof > waypoints_2.ali
!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Angstr   par_vers. 4
    34      1      2   11119.998    6936.677   00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34      5      2   10629.840    6305.115  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     10      2   10128.577    5507.739  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     15      2    9432.905    4706.045  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     20      2    8837.683    3903.710  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     26      2    8198.057    3051.293  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
eof

echo -n `date` Downloading PDB if necessary... 
test -f ../../hextend_new/scratch/$pdb_file && cp ../../hextend_new/scratch/$pdb_file .
test -f ${pdb_file}.gz || test -f ${pdb_file} || wget http://www.rcsb.org/pdb/files/${pdb_file}.gz >>& $log_file
test -f $pdb_file || gzip -d ${pdb_file}.gz >>& $log_file
echo " done."

echo -n `date` Converting PDB to density volume... 
#test -f ${pdb_file:r}.hdf || e2pdb2em.py -A 1.163 -B 192 --center n --O ${pdb_file} ${pdb_file:r}.hdf >>& $log_file
#test -f ${pdb_file:r}.hdf || e2pdb2em.py -A 4.652 -B 48 --center n --O ${pdb_file} ${pdb_file:r}.hdf >>& $log_file
#test -f ${pdb_file:r}_z.hdf || e2proc3d.py --meanshrink 8 --trans=0,0,4 ${pdb_file:r}.hdf ${pdb_file:r}_z.hdf >>& $log_file
#test -f ${pdb_file:r}.mrc || bgex -verbose 7 -size 48,48,48 -origin 24,24,18 -sampling ${pixel_size} -resolution 5.0 -coordinates ${pdb_file} ${pdb_file:r}.mrc >>& $log_file
test -f ${pdb_file:r}.mrc || bgex -verbose 7 -size 200,200,200 -origin 100,100,80 -sampling ${pixel_size} -resolution 2.327 -coordinates ${pdb_file} ${pdb_file:r}.mrc >>& $log_file
echo " done."

echo -n `date` Hextending model...
#cat > tmp.flx <<eof
#FREALIX_MODE PAD
#INPUT_FILENAME ${pdb_file:r}.mrc
#OUTPUT_FILENAME ${pdb_file:r}_${new_dim_large}.mrc
#NEW_DIM $new_dim_large
#eof
#test -f  ${pdb_file:r}_${new_dim_large}.mrc || $frealix_exec tmp.flx >>& $log_file
cp -v ${pdb_file:r}.mrc  ${pdb_file:r}_${new_dim_large}.mrc
cat > tmp.flx <<eof
FREALIX_MODE HEXTEND
INPUT_FILENAME  ${pdb_file:r}_${new_dim_large}.mrc
OUTPUT_FILENAME  ${pdb_file:r}_${new_dim_large}_h.mrc
TWIST_PER_SUBUNIT $h_twist
RISE_PER_SUBUNIT $h_rise
PIXEL_SIZE $pixel_size
HELIX_MAX_RADIUS $h_rad
NEW_ALGORITHM T
eof
test -f ${pdb_file:r}_${new_dim_large}_h.mrc || $frealix_exec tmp.flx >>& $log_file
echo "done"

echo "Padding reference model..."
cat > tmp.flx <<eof
FREALIX_MODE PAD 
INPUT_FILENAME ${pdb_file:r}_${new_dim_large}_h.mrc
OUTPUT_FILENAME ${pdb_file:r}_${new_dim_large}_h_${new_dim}.mrc
NEW_DIM $new_dim
eof
test -f ${pdb_file:r}_${new_dim_large}_h_${new_dim}.mrc || $frealix_exec tmp.flx >>& $log_file
echo "done"

echo "Masking reference model..."
cat > tmp.flx <<eof
FREALIX_MODE HMASK
INPUT_FILENAME ${pdb_file:r}_${new_dim_large}_h_${new_dim}.mrc
OUTPUT_FILENAME  ${pdb_file:r}_${new_dim_large}_h_${new_dim}_m.mrc
TWIST_PER_SUBUNIT $h_twist
RISE_PER_SUBUNIT $h_rise
PIXEL_SIZE $pixel_size
HELIX_MAX_RADIUS $h_rad
ASYM_UNIT_MW 17589.66
MASK_RESOLUTION 10.0
eof
test -f ${pdb_file:r}_${new_dim_large}_h_${new_dim}_m.mrc || $frealix_exec tmp.flx >>& $log_file
echo " done."

echo -n `date` Generating synthetic micrograph...
cat > synth.flx <<eof
FREALIX_MODE		SYNTH_MICROGRAPH
#test
MODEL_3D_FILENAME	${pdb_file:r}_${new_dim_large}_h_${new_dim}_m.mrc
INPUT_PAR_FILENAME	$starting_ali
OUTPUT_PAR_FILENAME	${starting_ali:t:r}_straight.${starting_ali:e}
CTFTILT_FILENAMES	../../ctf/A??_bin2_ctftilt.log
OUTPUT_FILM_FILENAMES	micrograph_????_bin4.mrc
STRAIGHTEN_FILAMENT     $straighten	
TWIST_PER_SUBUNIT	$h_twist
RISE_PER_SUBUNIT	$h_rise
HELIX_MAX_RADIUS	$h_rad
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		17589.66
PIXEL_SIZE		$pixel_size
TARGET_RESOLUTION 	2.5
SNR			10000.	#0.25
REAL_SPACE_PROJ         T
INDIVIDUAL_SUBUNITS     F
eof
$frealix_exec synth.flx >>& $log_file
echo " done."


echo -n `date` Refining and reconstructing using FF mode...
cat > refine.flx <<eof
FREALIX_MODE		FILAMENT_REFINE
MODEL_3D_FILENAME	${pdb_file:r}_${new_dim_large}_h_${new_dim}_m.mrc
INPUT_PAR_FILENAME	${starting_ali:t:r}_straight.${starting_ali:e}
OUTPUT_PAR_FILENAME	junk.ali
CTFTILT_FILES		../../ctf/A??_bin2_ctftilt.log
FILM_FILES		micrograph_????_bin4.mrc
TWIST_PER_SUBUNIT	$h_twist
RISE_PER_SUBUNIT	$h_rise
HELIX_MAX_RADIUS	$h_rad
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		17589.66
PIXEL_SIZE		$pixel_size
SKIP_REFINEMENT		F
WPS_AS_SPS		F
CALC_3D_RECONSTRUCTION	T
FINALISE_3D_RECONSTRUCTION	T
OUTPUT_3D_FILENAME		3drec_straight.mrc
MATCHING_PROJECTIONS_FILES       match_proj_????.mrc
WRITE_MATCHING_PROJECTIONS       F 
PSF_RADIUS		200
TARGET_RESOLUTION	2.5
CALC_FSC                F
eof
$frealix_exec refine.flx >>& $log_file
test -f gmon.out && gprof $frealix_exec | gprof2dot.py | dot -Tpng -o refine_profile.png
echo " done."

echo -n `date` Computing FSC...
cat > tmp.flx <<eof
FREALIX_MODE CALC_FSC
INPUT_VOLUME_1 ${pdb_file:r}_${new_dim_large}_h_${new_dim}_m.mrc
INPUT_VOLUME_2 3drec_straight_hm.mrc
eof
$frealix_exec tmp.flx >& fsc.log
cat fsc.log >>& $log_file
$plot_fsc_exec fsc.log fsc.png >>& $log_file
echo " done."

echo -n `date` "Checking FSC... "
cat > check_fsc.rb << EOF
#!/usr/bin/env ruby

require_relative "../../../scripts/rubylix.rb"
require_relative "../../../scripts/FrealixOutput.rb"

# Initialise log file & find out the frealix mode that was run
log = FrealixOutput.new("fsc.log")

fsc_values = []
fsc_values = log.getFSCValues()

fsc_values.delete_if {|x| x > 0.9}

if (fsc_values.count > 3) then
  puts "FSC looks bad"
  exit 1
else
  puts "FSC looks good"
  exit 0
end
EOF

chmod +x check_fsc.rb
./check_fsc.rb >>& $log_file

if ($?) then
	echo "Test failed"
	exit 1
endif

egrep "sorry|Fatal" $log_file >& /dev/null

if ($?) then
  echo "Test passed"
  exit 0
else
  echo "Test failed"
  echo "Have a look at log files and scratch/fsc.png"
  exit 1
endif

cd ..
