#!/bin/tcsh


if ($#argv != 1 ) then
echo "No arguments supplied...exiting"
exit 1
endif

if (! -f $argv[1]) then
echo File does not exist: $argv[1]
exit 1
endif

if (! -x $argv[1]) then
echo File is not executable: $argv[1]
exit 1
endif

set pdb_file = 2OM3.pdb1
set log_file = test.log
set frealix_exec = $1
set plot_fsc_exec = ${1:h}/../scripts/plot_fsc.rb
set starting_model = ../hextend_new/scratch/2OM3_h.mrc
set starting_ali = waypoints_2.ali

set h_rad = 105
set h_rise = 1.39
set h_twist = -22.03
set new_dim = 50
set straighten = T

# Convert to absolute paths
set starting_model = `readlink -f ${starting_model}`

rm -f $log_file
test -d scratch || mkdir -v scratch
cd scratch
rm -f $log_file

set starting_ali = `readlink -f ${starting_ali}`

cat <<eof > waypoints_2.ali
!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Angstr   par_vers. 4
    34      1      2    3528.700    4475.356   00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34      5      2    3043.283    3834.532  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     10      2    2473.771    3082.684  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     15      2    1846.401    2244.498  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     20      2    1236.631    1446.146  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     26      2     598.190     594.841  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
eof

cat <<eof > waypoints_2_theta84.ali
!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Angstr   par_vers. 4
    34      1      2    3528.700    4475.356   00000.000   00000.000    00000.000       84.000    00000.000   00000.000    0.223       T
    34      5      2    3043.283    3834.532  -00000.000   00000.000    00000.000       84.000    00000.000   00000.000    0.223       T
    34     10      2    2473.771    3082.684  -00000.000   00000.000    00000.000       84.000    00000.000   00000.000    0.223       T
    34     15      2    1846.401    2244.498  -00000.000   00000.000    00000.000       84.000    00000.000   00000.000    0.223       T
    34     20      2    1236.631    1446.146  -00000.000   00000.000    00000.000       84.000    00000.000   00000.000    0.223       T
    34     26      2     598.190     594.841  -00000.000   00000.000    00000.000       84.000    00000.000   00000.000    0.223       T
eof


echo -n `date` Padding reference model...
cat > tmp.flx <<eof
FREALIX_MODE PAD
INPUT_FILENAME $starting_model
OUTPUT_FILENAME ${starting_model:t:r}_130.${starting_model:e}
NEW_DIM 130
eof
$frealix_exec tmp.flx >>& $log_file
echo "done"

echo -n Hextending reference...
cat > tmp.flx <<eof
FREALIX_MODE HEXTEND
INPUT_FILENAME ${starting_model:t:r}_130.${starting_model:e}
OUTPUT_FILENAME ${starting_model:t:r}_130_h.${starting_model:e}
TWIST_PER_SUBUNIT $h_twist
RISE_PER_SUBUNIT $h_rise
PIXEL_SIZE 4.652
HELIX_MAX_RADIUS $h_rad
NEW_ALGORITHM T
eof
$frealix_exec tmp.flx >>& $log_file
echo "done"

echo -n Padding helical reference...
cat > tmp.flx <<eof
FREALIX_MODE PAD
INPUT_FILENAME ${starting_model:t:r}_130_h.${starting_model:e}
OUTPUT_FILENAME ${starting_model:t:r}_130_h_${new_dim}.${starting_model:e}
NEW_DIM $new_dim
eof
$frealix_exec tmp.flx >>& $log_file
echo "done"

echo -n "Masking reference..."
cat > tmp.flx <<eof
FREALIX_MODE HMASK
INPUT_FILENAME ${starting_model:t:r}_130_h_${new_dim}.${starting_model:e}
OUTPUT_FILENAME  ${starting_model:t:r}_130_h_${new_dim}_m.${starting_model:e}
TWIST_PER_SUBUNIT $h_twist
RISE_PER_SUBUNIT $h_rise
PIXEL_SIZE 4.652
HELIX_MAX_RADIUS $h_rad
ASYM_UNIT_MW 17589.66
MASK_RESOLUTION 18.6
eof
$frealix_exec tmp.flx >>& $log_file
echo " done."

echo -n `date` Generating synthetic micrograph...
cat > synth.flx <<eof
FREALIX_MODE		SYNTH_MICROGRAPH
#test
MODEL_3D_FILENAME	$starting_model
INPUT_PAR_FILENAME	$starting_ali
OUTPUT_PAR_FILENAME	${starting_ali:t:r}_straight.${starting_ali:e}
CTFTILT_FILENAMES	../../ctf/A??_bin2_ctftilt.log
OUTPUT_FILM_FILENAMES	micrograph_????_bin4.mrc
STRAIGHTEN_FILAMENT    $straighten	
TWIST_PER_SUBUNIT	$h_twist
RISE_PER_SUBUNIT	$h_rise
HELIX_MAX_RADIUS	$h_rad
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		17589.66
PIXEL_SIZE		4.652
TARGET_RESOLUTION 	9.4
SNR			10000.	#0.25
REAL_SPACE_PROJ         T
INDIVIDUAL_SUBUNITS     F

eof
$frealix_exec synth.flx >>& $log_file
echo " done."


echo -n `date` Refining using WPS_AS_SPS mode...
cat > refine.flx <<eof
FREALIX_MODE		FILAMENT_REFINE
MODEL_3D_FILENAME	$starting_model
INPUT_PAR_FILENAME  waypoints_2_theta84.ali
OUTPUT_PAR_FILENAME	junk.ali
CTFTILT_FILES		../../ctf/A??_bin2_ctftilt.log
FILM_FILES		micrograph_????_bin4.mrc
TWIST_PER_SUBUNIT	$h_twist
RISE_PER_SUBUNIT	$h_rise
HELIX_MAX_RADIUS	$h_rad
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		17589.66
PIXEL_SIZE		4.652
SKIP_REFINEMENT		F
WPS_AS_SPS		T
CALC_3D_RECONSTRUCTION	T
FINALISE_3D_RECONSTRUCTION	T
OUTPUT_3D_FILENAME		3drec_straight.mrc
MATCHING_PROJECTIONS_FILES       match_proj_????.mrc
WRITE_MATCHING_PROJECTIONS       F 
PSF_RADIUS		200
TARGET_RESOLUTION	9.4
CALC_FSC                F
output_film_size_x 1128
output_film_size_y 1128
use_brute_force t
refine_x t
refine_y t
refine_phi t
refine_psi t
refine_theta t
refine_subunit_num f
search_half_range_x 10.0
search_half_range_y 10.0
search_half_range_phi 13.0
search_half_range_theta 10.0
search_half_range_psi 5.0
search_step_size_phi 2.0
search_step_size_theta 2.0
search_step_size_psi 2.0
eof
$frealix_exec refine.flx >>& $log_file
echo " done."

echo -n `date` Computing FSC...
cat > tmp.flx <<eof
FREALIX_MODE CALC_FSC
INPUT_VOLUME_1 ${starting_model:t:r}_130_h_${new_dim}_m.${starting_model:e}
INPUT_VOLUME_2 3drec_straight_hm.mrc
eof
$frealix_exec tmp.flx >& fsc.log
cat fsc.log >>& $log_file
$plot_fsc_exec fsc.log fsc.png >>& $log_file
echo " done."

echo -n `date` "Checking FSC... "
cat > check_fsc.rb << EOF
#!/usr/bin/env ruby
require_relative "../../../scripts/rubylix.rb"
require_relative "../../../scripts/FrealixOutput.rb"

# Initialise log file & find out the frealix mode that was run
log = FrealixOutput.new("fsc.log")

fsc_values = []
fsc_values = log.getFSCValues()

fsc_values.delete_if {|x| x > 0.9}

if (fsc_values.count > 1) then
  puts "FSC looks bad"
  exit 1
else
  puts "FSC looks good"
  exit 0
end
EOF

chmod +x check_fsc.rb
./check_fsc.rb >>& $log_file

if ($?) then
	echo "Test failed"
	exit 1
endif

egrep "sorry|Fatal" $log_file >& /dev/null

if ($?) then
  echo "Test passed"
  exit 0
else
  echo "Test failed"
  echo "Have a look at log files and scratch/fsc.png"
  exit 1
endif

cd ..
