#!/bin/tcsh
#
# Compare seg and rep synth micrographs


if ($#argv != 1 ) then
echo "No arguments supplied...exiting"
exit 1
endif

if (! -f $argv[1]) then
echo File does not exist: $argv[1]
exit 1
endif

if (! -x $argv[1]) then
echo File is not executable: $argv[1]
exit 1
endif

set pdb_file = 2OM3.pdb1
set frealix_exec = $1
set bin_factor = 4
set rise = 1.408
set twist = -22.03
set max_rad = 105
set real_space = F
set new_h_algorithm = T


#### Sets of pixel sizes, box sizes etc (bin1 refers to 1.163A/pxl) #####

### bin 0.5
set pixel_size = 0.5815
set box_size = 384 
set origin = "192,192,170"
set origin_x = 192
### bin 4
#set pixel_size = 4.652
#set box_size = 48 
#set origin = "24,24,18"
#set origin_x = 24


####


######## Shouldn't have to edit below this point ######

set max_rad_pix = `echo ${max_rad}/${pixel_size} | bc -l`
set resolution = `echo "${pixel_size}*2.0" | bc -l`
echo `date` "Resolution = " $resolution
set plot_fsc_exec = ${1:h}/../scripts/plot_fsc.rb
set log_file = test.log
set pixel_size_new = `echo "${pixel_size}*${bin_factor}" | bc -l`
set resolution_new = `echo "${pixel_size_new}*2.0" | bc -l`
set box_size_new = `echo "${box_size}/${bin_factor}" | bc`

rm -f $log_file
test -d scratch || mkdir -v scratch
cd scratch
rm -f $log_file

setenv OMP_NUM_THREADS 11

# Set up the input waypoints file
cat <<eof > waypoints.ali
!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Angstr   par_vers. 4
    34      1      2   11119.998    6936.677   00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34      5      2   10629.840    6305.115  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
    34     10      2   10128.577    5507.739  -00000.000   00000.000    00000.000    00000.000    00000.000   00000.000    0.223       T
eof


echo -n `date` "Downlading PDB if necessary... "
test -f ${pdb_file}.gz || wget http://www.rcsb.org/pdb/files/${pdb_file}.gz >>& $log_file
test -f $pdb_file || gzip -d ${pdb_file}.gz >>& $log_file
echo " done."
echo "Here is the exit status for the last command: $?" >>& $log_file

echo -n `date` "Converting PDB to density volume... "
#test -f ${pdb_file:r}.mrc || 
#bgex -verbose 7 -size ${box_size},${box_size},${box_size} -origin $origin -sampling ${pixel_size} -resolution $resolution -coordinates scratch/${pdb_file} scratch/${pdb_file:r}.mrc >>& $log_file
test -f ${pdb_file:r}.mrc || bgex -verbose 7 -size ${box_size},${box_size},${box_size} -origin $origin -sampling ${pixel_size} -resolution $resolution -coordinates ${pdb_file} ${pdb_file:r}.mrc >>& $log_file
#bgex -verbose 7 -size ${box_size},${box_size},${box_size} -origin $origin -sampling ${pixel_size} -resolution 0.5 -coordinates scratch/${pdb_file} scratch/${pdb_file:r}.mrc >>& $log_file
echo " done."

echo -n `date` "Hextending model... "
cat <<eof > tmp.txt
FREALIX_MODE HEXTEND
INPUT_FILENAME ${pdb_file:r}.mrc
OUTPUT_FILENAME ${pdb_file:r}_h.mrc
TWIST_PER_SUBUNIT $twist
RISE_PER_SUBUNIT $rise
PIXEL_SIZE ${pixel_size}
HELIX_MAX_RADIUS $max_rad
NEW_ALGORITHM $new_h_algorithm
WRITE_OUT_ASYM_UNIT T
ASYM_UNIT_FILENAME ${pdb_file:r}_asym.mrc
eof
test -f ${pdb_file:r}_h.mrc || ${frealix_exec} tmp.txt >>& $log_file
echo "done."

echo -n `date` "Binning model... "
cat <<eof > tmp.txt
FREALIX_MODE BIN
INPUT_FILENAME ${pdb_file:r}_h.mrc
OUTPUT_FILENAME ${pdb_file:r}_h_bin${bin_factor}.mrc
BINNING_FACTOR $bin_factor
UNBIN F
CORRECT_ORIGIN T
eof
test -f ${pdb_file:r}_h_bin${bin_factor}.mrc || ${frealix_exec} tmp.txt >>& $log_file
echo "done."

echo -n `date` "Generating synthetic micrograph (from binned model) (segments)... "
cat <<eof > tmp_seg.txt
FREALIX_MODE		SYNTH_MICROGRAPH
MODEL_3D_FILENAME	${pdb_file:r}_h_bin${bin_factor}.mrc
INPUT_PAR_FILENAME	waypoints.ali
OUTPUT_PAR_FILENAME	waypoints_straight_bin${bin_factor}.ali
CTFTILT_FILENAMES	../ctf/A??_bin2_ctftilt.log
OUTPUT_FILM_FILENAMES	../scratch/micrograph_????_from_bin${bin_factor}_model_seg.mrc
STRAIGHTEN_FILAMENT     T	
TWIST_PER_SUBUNIT	$twist
RISE_PER_SUBUNIT	$rise
HELIX_MAX_RADIUS	$max_rad
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		17589.66
PIXEL_SIZE		${pixel_size_new}
TARGET_RESOLUTION 	$resolution_new
SNR			10000.
REAL_SPACE_PROJ         $real_space
INDIVIDUAL_SUBUNITS	F
APPLY_CTF		F
eof
test -f  micrograph_0002_from_bin${bin_factor}_model_seg.mrc || ${frealix_exec} tmp_seg.txt >>& $log_file
echo "done."

echo -n `date` "Generating synthetic micrograph (from binned model) (repeats)... "
cat <<eof > tmp_rep.txt
FREALIX_MODE		SYNTH_MICROGRAPH
MODEL_3D_FILENAME	${pdb_file:r}_h_bin${bin_factor}.mrc
INPUT_PAR_FILENAME	waypoints.ali
OUTPUT_PAR_FILENAME	waypoints_straight_bin${bin_factor}.ali
CTFTILT_FILENAMES	../ctf/A??_bin2_ctftilt.log
OUTPUT_FILM_FILENAMES	../scratch/micrograph_????_from_bin${bin_factor}_model_rep.mrc
STRAIGHTEN_FILAMENT     T	
TWIST_PER_SUBUNIT	$twist
RISE_PER_SUBUNIT	$rise
HELIX_MAX_RADIUS	$max_rad
SYMMETRY_AXIAL		1
SYMMETRY_PERP		1
ASYM_UNIT_MW		17589.66
PIXEL_SIZE		${pixel_size_new}
TARGET_RESOLUTION 	$resolution_new
SNR			10000.
REAL_SPACE_PROJ         $real_space
INDIVIDUAL_SUBUNITS	T
APPLY_CTF		F
eof
test -f  micrograph_0002_from_bin${bin_factor}_model_rep.mrc || ${frealix_exec} tmp_rep.txt >>& $log_file
echo "done."

echo -n `date`  "Masking synthetic micrographs... "
#header micrograph_0002_from_bin${bin_factor}_model_seg.mrc
set micr_dim = `header micrograph_0002_from_bin${bin_factor}_model_seg.mrc | grep columns | awk '{print $7}'`
echo "Here is the exit status for the last command: $?"
echo "The micrograph dimension should be: $micr_dim"
echo "$micr_dim*0.23" | bc -l
set mask_rad = `echo "$micr_dim*0.23" | bc -l`
echo "Here is the exit status for the last command: $?"
echo "The masked radius should be: $mask_rad"

cat <<eof > tmp.txt
FREALIX_MODE MASK
INPUT_FILENAME micrograph_0002_from_bin${bin_factor}_model_seg.mrc
OUTPUT_FILENAME micrograph_0002_from_bin${bin_factor}_model_seg_mas.mrc
MASK_TYPE COSINE_DISC
MASK_RADIUS $mask_rad
MASK_FALLOFF 5.0
MASK_WITH_AVERAGE F
eof
test -f  micrograph_0002_from_bin${bin_factor}_model_seg_mas.mrc || ${frealix_exec} tmp.txt >>& $log_file

cat <<eof > tmp.txt
FREALIX_MODE MASK
INPUT_FILENAME micrograph_0002_from_bin${bin_factor}_model_rep.mrc
OUTPUT_FILENAME micrograph_0002_from_bin${bin_factor}_model_rep_mas.mrc
MASK_TYPE COSINE_DISC
MASK_RADIUS $mask_rad
MASK_FALLOFF 5.0
MASK_WITH_AVERAGE F
eof
test -f  micrograph_0002_from_bin${bin_factor}_model_rep_mas.mrc || ${frealix_exec} tmp.txt >>& $log_file
echo "done."

echo -n `date`  "Calculating FRC between synthetic micrographs... "
cat <<eof > tmp.txt
FREALIX_MODE CALC_FSC
INPUT_VOLUME_1 micrograph_0002_from_bin${bin_factor}_model_seg_mas.mrc
INPUT_VOLUME_2 micrograph_0002_from_bin${bin_factor}_model_rep_mas.mrc
eof
${frealix_exec} tmp.txt >& fsc.log
cat fsc.log >>& $log_file
echo "done."
${plot_fsc_exec} -p $pixel_size_new fsc.log fsc.png >>& $log_file

echo -n `date` "Checking FRC... "
cat > check_fsc.rb << EOF
#!/usr/bin/env ruby
require_relative "../../../scripts/rubylix.rb"
require_relative "../../../scripts/FrealixOutput.rb"

# Initialise log file & find out the frealix mode that was run
log = FrealixOutput.new("fsc.log")

fsc_values = []
fsc_values = log.getFSCValues()

fsc_values.delete_if {|x| x > 0.8}
p fsc_values.count
if (fsc_values.count > 47) then
  puts "FSC looks bad"
  exit 1
else
  puts "FSC looks ok"
  exit 0
end
EOF

chmod +x check_fsc.rb
./check_fsc.rb >>& $log_file

if ($?) then
	echo "Test failed"
	exit 1
endif

egrep "sorry|Fatal" $log_file >& /dev/null

if ($?) then
  echo "Test passed"
  exit 0
else
  echo "Test failed"
  echo "Have a look at log files and fsc_bin4.png"
  exit 1
endif

